/*-----------------------------------------------------------------------
 * File: BUILDDER.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* 
 * Gather up parts of values and concatentate into a larger BER object 
 */
uint32 
DER_ConstructItem(
    const uint8 *BerTag,              /* top-level tag to output */
    sint32 Count,               /* number of elements */
    BER_PARSED_ITEM_PTR Inputs, /* array of parsed items */
    uint8 *Octets)              /* target memory for the output */
{
    uint32 length, len, lengthContent;
    sint16 i;
    const uint8 *source;

    // compute length of content
    lengthContent = 0;
    for (i = 0; i < Count; i++) {
        lengthContent += BER_LengthOfTag(Inputs[i].Tag) +
            Inputs[i].ContentLength + 
            BER_LengthOfLength(Inputs[i].ContentLength);
    }
    length = BER_OutputTag(BerTag, Octets);
    Octets += length;
    len = BER_OutputLength(lengthContent, Octets);
    Octets += len;
    length += len;
    // concatentate the fields into the content
    for (i = 0; i < Count; i++) {
        /* output tag */
        len = BER_OutputTag(Inputs[i].Tag, Octets); 
        Octets += len;
        length += len;
        /* output length */
        len = BER_OutputLength(Inputs[i].ContentLength, Octets);
        Octets += len;
        length += len;
        /* output content */
        length += Inputs[i].ContentLength;
        source = Inputs[i].Content;
        for (len = Inputs[i].ContentLength; len > 0; len--)
            *Octets++ = *source++;
    }

    return length;
}
