/*-----------------------------------------------------------------------
 * File: GENERATE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"
/* 
 * Recursively check that all nodes are typed.
 */
static int typed(const DER_NODE *InputTree) 
{
    int i;
    if (!InputTree->Type)
        return 0;
    for (i = 0; i < InputTree->Count; i++) {
        if (InputTree->Child[i].X.Node && !typed(InputTree->Child[i].X.Node))
            return 0;
    }
    return 1;
}
/* 
 * Generate a DER-encoded, flat octet representation of a given parse tree.
 * The input parse tree is *not* modified (a temporary copy is modified).
 * A buffer containing the DER is returned if successful, NULL if unsuccessful.
 * The user is responsible for deallocating the buffer using the memory free function.
 */
uint8 *
DER_TypedGenerate(
    BER_HANDLE AppHandle,
    BER_MEMORY_FUNCS_PTR MemoryFunctions,
    const DER_NODE *InputTree)
{
    if (typed(InputTree))
        return  DER_Generate(AppHandle, MemoryFunctions, InputTree);
    else return 0;
}

