/*-----------------------------------------------------------------------
 * File: ACLTUP2C.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
#include "maf_include.h"
#include "cssm.h"
#include "cssmport.h"
#include "internal.h"
#include "spkiutil.h"

/***************************************************************************/
/* acltup2c                                                                */
/*                                                                         */
/* Take an ACL tuple, and form a packed canonical S-expression ACL from it.*/
/* Then store that in a CSSM_ENCODED_CERT structure                        */
/***************************************************************************/

CSSM_RETURN acltup2c( CSSM_TP_HANDLE TPHandle,
                      CSSM_TUPLE_PTR acltuple, 
                      CSSM_ENCODED_CERT *certptr)
{
    uint32 size = 10 ;            /* to count the size of the output */
                                /* init with room for (3:acl) & slop */
    uint8 *w ;                    /* memory walker */

    if (acltuple == NULL)
    {
        return ( CSSMERR_TP_INVALID_INPUT_POINTER ) ;
    }
    if (certptr == NULL)
    {
        return ( CSSMERR_TP_INVALID_OUTPUT_POINTER ) ;
    }

    /* init result */
    port_memset(certptr, 0, sizeof(CSSM_ENCODED_CERT));

    size += size_tuple(acltuple) ;

    w = certptr->CertBlob.Data = (uint8 *) App_Malloc( TPHandle, size ) ;

    if (!w) 
    {
        return ( CSSMERR_TP_MEMORY_ERROR ) ;
    }
    port_memcpy( w, "(3:acl", 6 ) ; 
    w += 6 ; /* init the output */

    pack_tuple( &w, acltuple ) ;
    *(w++) = ')' ;
    certptr->CertBlob.Length = w - certptr->CertBlob.Data ; /* note the length */
    certptr->CertType = CSSM_CERT_SPKI;
    certptr->CertEncoding = CSSM_CERT_ENCODING_SEXPR;
    return ( CSSM_OK ) ;
} /* acltup2c */


