/*-----------------------------------------------------------------------
 * File: TPTGTOCG.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "maf_include.h"
#include "cssm.h"
#include "internal.h"
#include "spkiutil.h"
/***************************************************************************/
/* Map a tuplegroup to a cert group (as a sequence).                       */
/***************************************************************************/

CSSM_RETURN CSSMTPI TP_TupleGroupToCertGroup(
                    CSSM_TP_HANDLE TPHandle,
                    CSSM_CL_HANDLE CLHandle,
                    const CSSM_TUPLEGROUP *TupleGroup,
                    CSSM_CERTGROUP_PTR *CertTemplates )
{
    CSSM_RETURN ret = CSSM_OK;
    uint32 i = 0;

    CSSM_CERTGROUP_PTR certgrp_ptr = NULL;

    /* Validate Inputs */
    if (!TupleGroup)
    {
        return CSSMERR_TP_INVALID_INPUT_POINTER;
    }
    if (!CertTemplates)
    {
        return CSSMERR_TP_INVALID_OUTPUT_POINTER;
    }

    /* We deem an empty tuple to be an error. */
    if (!TupleGroup->NumberOfTuples || !TupleGroup->Tuples)
    {
        return CSSMERR_TP_INVALID_TUPLE;
    }
    
    hashtab_init();

    certgrp_ptr = App_Calloc(TPHandle, 1, sizeof(CSSM_CERTGROUP));
    if (!certgrp_ptr)
    {
        return CSSMERR_TP_MEMORY_ERROR;
    }

    certgrp_ptr->CertType = CSSM_CERT_SPKI;
    certgrp_ptr->CertEncoding = CSSM_CERT_ENCODING_SEXPR;
    certgrp_ptr->NumCerts = TupleGroup->NumberOfTuples;
    certgrp_ptr->CertGroupType = CSSM_CERTGROUP_ENCODED_CERT;
    certgrp_ptr->GroupList.EncodedCertList = App_Calloc(TPHandle,
                                                    TupleGroup->NumberOfTuples,
                                                    sizeof(CSSM_ENCODED_CERT));
    if (!certgrp_ptr->GroupList.EncodedCertList)
    {
        App_Free(TPHandle, certgrp_ptr);
        return CSSMERR_TP_MEMORY_ERROR;
    }
    for(i=0;i<TupleGroup->NumberOfTuples;i++)
    {
        CSSM_TUPLE_PTR cur_tup_ptr = &(TupleGroup->Tuples[i]);
        CSSM_ENCODED_CERT_PTR cur_cert_ptr = 
            &(certgrp_ptr->GroupList.EncodedCertList[i]);

        if (!cur_tup_ptr->Issuer.Head || !cur_tup_ptr->Issuer.Tail)
        {
            /* Empty Issuer: cur_tup_ptr is an ACL*/
            ret = acltup2c(TPHandle, cur_tup_ptr, cur_cert_ptr);
        }
        else
        {
            /* Non-Empty Issuer: cur_tup_ptr is a cert */
            ret = certup2c(TPHandle, cur_tup_ptr, cur_cert_ptr);
        }
        if (CSSM_OK != ret)
        {
            uint32 j;
            for(j=0;j<i;j++)
            {
                App_Free(TPHandle, 
                    certgrp_ptr->GroupList.EncodedCertList[i].CertBlob.Data);
            }
            App_Free(TPHandle, certgrp_ptr->GroupList.EncodedCertList);
            App_Free(TPHandle, certgrp_ptr);
            return ret;
        }
    }

    *CertTemplates = certgrp_ptr;
    hashtab_clear();
    dump_free_cache();
    return CSSM_OK;

}
