/*-----------------------------------------------------------------------
 * File: CLBERDER.H
 *
 * Copyright (c) 1995-1999 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _CLBERDER_H
#define _CLDERBER_H
#include "mempool.h"

#define Init_DER_NODE_Header(node, MyTag, OrigTag, MyCount, MyType) \
    (node)->Tag = (MyTag);\
    (node)->OriginalTag = (OrigTag);\
    (node)->Count = (MyCount);\
    (node)->Type = (&MyType)

CSSM_BOOL IsBadBerTag(CSSM_BER_TAG Tag);

CSSM_RETURN  MP_CssmDataToBerItemPtr   (MEMORY_POOL_PTR poolptr,
                                        CSSM_BER_TAG Tag, 
                                        const CSSM_DATA Data,
                                        BER_PARSED_ITEM_PTR ber_item_ptr);

CSSM_RETURN MP_EncodeObjectToBerItem(MEMORY_POOL_PTR poolptr, CSSM_BER_TAG Tag, CSSM_DATA Data,
                            BER_PARSED_ITEM_PTR ber_item_ptr);

CSSM_RETURN MP_EncodeObject(MEMORY_POOL_PTR poolptr, CSSM_BER_TAG Tag, CSSM_DATA Data,
                            CSSM_DATA_PTR return_val);

CSSM_RETURN MP_CreateAlgorithmIdNode(MEMORY_POOL_PTR poolptr, 
                                     const CSSM_X509_ALGORITHM_IDENTIFIER algorithm,
                                     DER_NODE_PTR *nodeptr);

CSSM_RETURN MP_CreateNameNode(MEMORY_POOL_PTR poolptr, 
                             const CSSM_X509_NAME name,
                             DER_NODE_PTR *nodeptr);

CSSM_RETURN MP_DerEncodeName(MEMORY_POOL_PTR poolptr,
                              const CSSM_X509_NAME *name,
                              CSSM_DATA_PTR *result);

CSSM_RETURN MP_DerEncodeExtnValue(MEMORY_POOL_PTR poolptr,
                                   const CSSM_X509_EXTENSION Extension,
                                   CSSM_DATA_PTR result);

CSSM_RETURN MP_CreateExtensionsNode(MEMORY_POOL_PTR poolptr, 
                             const CSSM_X509_EXTENSION *Extensions, 
                             uint32 NumExtensions,
                             DER_NODE_PTR *nodeptr);

CSSM_RETURN MP_CreateTbsCertNode(MEMORY_POOL_PTR poolptr, 
                                 CSSM_X509_TBS_CERTIFICATE_PTR tbs_cert_ptr,
                                 DER_NODE_PTR *result_val);

CSSM_RETURN MP_CreateSignedCertNode(MEMORY_POOL_PTR poolptr, 
                                 CSSM_X509_SIGNED_CERTIFICATE_PTR signed_cert_ptr,
                                 DER_NODE_PTR *result_val);

CSSM_RETURN MP_DerEncodeTbsCertificate(MEMORY_POOL_PTR poolptr, 
                                       CSSM_X509_TBS_CERTIFICATE_PTR tbs_cert_ptr,
                                       CSSM_DATA_PTR *enc_tbs_cert_ptr);

CSSM_RETURN MP_DerEncodeSignedCertificate(MEMORY_POOL_PTR poolptr, 
                                       CSSM_X509_SIGNED_CERTIFICATE_PTR signed_cert_ptr,
                                       CSSM_DATA_PTR *enc_signed_cert_ptr);
CSSM_RETURN MP_DerEncodeSignature(MEMORY_POOL_PTR poolptr, 
                                  const CSSM_X509_SIGNATURE *signature,
                                  CSSM_DATA_PTR *result);

CSSM_RETURN MP_DerDecodeObject(MEMORY_POOL_PTR poolptr, 
                               CSSM_DATA Src, 
                               CSSM_DATA_PTR result);

CSSM_RETURN MP_DerDecodeItem(MEMORY_POOL_PTR poolptr,
                             BER_PARSED_ITEM_PTR BerItemPtr,
                             CSSM_DATA_PTR result);

CSSM_RETURN MP_DerDecodeExtensions(MEMORY_POOL_PTR poolptr, 
                                   const CSSM_DATA *Value, 
                                   CSSM_X509_EXTENSIONS_PTR *extensions_ptr_ptr);

CSSM_RETURN MP_DerDecodeExtensionsNode(MEMORY_POOL_PTR poolptr, 
                                       DER_NODE_PTR extns_node_ptr, 
                                       CSSM_X509_EXTENSIONS_PTR *extensions_ptr_ptr);

CSSM_RETURN MP_DerDecodeExtension(MEMORY_POOL_PTR poolptr, 
                                  const CSSM_DATA *Value, 
                                  CSSM_X509_EXTENSION_PTR *extension_ptr_ptr);

CSSM_RETURN MP_DerDecodeName(MEMORY_POOL_PTR poolptr, const CSSM_DATA *Value, CSSM_X509_NAME_PTR *name_ptr_ptr);


CSSM_RETURN MP_DerDecodeNameNode(MEMORY_POOL_PTR poolptr, DER_NODE_PTR name_node_ptr, CSSM_X509_NAME_PTR *name_ptr_ptr);

CSSM_RETURN MP_DerDecodeTbsCertificate(MEMORY_POOL_PTR poolptr, 
                                       const CSSM_DATA *Value, 
                                       CSSM_X509_TBS_CERTIFICATE_PTR *cert_ptr_ptr);

CSSM_RETURN MP_DerDecodeTbsCertificateNode(MEMORY_POOL_PTR poolptr, 
                                           DER_NODE_PTR cert_node_ptr, 
                                           CSSM_X509_TBS_CERTIFICATE_PTR *cert_ptr_ptr);


CSSM_RETURN MP_DerDecodeSignedCertificate(MEMORY_POOL_PTR poolptr, 
                                    const CSSM_DATA  *Value, 
                                    CSSM_X509_SIGNED_CERTIFICATE_PTR *signed_cert_ptr_ptr);


CSSM_RETURN MP_DerDecodeCertificateNode(MEMORY_POOL_PTR poolptr, 
                                        DER_NODE_PTR signed_cert_node_ptr, 
                                        CSSM_X509_SIGNED_CERTIFICATE_PTR *signed_cert_ptr_ptr);

CSSM_RETURN MP_DuplicateSPKI (MEMORY_POOL_PTR poolptr, 
                                      const CSSM_X509_SUBJECT_PUBLIC_KEY_INFO *SrcKey,
                                      CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR *result_val);

CSSM_RETURN MP_DuplicateSignature (MEMORY_POOL_PTR poolptr, 
                                      const CSSM_X509_SIGNATURE *SrcSig,
                                      CSSM_X509_SIGNATURE_PTR *result_val);

CSSM_RETURN MP_DuplicateCNameStruct(MEMORY_POOL_PTR poolptr,
                                      const CSSM_X509_NAME *SrcName,
                                      CSSM_X509_NAME_PTR *result_val);

CSSM_RETURN MP_CSSMKeyToSpki(MEMORY_POOL_PTR poolptr, 
                             CSSM_KEY_PTR CSSMKey, 
                             CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR *spki_ptr_ptr);

CSSM_RETURN MP_CopyAlgId(MEMORY_POOL_PTR poolptr, 
                         CSSM_X509_ALGORITHM_IDENTIFIER_PTR AlgIdIn,
                         CSSM_X509_ALGORITHM_IDENTIFIER_PTR AlgIdOut);

CSSM_RETURN MP_SpkiToCSSMKey(MEMORY_POOL_PTR poolptr,
                             CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR SPKI,
                             CSSM_KEY_PTR *key_ptr);


CSSM_RETURN MP_ObtainEffectiveKeySize (CSSM_KEY_PTR Key,
                                       uint32 *EffKeySize);


CSSM_RETURN MP_GetKeyFromSpki(MEMORY_POOL_PTR poolptr,
                              CSSM_ALGORITHMS AlgId, 
                              CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR SpkiPtr,
                              CSSM_DATA_PTR *key_ptr);


CSSM_RETURN MP_DSAKeyRejoin(MEMORY_POOL_PTR poolptr,                            
                            CSSM_X509_SUBJECT_PUBLIC_KEY_INFO_PTR SpkiPtr,
                            CSSM_DATA_PTR *key_ptr);

#endif
