/*-----------------------------------------------------------------------
 * File: CRTGETAL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
#include "maf_include.h"
    /* Windows & CSSM includes */
#ifndef VMS
#pragma warning (disable:4201 4214 4115)
#include <stdio.h>
#pragma warning (default:4201 4214 4115)
#else
#include <stdio.h>
#endif

#include "cssm.h"
#include "cssmspi.h"
#include "guids.h"
#include "cssmport.h"
#include "cssmutil.h"

    /* Certificate-specific includes */
#include "x509defs.h"
#include "x_fndefs.h"

#include "oidsx509.h"
#include "oidsalg.h"
#include "x509hlpr.h"
#include "clutil.h"
#include "x_qtable.h"
#include "internal.h"

#include "api_shim.h"
#include "maf_include.h"
#include "clberder.h"

#include <cssmMemTrack.h>

#define X509_FORMAT_VERSION 2

extern MEMORY_POOL_FUNCS memory_pool_funcs;
extern const CSSM_API_MEMORY_FUNCS Addin_APIMemFuncs;

/*-----------------------------------------------------------------------------
 * Name: CL_CertGetAllFields
 *
 * Description:
 * This function returns a list of the field identifiers used to describe 
 * the input certificate.
 *
 * Parameters: 
 * Cert (input)           : The certificate to be analyzed
 * FieldIdList (output)   : A pointer to the array of the CSSM certificates 
 *                          field identifiers (tags).
 * NumberOfFields (output): The number of fields in the certificate.  
 *                          Also, the length of the FieldIdList array.
 *
 *---------------------------------------------------------------------------*/
CSSM_OID_PTR RETRIEVABLE_X509_CERT_OIDS[] = {
    &CSSMOID_X509V1Version,
    &CSSMOID_X509V1SerialNumber,
    &CSSMOID_X509V1IssuerName,
    &CSSMOID_X509V1ValidityNotBefore,
    &CSSMOID_X509V1ValidityNotAfter,

    &CSSMOID_X509V1SubjectName,
/*  &CSSMOID_CSSMKeyStruct,   */
    &CSSMOID_X509V1SubjectPublicKeyAlgorithm,
    &CSSMOID_X509V1SubjectPublicKeyAlgorithmParameters,
    &CSSMOID_X509V1SubjectPublicKey,

    &CSSMOID_X509V1CertificateIssuerUniqueId,
    &CSSMOID_X509V1CertificateSubjectUniqueId,
    &CSSMOID_X509V3CertificateNumberOfExtensions,
    &CSSMOID_X509V3CertificateExtensionStruct,
    &CSSMOID_X509V3CertificateExtensionId,

    &CSSMOID_X509V3CertificateExtensionCritical,
    &CSSMOID_X509V3CertificateExtensionType,
    &CSSMOID_X509V3CertificateExtensionValue,
    &CSSMOID_X509V1SignatureStruct,
    &CSSMOID_X509V1SignatureAlgorithm,

    &CSSMOID_X509V1SignatureAlgorithmParameters,
    &CSSMOID_X509V1Signature,    
/*  &CSSMOID_X509V3SignedCertificate, */
    &CSSMOID_X509V3Certificate,
    &CSSMOID_X509V1IssuerNameCStruct,

    &CSSMOID_X509V1SubjectNameCStruct,
    &CSSMOID_X509V1IssuerNameLDAP,
    &CSSMOID_X509V1SubjectNameLDAP,
    &CSSMOID_X509V1SubjectPublicKeyCStruct,
    &CSSMOID_X509V1SignatureCStruct, 

/*  &CSSMOID_X509ExtBasicConstraints, */
/*  &CSSMOID_X509ExtPrivateUseExemptions, */
    &CSSMOID_X509V3SignedCertificateCStruct,
    &CSSMOID_X509V3CertificateCStruct,
    &CSSMOID_X509V3CertificateExtensionsCStruct

/*  &CSSMOID_X509V3CertificateExtensionsStruct, */
/*  &CSSMOID_X509V3CertifciateExtensionCStruct */
};

#define NUMBER_RETRIEVABLE_X509_CERT_OIDS  (sizeof(RETRIEVABLE_X509_CERT_OIDS)/sizeof(CSSM_OID_PTR))


CSSM_RETURN CSSMCLI CL_CertGetAllFields(CSSM_CL_HANDLE CLHandle,
                                            const CSSM_DATA *Cert,
                                            uint32 *NumberOfFields,
                                            CSSM_FIELD_PTR *result_val)
{
    
    CSSM_FIELD_PTR fieldp = NULL;
    CSSM_FIELD_PTR tofieldp = NULL;
    CSSM_RETURN ret = CSSM_OK;
    MEMORY_POOL pool;
    uint32 i = 0;
    
    if (port_IsBadWritePtr (result_val, sizeof(CSSM_FIELD_PTR)))
    {
        return CSSMERR_CL_INVALID_OUTPUT_POINTER;
    }
    *result_val = NULL;   
    /* Setup the Memory pool */
    ret = (*memory_pool_funcs.init)((CSSM_MEMORY_FUNCS_PTR)&Addin_APIMemFuncs, NULL, &pool);
    if (CSSM_OK != ret)
    {
        return ret;
    }

    
    ret = cl_CertGetManyFields(CLHandle, 
                               &pool,
                                Cert, 
                                (const struct cssm_data **)RETRIEVABLE_X509_CERT_OIDS, 
                                NUMBER_RETRIEVABLE_X509_CERT_OIDS, 
                                NumberOfFields, &fieldp);

    
    

    if (CSSM_OK != ret || !fieldp)
    {
        (*memory_pool_funcs.recycle)(&pool);
        return ret;
    }

    tofieldp = App_Malloc(CLHandle, (*NumberOfFields)*sizeof(CSSM_FIELD));
    if (!tofieldp)
    {
        ret = CSSMERR_CL_MEMORY_ERROR;
        goto CLEANUP_AND_EXIT;
    }
    /* Copy fieldp to tofieldp in heap space */
    for(i=0;i<*NumberOfFields;i++)
    {
        tofieldp[i].FieldOid.Length = fieldp[i].FieldOid.Length;
        tofieldp[i].FieldOid.Data = App_Malloc(CLHandle, tofieldp[i].FieldOid.Length);
        if (!(tofieldp[i].FieldOid.Data))
        {
            ret = CSSMERR_CL_MEMORY_ERROR;
            goto CLEANUP_AND_EXIT;
        }
        
        port_memcpy(tofieldp[i].FieldOid.Data, fieldp[i].FieldOid.Data, tofieldp[i].FieldOid.Length);
        
        /* Malloc and copy if not one of the values already on the heap */
        switch (x509_cl_CertOidToInt(fieldp[i].FieldOid))
        {
            case INT_IssuerNameCStruct:
            case INT_SubjectNameCStruct:
            case INT_SubjectPublicKeyCStruct:
            case INT_SignatureCStruct:
            case INT_IssuerNameLDAP:
            case INT_SubjectNameLDAP:
            case INT_SignedCertificateCStruct:
            case INT_CertificateCStruct:
            case INT_CertificateExtensionsCStruct:
                /* Just do an assignment */
                tofieldp[i].FieldValue.Length = fieldp[i].FieldValue.Length;
                tofieldp[i].FieldValue.Data = fieldp[i].FieldValue.Data;
                break;
            default:
                /* Malloc and copy */
                tofieldp[i].FieldValue.Length = fieldp[i].FieldValue.Length;
                tofieldp[i].FieldValue.Data = App_Malloc(CLHandle, tofieldp[i].FieldValue.Length);
                if (!(tofieldp[i].FieldValue.Data))
                {
                    ret = CSSMERR_CL_MEMORY_ERROR;
                    goto CLEANUP_AND_EXIT;
                }
        
                port_memcpy(tofieldp[i].FieldValue.Data, 
                            fieldp[i].FieldValue.Data, 
                            tofieldp[i].FieldValue.Length);
                break;
                
        }
        
        
    }

    *result_val = tofieldp;
    (*memory_pool_funcs.recycle)(&pool);
    return CSSM_OK;

    /* Error exit cleanup code */
CLEANUP_AND_EXIT:
    for(i=0;i< *NumberOfFields; i++)
    {
        switch (x509_cl_CertOidToInt(fieldp[i].FieldOid))
        {
            case INT_IssuerNameCStruct:
            case INT_SubjectNameCStruct:
            case INT_SubjectPublicKeyCStruct:
            case INT_SignatureCStruct:
            case INT_IssuerNameLDAP:
            case INT_SubjectNameLDAP:
            case INT_SignedCertificateCStruct:
            case INT_CertificateCStruct:
            case INT_CertificateExtensionsCStruct:
                /* Free data since its not from the mempool */
                App_Free(CLHandle, fieldp[i].FieldValue.Data);
            default:
                /* Do nothing */
                ;
        }
        /* Free data since its not from the mempool */
        if (tofieldp[i].FieldValue.Data)
        {
            App_Free(CLHandle, tofieldp[i].FieldValue.Data);
        }
        if (tofieldp[i].FieldOid.Data)
        {
            App_Free(CLHandle, tofieldp[i].FieldOid.Data);
        }
    }
    App_Free(CLHandle, tofieldp);
    *NumberOfFields = 0;
    (*memory_pool_funcs.recycle)(&pool);
    return ret;

}



