/*-----------------------------------------------------------------------
 * File: SPKI_KEY.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


#include "oidsalg.h"

#include "guids.h"
#include "x_fndefs.h"
#include "x509hlpr.h"
#include "clutil.h"
#include "api_shim.h"
#include "internal.h"
#include "maf_include.h"
#include "CSM/csm_keyparse.h"
#include <cssmMemTrack.h>
#include "mds_util_api.h"


CSSM_SUBSERVICE_UID_PTR 
x509_cl_DoesCspSupportthisAlgKeySize(CSSM_HANDLE AppHandle, 
                                     CSSM_UPCALLS_PTR MemoryFunctions, 
                                     CSSM_ALGORITHMS AlgorithmId, 
                                     uint32 EffectiveKeySizeInBits,
                                     MDSUTIL_MODULE_INFO_PTR mod_info,
                                     CSSM_SUBSERVICE_UID SubServiceUid)
{
    CSSM_CONTEXT_PTR cap_list = NULL; 
    uint32 num_cap=0; 
    uint32 j=0, k=0, l=0, m; 
    CSSM_SUBSERVICE_UID_PTR subservice_uid = NULL; 

    if (mod_info) 
    {
        for (j=0; j < mod_info->NumberOfServices; j++) 
        { 
            if (mod_info->ServiceList[j].Type != CSSM_SERVICE_CSP) 
            {
                continue; 
            }
            for (k=0; k < mod_info->ServiceList[j].NumberOfSubServices; k++) 
            { 
                /* Find out if this is a supported alg */ 
                num_cap =
                    ((MDSUTIL_CSPSUBSERVICE_PTR)mod_info->ServiceList[j].SubServiceList[k].SubServiceInfo)->SubServiceInfo.HardwareCspSubService.NumberOfCapabilities;
                cap_list =
                    ((MDSUTIL_CSPSUBSERVICE_PTR)mod_info->ServiceList[j].SubServiceList[k].SubServiceInfo)->SubServiceInfo.HardwareCspSubService.CapabilityList;

                for (l=0; l < num_cap; l++) 
                { 
                    if (cap_list[l].AlgorithmType == (sint32) AlgorithmId) 
                    {
                        if (EffectiveKeySizeInBits == 0)
                        {
                            break;
                        }
                        for (m=0; m < cap_list[l].NumberOfAttributes; m++)
                        {
                            if (cap_list[l].ContextAttributes[m].AttributeType == CSSM_ATTRIBUTE_KEY_LENGTH)
                            {
                                if (cap_list[l].ContextAttributes[m].Attribute.Uint32 == EffectiveKeySizeInBits)
                                {
                                    break;
                                }
                            }

                            if(cap_list[l].ContextAttributes[m].AttributeType == CSSM_ATTRIBUTE_KEY_LENGTH_RANGE)
                            {
                                uint32 min = cap_list[l].ContextAttributes[m].Attribute.Range->Min;
                                uint32 max = cap_list[l].ContextAttributes[m].Attribute.Range->Max;
                                if(min > max)
                                {
                                    uint32 temp;
                                    temp = min;
                                    min = max;
                                    max = temp;

                                }
                                
                                if((EffectiveKeySizeInBits >= min) ||
                                    (EffectiveKeySizeInBits <= max))
                                {

                                    /* we have a match for this search param*/
                                    break;  
                                }
                            }


                        }
                        if (m != cap_list[l].NumberOfAttributes)
                        {
                            break;
                        }
                    } 
                } 
                if (l != num_cap)  /* This algorithm is available from this CSP */ 
                { 
                    subservice_uid = CSSM_MALLOC_WRAPPER(MemoryFunctions->malloc_func, AppHandle, sizeof(CSSM_SUBSERVICE_UID)); 
                    if (subservice_uid) 
                    {
                        *subservice_uid = SubServiceUid; 
                    }
                    return subservice_uid; 
                } 
            } /* Loop to next subservice */ 
        } /* Loop to next service */ 
    }
    return subservice_uid;
}

/*----------------------------------------------------------------------------- 
   * Name: x509_cl_FindCSPSupportingThisAlg 
   * 
   * Description: 
   * 
   * Parameters: 
   * 
   * Return value: 
   * 
   * Error Codes: 
   *---------------------------------------------------------------------------*/ 
CSSM_SUBSERVICE_UID_PTR 
x509_cl_FindCSPSupportingThisAlg(CSSM_HANDLE AppHandle, 
                                 CSSM_UPCALLS_PTR MemoryFunctions, 
                                 CSSM_ALGORITHMS AlgorithmId, 
                                 uint32 EffectiveKeySizeInBits,
                                 CSSM_USEE_TAG *Tag)
{ 
    CSSM_SUBSERVICE_UID_PTR subservice_uid = NULL; 
    MDSUTIL_LIST_PTR mod_list = NULL; 
    MDSUTIL_MODULE_INFO_PTR mod_info = NULL; 
    uint32 i=0; 
    CSSM_RETURN ret = CSSM_OK;

    /* For each CSP, */ 
    ret = MDSUTIL_ListModules(CSSM_SERVICE_CSP, CSSM_FALSE, &mod_list); 
    if (!mod_list)  
    {
        return NULL; 
    }

    for (i=0; i < mod_list->NumberItems; i++) 
    { 
        ret = MDSUTIL_GetModuleInfo (&mod_list->Items[i].SubserviceUid.Guid, 
                                        CSSM_SERVICE_CSP, 
                                        mod_list->Items[i].SubserviceUid.SubserviceId, 
                                        CSSM_USEE_NONE,
                                        &mod_info); 
        if(mod_info)
        {
            subservice_uid = 
                x509_cl_DoesCspSupportthisAlgKeySize(AppHandle,
                                                     MemoryFunctions,
                                                     AlgorithmId,EffectiveKeySizeInBits, 
                                                     mod_info,
                                                     mod_list->Items[i].SubserviceUid);
            MDSUTIL_FreeModuleInfo(mod_info); 
            if(subservice_uid != NULL)
            {
                MDSUTIL_FreeModuleList(mod_list);
                *Tag = CSSM_USEE_NONE;
                return subservice_uid;
            }
        }
        
        ret = MDSUTIL_GetModuleInfo (&mod_list->Items[i].SubserviceUid.Guid, 
                                            CSSM_SERVICE_CSP, 
                                            mod_list->Items[i].SubserviceUid.SubserviceId, 
                                            CSSM_USEE_AUTHENTICATION,
                                            &mod_info); 
        if( mod_info)
        {
            subservice_uid = 
                x509_cl_DoesCspSupportthisAlgKeySize (AppHandle,
                                                          MemoryFunctions,
                                                          AlgorithmId,
                                                          EffectiveKeySizeInBits, 
                                                          mod_info,
                                                          mod_list->Items[i].SubserviceUid);
            MDSUTIL_FreeModuleInfo(mod_info); 
            if(subservice_uid != NULL)
            {
                MDSUTIL_FreeModuleList(mod_list); 
                *Tag = CSSM_USEE_AUTHENTICATION;
                return subservice_uid;
            }
        }

    } /* Loop to next module */ 
    MDSUTIL_FreeModuleList(mod_list); 
    return NULL; 
} 


