/*-----------------------------------------------------------------------
 * File: MAF_CONFIG.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef MAF_CONFIG_H
#define MAF_CONFIG_H

#ifndef MAF_CSM_INCLUDED
#define MAF_CSM_INCLUDED
#endif


#include "CSM/csm_sesn.h"
#ifdef VMS
#include "csm_keymgr.h"
#else
#include "CSM/keymgr/csm_keymgr.h"
#endif

/* GUID of the addin module */
/* {67EF50D0-FE74-11d2-A8E6-0090271D266F} */
#define ADDIN_GUID_INITIALIZER              {                             \
                                              0x67ef50d0, 0xfe74, 0x11d2, \
                                              {                           \
                                                 0xa8, 0xe6, 0x0, 0x90,   \
                                                 0x27,0x1d, 0x26, 0x6f    \
                                              }                           \
                                            }

/* Module name of the service provider */
#define ADDIN_NAME                          "eaycsp300.dll"
#define ADDIN_PRINT_NAME                    "SSLeay Crypto Based CSP"
#ifdef VMS
#define ADDIN_VENDOR_NAME                   "Compaq Computer Corporation"
#else
#define ADDIN_VENDOR_NAME                   "Intel Corporation"
#endif
#define ADDIN_STANDARD_DESCRIPTION          "SSLeay 0.9.0 Crypto"
#define ADDIN_STANDARD_VERSION_MAJOR        0
#define ADDIN_STANDARD_VERSION_MINOR        9
#define ADDIN_PRODUCT_DESCRIPTION           ADDIN_STANDARD_DESCRIPTION
#define ADDIN_PRODUCT_VERSION_MAJOR         3
#define ADDIN_PRODUCT_VERSION_MINOR         1

/* Version of the CSSM is compatible with this module */
#define ADDIN_VERSION_MAJOR                 CSSM_MAJOR
#define ADDIN_VERSION_MINOR                 CSSM_MINOR

/* Name of the manifest section used to verify the module */
#ifdef VMS
#define ADDIN_SELF_CHECK_SECTION            "cdsa$eaycsp300_shr"
#else
#define ADDIN_SELF_CHECK_SECTION            "eaycsp300"
#endif

/* Is the addin module thread safe? */
#define ADDIN_THREAD_SAFE                   1

/* Does the addin module require the writer lock durring SPI functions on the load 
tracker node (is this node being modified?) */
#define ADDIN_NEED_LOAD_WRITER_LOCK         0

/* Does the addin module require the writer lock durring SPI functions on the attach 
tracker node (is this node being modified?) */
#define ADDIN_NEED_ATTACH_WRITER_LOCK       0

/* Does the addin module require the custom ADDIN_LOAD_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_LOAD_STRUCT        1

/* Does the addin module require the custom ADDIN_ATTACH_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_ATTACH_STRUCT      1

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_CSP_SUBSERVICE_ID                0
#define ADDIN_IS_VALID_SUBSERVICE(SubServiceId, SubServiceType) \
    ( ( (SubServiceId) == ADDIN_CSP_SUBSERVICE_ID) && \
      ( (SubServiceType) == CSSM_SERVICE_CSP ) )

#if (ADDIN_NEED_ADDIN_LOAD_STRUCT)
/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_LOAD_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_load_data
{
    /* Contains the handle to the private key file for this load */
    CSM_KM_HANDLE hKeyFile;
};
typedef struct addin_load_data ADDIN_LOAD_DATA;

#endif /* if ADDIN_NEED_ADDIN_LOAD_STRUCT */


#if (ADDIN_NEED_ADDIN_ATTACH_STRUCT)

/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_ATTACH_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_attach_data
{
    /* Maintains a list of CSP "sessions" in this attach. It does not
     * require the attach record to be locked since it has internal
     * locking machanisms. If the record is not locked, then this
     * value will have to be casted to (CSM_SESSION_LIST*) to remove
     * the 'const' declaration of the value returned by 
     * Addin_GetAttachData when ADDIN_NEED_ATTACH_WRITER_LOCK==0.
     */
    CSM_SESSION_LIST slSessions;
};
typedef struct addin_attach_data ADDIN_ATTACH_DATA;

#endif /* if ADDIN_NEED_ADDIN_ATTACH_STRUCT */

#endif
