/*-----------------------------------------------------------------------
 * File: CSP_STRT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
#include "geninc.h"

B_ALGORITHM_OBJ g_RandomNumGen = (B_ALGORITHM_OBJ)NULL_PTR;

/* BSafe MD5 Random Number Generation Algorithm Chooser */
B_ALGORITHM_METHOD  *MD5_RANDOM_CHOOSER[] = {
    &AM_MD5_RANDOM,
    (B_ALGORITHM_METHOD*)NULL_PTR
};

CSSM_RETURN csp_ProcStartup()
{
    int             status;
    uint8           randomSeed[256];
    uint8           *pSeedBuf = randomSeed;
    uint32          randomSeedLen;
    uint32          dataLen = 0;
    CSSM_RETURN     retval = CSSM_OK;

    /* Create a global MD5 random number generator on startup for internal
       random number generation */

    if (!g_RandomNumGen) {
        /* Step 1: Create the Algorithm Object */
        if ((status = B_CreateAlgorithmObject(&g_RandomNumGen)) != 0)
        {
            retval = ConvertBsafeError(status);
            goto FAIL;
        }

        /* Step 2: Set the Algorithm Object */
        if ((status = B_SetAlgorithmInfo(g_RandomNumGen, 
                                AI_MD5Random, NULL_PTR)) != 0)
        {
            retval = ConvertBsafeError(status);
            goto FAIL;
        }

        /* Step 3: Initialize */
        if ((status = B_RandomInit(g_RandomNumGen, MD5_RANDOM_CHOOSER,
                                (A_SURRENDER_CTX*)NULL_PTR)) != 0)
        {
            retval = ConvertBsafeError(status);
            goto FAIL;
        }

        /* Step 4: Update */
        /* Use TAL Utility function to generate seed bytes */
        while (dataLen < sizeof(randomSeed)/2) {
            randomSeedLen = sizeof(randomSeed) - dataLen;
            if ((retval = csp_GetEnvironmentRandom(pSeedBuf, 
                                &randomSeedLen)) != CSSM_OK)
            {
                goto FAIL;
            }
            pSeedBuf += randomSeedLen;
            dataLen += randomSeedLen;
        }

        if ((status = B_RandomUpdate(g_RandomNumGen, randomSeed, 
                                dataLen, (A_SURRENDER_CTX*)NULL_PTR)) != 0)
        {
            retval = ConvertBsafeError(status);
        }
    }

    if (retval != CSSM_OK)
    {
        B_DestroyAlgorithmObject(&g_RandomNumGen);
    }
    /* Initialize the Key Manager */
    else
    {
        retval = CSM_KMInit( );
    }

    return retval;

FAIL:
    if (g_RandomNumGen)
    {
        B_DestroyAlgorithmObject(&g_RandomNumGen);
    }

    return retval;
}

CSSM_RETURN csp_ProcShutdown()
{
    CSSM_RETURN     rtn = CSSM_OK;
    
    /* If our global random number generator is around, remove it */
    if (g_RandomNumGen)
    {
        B_DestroyAlgorithmObject(&g_RandomNumGen);
        g_RandomNumGen = NULL_PTR;
    }
    
    /* Terminate the Key Manager */
    rtn = CSM_KMTerm( );

    return rtn;
}
