/*-----------------------------------------------------------------------
 * File: ERRCONV.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * File Description: Contains function to convert from BSafe error codes to
 *                   CDSA error codes.
 */

/*
 * OpenVMS History
 *
 *    23-Oct-1999       Carol Kelly
 *              o Merge Unix changes for BSAFE_VERSION > 3
 *
 *
 */
#include "geninc.h"

uint32 ConvertBsafeError(int error)
{
    switch (error)
    {
    case BE_ALGORITHM_ALREADY_SET:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALGORITHM_INFO:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALGORITHM_NOT_INITIALIZED:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALGORITHM_NOT_SET:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALGORITHM_OBJ:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALG_OPERATION_UNKNOWN:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_ALLOC:
        return CSSMERR_CSP_MEMORY_ERROR;
    case BE_CANCEL:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_DATA:
        return CSSMERR_CSP_INVALID_DATA;
    case BE_EXPONENT_EVEN:
        return CSSMERR_CSP_INVALID_KEY;
    case BE_EXPONENT_LEN:
        return CSSMERR_CSP_INVALID_KEY;
    case BE_HARDWARE:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_INPUT_DATA:
        return CSSMERR_CSP_INVALID_DATA;
    case BE_INPUT_LEN:
        return CSSMERR_CSP_INPUT_LENGTH_ERROR;
    case BE_KEY_ALREADY_SET:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_KEY_INFO:
        return CSSMERR_CSP_INVALID_KEY_FORMAT;
    case BE_KEY_LEN:
        return CSSMERR_CSP_INVALID_ATTR_KEY_LENGTH;
    case BE_KEY_NOT_SET:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_KEY_OBJ:
        return CSSMERR_CSP_INVALID_KEY;
    case BE_KEY_OPERATION_UNKNOWN:
        return CSSMERR_CSP_INVALID_ALGORITHM;
    case BE_MEMORY_OBJ:
        return CSSMERR_CSP_MEMORY_ERROR;
    case BE_MODULUS_LEN:
        return CSSMERR_CSP_INVALID_ATTR_KEY_LENGTH;
    case BE_NOT_INITIALIZED:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_NOT_SUPPORTED:
        return CSSMERR_CSP_FUNCTION_NOT_IMPLEMENTED;
    case BE_OUTPUT_LEN:
        return CSSMERR_CSP_OUTPUT_LENGTH_ERROR;
    case BE_OVER_32K:
        return CSSMERR_CSP_INVALID_DATA;
    case BE_RANDOM_NOT_INITIALIZED:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_RANDOM_OBJ:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_SIGNATURE:
        return CSSMERR_CSP_VERIFY_FAILED; 
    case BE_WRONG_ALGORITHM_INFO:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_WRONG_KEY_INFO:
        return CSSMERR_CSP_INVALID_KEY;
    case BE_INPUT_COUNT:
        return CSSMERR_CSP_INVALID_DATA_COUNT;
    case BE_OUTPUT_COUNT:
        return CSSMERR_CSP_INVALID_DATA_COUNT;
    case BE_METHOD_NOT_IN_CHOOSER:
        return CSSMERR_CSP_INTERNAL_ERROR;
    case BE_KEY_WEAK:
        return CSSMERR_CSP_INVALID_KEY;
#if BSAFE_VERSION > 3
    case BE_EXPONENT_ONE:
        return CSSMERR_CSP_INVALID_KEY;
    case BE_BAD_POINTER:
        return CSSMERR_CSP_INVALID_POINTER;
    case BE_BAD_PASSPHRASE:
        return CSSMERR_CSP_INVALID_ACCESS_CREDENTIALS;
#endif
    default:
        return CSSMERR_CSP_INTERNAL_ERROR;
    }
}
