/*-----------------------------------------------------------------------
 * File: DAL_CREATE.CPP
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 *  Contains module management functions for DAL Default Schemas
 */

#include "cssmport.h"
#include "cssmlock.h"
#include "cssmerr.h"
#include "maf_integ.h"
#include "maf_collectn.h"
#include "maf_util.h"
#include "dal_common.h"
#include "dal_schema.h"

/*-----------------------------------------------------------------------------
                        Defines for UINDEX and INDEX
-----------------------------------------------------------------------------*/

#define UINDEX(attribute) { \
    CSSM_DB_INDEX_UNIQUE, /* IndexType */ \
    CSSM_DB_INDEX_ON_ATTRIBUTE, /* IndexedDataLocation */ \
    attribute \
}

#define INDEX(attribute) { \
    CSSM_DB_INDEX_NONUNIQUE, /* IndexType */ \
    CSSM_DB_INDEX_ON_ATTRIBUTE, /* IndexedDataLocation */ \
    attribute \
}


/*-----------------------------------------------------------------------------
 Get Attribute and Get Index functions for Open Group Application Record Types
-----------------------------------------------------------------------------*/

/* CERT */

void dal_CERT_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_CERTTYPE,
        DAL_ATTRINFO_CERTENCODING,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_CERT);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_CERT_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_CERT_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_CERTTYPE),
        INDEX(DAL_ATTRINFO_CERTENCODING),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_CERT);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_CERT_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* CRL */

void dal_CRL_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_CRLTYPE,
        DAL_ATTRINFO_CRLENCODING,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_CRL);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_CRL_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_CRL_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_CRLTYPE),
        INDEX(DAL_ATTRINFO_CRLENCODING),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_CRL);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_CRL_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* POLICY */

void dal_POLICY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_POLICYNAME,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_POLICY);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_POLICY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_POLICY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_POLICYNAME),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_POLICY);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_POLICY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* GENERIC */

void dal_GENERIC_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_GENERIC);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_GENERIC_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_GENERIC_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_GENERIC);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_GENERIC_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* PUBLIC_KEY */

void dal_PUBLIC_KEY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_KEYCLASS,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS,
        DAL_ATTRINFO_PERMANENT,
        DAL_ATTRINFO_PRIVATE,
        DAL_ATTRINFO_MODIFIABLE,
        DAL_ATTRINFO_LABEL,
        DAL_ATTRINFO_APPLICATIONTAG,
        DAL_ATTRINFO_KEYCREATOR,
        DAL_ATTRINFO_KEYTYPE,
        DAL_ATTRINFO_KEYSIZEINBITS,
        DAL_ATTRINFO_EFFECTIVEKEYSIZE,
        DAL_ATTRINFO_STARTDATE,
        DAL_ATTRINFO_ENDDATE,
        DAL_ATTRINFO_SENSITIVE,
        DAL_ATTRINFO_ALWAYSSENSITIVE,
        DAL_ATTRINFO_EXTRACTABLE,
        DAL_ATTRINFO_NEVEREXTRACTABLE,
        DAL_ATTRINFO_ENCRYPT,
        DAL_ATTRINFO_DECRYPT,
        DAL_ATTRINFO_DERIVE,
        DAL_ATTRINFO_SIGN,
        DAL_ATTRINFO_VERIFY,
        DAL_ATTRINFO_SIGNRECOVER,
        DAL_ATTRINFO_VERIFYRECOVER,
        DAL_ATTRINFO_WRAP,
        DAL_ATTRINFO_UNWRAP
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_PUBLIC_KEY);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_PUBLIC_KEY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_PUBLIC_KEY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_KEYCLASS),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS),
        INDEX(DAL_ATTRINFO_PERMANENT),
        INDEX(DAL_ATTRINFO_PRIVATE),
        INDEX(DAL_ATTRINFO_MODIFIABLE),
        INDEX(DAL_ATTRINFO_LABEL),
        INDEX(DAL_ATTRINFO_APPLICATIONTAG),
        INDEX(DAL_ATTRINFO_KEYCREATOR),
        INDEX(DAL_ATTRINFO_KEYTYPE),
        INDEX(DAL_ATTRINFO_KEYSIZEINBITS),
        INDEX(DAL_ATTRINFO_EFFECTIVEKEYSIZE),
        INDEX(DAL_ATTRINFO_STARTDATE),
        INDEX(DAL_ATTRINFO_ENDDATE),
        INDEX(DAL_ATTRINFO_SENSITIVE),
        INDEX(DAL_ATTRINFO_ALWAYSSENSITIVE),
        INDEX(DAL_ATTRINFO_EXTRACTABLE),
        INDEX(DAL_ATTRINFO_NEVEREXTRACTABLE),
        INDEX(DAL_ATTRINFO_ENCRYPT),
        INDEX(DAL_ATTRINFO_DECRYPT),
        INDEX(DAL_ATTRINFO_DERIVE),
        INDEX(DAL_ATTRINFO_SIGN),
        INDEX(DAL_ATTRINFO_VERIFY),
        INDEX(DAL_ATTRINFO_SIGNRECOVER),
        INDEX(DAL_ATTRINFO_VERIFYRECOVER),
        INDEX(DAL_ATTRINFO_WRAP),
        INDEX(DAL_ATTRINFO_UNWRAP)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_PUBLIC_KEY);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_PUBLIC_KEY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* PRIVATE_KEY */

void dal_PRIVATE_KEY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_KEYCLASS,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS,
        DAL_ATTRINFO_PERMANENT,
        DAL_ATTRINFO_PRIVATE,
        DAL_ATTRINFO_MODIFIABLE,
        DAL_ATTRINFO_LABEL,
        DAL_ATTRINFO_APPLICATIONTAG,
        DAL_ATTRINFO_KEYCREATOR,
        DAL_ATTRINFO_KEYTYPE,
        DAL_ATTRINFO_KEYSIZEINBITS,
        DAL_ATTRINFO_EFFECTIVEKEYSIZE,
        DAL_ATTRINFO_STARTDATE,
        DAL_ATTRINFO_ENDDATE,
        DAL_ATTRINFO_SENSITIVE,
        DAL_ATTRINFO_ALWAYSSENSITIVE,
        DAL_ATTRINFO_EXTRACTABLE,
        DAL_ATTRINFO_NEVEREXTRACTABLE,
        DAL_ATTRINFO_ENCRYPT,
        DAL_ATTRINFO_DECRYPT,
        DAL_ATTRINFO_DERIVE,
        DAL_ATTRINFO_SIGN,
        DAL_ATTRINFO_VERIFY,
        DAL_ATTRINFO_SIGNRECOVER,
        DAL_ATTRINFO_VERIFYRECOVER,
        DAL_ATTRINFO_WRAP,
        DAL_ATTRINFO_UNWRAP
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_PRIVATE_KEY);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_PRIVATE_KEY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_PRIVATE_KEY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_KEYCLASS),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS),
        INDEX(DAL_ATTRINFO_PERMANENT),
        INDEX(DAL_ATTRINFO_PRIVATE),
        INDEX(DAL_ATTRINFO_MODIFIABLE),
        INDEX(DAL_ATTRINFO_LABEL),
        INDEX(DAL_ATTRINFO_APPLICATIONTAG),
        INDEX(DAL_ATTRINFO_KEYCREATOR),
        INDEX(DAL_ATTRINFO_KEYTYPE),
        INDEX(DAL_ATTRINFO_KEYSIZEINBITS),
        INDEX(DAL_ATTRINFO_EFFECTIVEKEYSIZE),
        INDEX(DAL_ATTRINFO_STARTDATE),
        INDEX(DAL_ATTRINFO_ENDDATE),
        INDEX(DAL_ATTRINFO_SENSITIVE),
        INDEX(DAL_ATTRINFO_ALWAYSSENSITIVE),
        INDEX(DAL_ATTRINFO_EXTRACTABLE),
        INDEX(DAL_ATTRINFO_NEVEREXTRACTABLE),
        INDEX(DAL_ATTRINFO_ENCRYPT),
        INDEX(DAL_ATTRINFO_DECRYPT),
        INDEX(DAL_ATTRINFO_DERIVE),
        INDEX(DAL_ATTRINFO_SIGN),
        INDEX(DAL_ATTRINFO_VERIFY),
        INDEX(DAL_ATTRINFO_SIGNRECOVER),
        INDEX(DAL_ATTRINFO_VERIFYRECOVER),
        INDEX(DAL_ATTRINFO_WRAP),
        INDEX(DAL_ATTRINFO_UNWRAP)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_PRIVATE_KEY);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_PRIVATE_KEY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* SYMMETRIC_KEY */

void dal_SYMMETRIC_KEY_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_KEYCLASS,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS,
        DAL_ATTRINFO_PERMANENT,
        DAL_ATTRINFO_PRIVATE,
        DAL_ATTRINFO_MODIFIABLE,
        DAL_ATTRINFO_LABEL,
        DAL_ATTRINFO_APPLICATIONTAG,
        DAL_ATTRINFO_KEYCREATOR,
        DAL_ATTRINFO_KEYTYPE,
        DAL_ATTRINFO_KEYSIZEINBITS,
        DAL_ATTRINFO_EFFECTIVEKEYSIZE,
        DAL_ATTRINFO_STARTDATE,
        DAL_ATTRINFO_ENDDATE,
        DAL_ATTRINFO_SENSITIVE,
        DAL_ATTRINFO_ALWAYSSENSITIVE,
        DAL_ATTRINFO_EXTRACTABLE,
        DAL_ATTRINFO_NEVEREXTRACTABLE,
        DAL_ATTRINFO_ENCRYPT,
        DAL_ATTRINFO_DECRYPT,
        DAL_ATTRINFO_DERIVE,
        DAL_ATTRINFO_SIGN,
        DAL_ATTRINFO_VERIFY,
        DAL_ATTRINFO_SIGNRECOVER,
        DAL_ATTRINFO_VERIFYRECOVER,
        DAL_ATTRINFO_WRAP,
        DAL_ATTRINFO_UNWRAP
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_SYMMETRIC_KEY);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_SYMMETRIC_KEY_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_SYMMETRIC_KEY_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_KEYCLASS),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS),
        INDEX(DAL_ATTRINFO_PERMANENT),
        INDEX(DAL_ATTRINFO_PRIVATE),
        INDEX(DAL_ATTRINFO_MODIFIABLE),
        INDEX(DAL_ATTRINFO_LABEL),
        INDEX(DAL_ATTRINFO_APPLICATIONTAG),
        INDEX(DAL_ATTRINFO_KEYCREATOR),
        INDEX(DAL_ATTRINFO_KEYTYPE),
        INDEX(DAL_ATTRINFO_KEYSIZEINBITS),
        INDEX(DAL_ATTRINFO_EFFECTIVEKEYSIZE),
        INDEX(DAL_ATTRINFO_STARTDATE),
        INDEX(DAL_ATTRINFO_ENDDATE),
        INDEX(DAL_ATTRINFO_SENSITIVE),
        INDEX(DAL_ATTRINFO_ALWAYSSENSITIVE),
        INDEX(DAL_ATTRINFO_EXTRACTABLE),
        INDEX(DAL_ATTRINFO_NEVEREXTRACTABLE),
        INDEX(DAL_ATTRINFO_ENCRYPT),
        INDEX(DAL_ATTRINFO_DECRYPT),
        INDEX(DAL_ATTRINFO_DERIVE),
        INDEX(DAL_ATTRINFO_SIGN),
        INDEX(DAL_ATTRINFO_VERIFY),
        INDEX(DAL_ATTRINFO_SIGNRECOVER),
        INDEX(DAL_ATTRINFO_VERIFYRECOVER),
        INDEX(DAL_ATTRINFO_WRAP),
        INDEX(DAL_ATTRINFO_UNWRAP)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_SYMMETRIC_KEY);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_SYMMETRIC_KEY_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/* ALL_KEYS */

void dal_ALL_KEYS_GetAttributes(CSSM_DB_RECORD_ATTRIBUTE_INFO *pList)
{
    static CSSM_DB_ATTRIBUTE_INFO rgAttributes[] = {
        DAL_ATTRINFO_KEYCLASS,
        DAL_ATTRINFO_PRINTNAME,
        DAL_ATTRINFO_ALIAS,
        DAL_ATTRINFO_PERMANENT,
        DAL_ATTRINFO_PRIVATE,
        DAL_ATTRINFO_MODIFIABLE,
        DAL_ATTRINFO_LABEL,
        DAL_ATTRINFO_APPLICATIONTAG,
        DAL_ATTRINFO_KEYCREATOR,
        DAL_ATTRINFO_KEYTYPE,
        DAL_ATTRINFO_KEYSIZEINBITS,
        DAL_ATTRINFO_EFFECTIVEKEYSIZE,
        DAL_ATTRINFO_STARTDATE,
        DAL_ATTRINFO_ENDDATE,
        DAL_ATTRINFO_SENSITIVE,
        DAL_ATTRINFO_ALWAYSSENSITIVE,
        DAL_ATTRINFO_EXTRACTABLE,
        DAL_ATTRINFO_NEVEREXTRACTABLE,
        DAL_ATTRINFO_ENCRYPT,
        DAL_ATTRINFO_DECRYPT,
        DAL_ATTRINFO_DERIVE,
        DAL_ATTRINFO_SIGN,
        DAL_ATTRINFO_VERIFY,
        DAL_ATTRINFO_SIGNRECOVER,
        DAL_ATTRINFO_VERIFYRECOVER,
        DAL_ATTRINFO_WRAP,
        DAL_ATTRINFO_UNWRAP
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_ALL_KEYS);
    pList->NumberOfAttributes = CSSM_DL_DB_RECORD_ALL_KEYS_NUM_ATTRIBUTES;
    pList->AttributeInfo = rgAttributes;
    ASSERT(sizeof(rgAttributes) / sizeof(CSSM_DB_ATTRIBUTE_INFO) == pList->NumberOfAttributes);
}

void dal_ALL_KEYS_GetIndexes(CSSM_DB_RECORD_INDEX_INFO *pList)
{
    static CSSM_DB_INDEX_INFO rgIndexes[] = {
        INDEX(DAL_ATTRINFO_KEYCLASS),
        INDEX(DAL_ATTRINFO_PRINTNAME),
        INDEX(DAL_ATTRINFO_ALIAS),
        INDEX(DAL_ATTRINFO_PERMANENT),
        INDEX(DAL_ATTRINFO_PRIVATE),
        INDEX(DAL_ATTRINFO_MODIFIABLE),
        INDEX(DAL_ATTRINFO_LABEL),
        INDEX(DAL_ATTRINFO_APPLICATIONTAG),
        INDEX(DAL_ATTRINFO_KEYCREATOR),
        INDEX(DAL_ATTRINFO_KEYTYPE),
        INDEX(DAL_ATTRINFO_KEYSIZEINBITS),
        INDEX(DAL_ATTRINFO_EFFECTIVEKEYSIZE),
        INDEX(DAL_ATTRINFO_STARTDATE),
        INDEX(DAL_ATTRINFO_ENDDATE),
        INDEX(DAL_ATTRINFO_SENSITIVE),
        INDEX(DAL_ATTRINFO_ALWAYSSENSITIVE),
        INDEX(DAL_ATTRINFO_EXTRACTABLE),
        INDEX(DAL_ATTRINFO_NEVEREXTRACTABLE),
        INDEX(DAL_ATTRINFO_ENCRYPT),
        INDEX(DAL_ATTRINFO_DECRYPT),
        INDEX(DAL_ATTRINFO_DERIVE),
        INDEX(DAL_ATTRINFO_SIGN),
        INDEX(DAL_ATTRINFO_VERIFY),
        INDEX(DAL_ATTRINFO_SIGNRECOVER),
        INDEX(DAL_ATTRINFO_VERIFYRECOVER),
        INDEX(DAL_ATTRINFO_WRAP),
        INDEX(DAL_ATTRINFO_UNWRAP)
    };

    ASSERT(pList->DataRecordType == CSSM_DL_DB_RECORD_ALL_KEYS);
    pList->NumberOfIndexes = CSSM_DL_DB_RECORD_ALL_KEYS_NUM_INDEXES;
    pList->IndexInfo = rgIndexes;
    ASSERT(sizeof(rgIndexes) / sizeof(CSSM_DB_INDEX_INFO) == pList->NumberOfIndexes);
}

/*-----------------------------------------------------------------------------
                    Functions for Schema Manipulation
-----------------------------------------------------------------------------*/

/*
 *  REPLACE the given user schema WITH our DEFAULT defined SCHEMA
 */
CSSM_RETURN dal_ReplaceWithDefaultSchema(CSSM_DB_RECORD_ATTRIBUTE_INFO *AttributeInfoPtr,
                                         CSSM_DB_RECORD_INDEX_INFO *IndexInfoPtr)
{
    /*  
     *  Check whether attributes or indexes have been given by user.
     *  If app. has passed these, then use the schema passed.
     *  Else, replace the user passed (app. passed) schema with our default one.
     */
    if ((AttributeInfoPtr->NumberOfAttributes == 0) && (IndexInfoPtr->NumberOfIndexes == 0))
    {
        switch(AttributeInfoPtr->DataRecordType)
        {
            case CSSM_DL_DB_RECORD_CERT:
                dal_CERT_GetAttributes(AttributeInfoPtr);
                dal_CERT_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_CRL:
                dal_CRL_GetAttributes(AttributeInfoPtr);
                dal_CRL_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_POLICY:
                dal_POLICY_GetAttributes(AttributeInfoPtr);
                dal_POLICY_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_GENERIC:
                dal_GENERIC_GetAttributes(AttributeInfoPtr);
                dal_GENERIC_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_PUBLIC_KEY:
                dal_PUBLIC_KEY_GetAttributes(AttributeInfoPtr);
                dal_PUBLIC_KEY_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_PRIVATE_KEY:
                dal_PRIVATE_KEY_GetAttributes(AttributeInfoPtr);
                dal_PRIVATE_KEY_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_SYMMETRIC_KEY:
                dal_SYMMETRIC_KEY_GetAttributes(AttributeInfoPtr);
                dal_SYMMETRIC_KEY_GetIndexes(IndexInfoPtr);
                break;
            case CSSM_DL_DB_RECORD_ALL_KEYS:
                dal_ALL_KEYS_GetAttributes(AttributeInfoPtr);
                dal_ALL_KEYS_GetIndexes(IndexInfoPtr);
                break;
            default:
                return (CSSM_FALSE);
                break;
        }
    }

    return (CSSM_OK);
}
