/*-----------------------------------------------------------------------
 * File: CRLSIGN.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include <stdlib.h>             /* NULL */

/* CDSA includes */
#include "cssm.h"               /* CSSM types, errors, and APIs */

/* local includes */
#include "api_shim.h"
#include "Chn_util.h"

/*---------------------------------------------------------------
 *
 * Name: TP_CrlSign
 *
 * Description:
 *   Sign the CRL given the inputs. Maps directly to a call to
 *   CSSM_CL_CrlSign().
 *
 * Parameters: 
 *   TPHandle (input)       - Handle of the TP module
 *   CLHandle (input)       - Handle of the CL module
 *   CCHandle (input)       - Handle of the cryptographic context
 *                            to be used to sign the CrlToBeSigned
 *   CrlToBeSigned (input)  - pointer to a structure including the 
 *                            CRL to be signed
 *   SignerCertGroup (input)
 *                          - pointer to the certificate group to 
 *                            use to sign the CrlToBeSigned
 *                            [unused - must be NULL]
 *   SignerVerifyContext (input)      
 *                          - pointer to data that describes the 
 *                            verification context
 *                            [unused - must be NULL]
 *   SignerVerifyResult (output)
 *                          - pointer to evidence associated with
 *                            signer context verification
 *                            [unused - must be NULL]
 *   SignedCrl (output)     - the Signed CRL
 *                           
 *
 * Returns:
 *   CSSM_OK                - CRL signed and returned in SignedCrl
 *   error code             - error detected
 *
 *-------------------------------------------------------------*/

CSSM_RETURN CSSMAPI 
TP_CrlSign( CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_ENCODED_CRL *CrlToBeSigned,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
                        CSSM_DATA_PTR SignedCrl )
{
    CSSM_RETURN Rtn = CSSM_OK;

    /* unused parameters */
    TPHandle;
    SignerCertGroup;
    SignerVerifyContext;
    SignerVerifyResult;

    if (CrlToBeSigned == NULL)
	{
        return CSSMERR_TP_INVALID_CRL_POINTER;
    }
	if (!tp_IsValidCrlType(CrlToBeSigned->CrlType))
	{
		return ( CSSMERR_TP_INVALID_CRL_TYPE );
	}
	if (!tp_IsValidCrlEncoding(CrlToBeSigned->CrlEncoding))
	{
		return ( CSSMERR_TP_INVALID_CRL_ENCODING );
	}

    /* --CALL CSSM_CL_CrlSign() AND RETURN THE VALUE-- */
    Rtn = addin_cl_CrlSign(CLHandle, CCHandle, 
                            &CrlToBeSigned->CrlBlob, 
                            NULL, 0, 
                            SignedCrl);

    return Rtn;
}
