/*-----------------------------------------------------------------------
 * File: CRTGRPCO.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

/* CSSM INCLUDES */
#include "cssm.h"
#include "cssmtpi.h"
#include "cssmspi.h"

/* LOCAL INCLUDES */
#include "inx509tp.h"           /* func and data definitions specific to this module */
#include "Chn_util.h"
#include "api_shim.h"
#include "maf_include.h"

#include <cssmMemTrack.h>

/*-----------------------------------------------------------------------------
 * Name: TP_CertGroupConstruct
 *
 * Description:
 *   This function constructs a cert chain ending in a self-signed root 
 *   cert from a fragment and a list of databases. 
 *
 * Parameters: 
 *   TPHandle (input):         handle of TP executing call
 *   CLHandle (input):         handle of CL to use with this call
 *   CSPHandle (input):        handle of CSP to use with this call
 *                             [unused - must be 0]
 *   CertGroupFrag (input):    the cert group to construct from
 *   DBList (input):           list of databases to be used to find certs in the
 *                             chain.  (Only these DB's will be accessed during
 *                             the construct.)
 *   
 * Return value:
 *   A pointer to the constructed cert group on success.
 *   NULL on failure
 *
 * Error Codes:
 *   CSSM_INVALID_ADDIN_HANDLE
 *   CSSM_TP_INVALID_CL_HANDLE
 *   CSSM_TP_INVALID_CSP_HANDLE
 *   CSSM_INVALID_CERTGROUP_PTR
 *
 *---------------------------------------------------------------------------*/

CSSM_RETURN CSSMAPI
TP_CertGroupConstruct(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        const void *ConstructParams,
                        const CSSM_CERTGROUP *CertGroupFrag,
                        CSSM_CERTGROUP_PTR *CertGroup )
{
    CSSM_RETURN RetCode = CSSM_OK;

/* --CHECK INPUTS-- */
    if (TPHandle == CSSM_INVALID_HANDLE)
    {
        return(CSSMERR_TP_INTERNAL_ERROR);
    }
    if (CLHandle == CSSM_INVALID_HANDLE)
    {
        return(CSSMERR_TP_INVALID_CL_HANDLE);
    }
    if (CertGroupFrag == NULL)
    {
        /* no frag, no starting point */
        return(CSSMERR_TP_INVALID_CERTGROUP_POINTER);
    }
    if (CertGroupFrag->NumCerts == 0)
    {
        /* no certs */
        return(CSSMERR_TP_INVALID_CERTGROUP_POINTER);
    }
    if(NULL == CertGroup)
    {
        return (CSSMERR_TP_INVALID_POINTER);
    }

    RetCode = tp_CertGroupConstruct(TPHandle, CLHandle, CSPHandle, 
                                    DBList, CertGroupFrag, CSSM_TRUE, CertGroup);

    return RetCode;
}
