/*-----------------------------------------------------------------------
 * File: MODINSTALL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include <sys/types.h>
#include <sys/stat.h>

#include "cssm.h"
#include "cssmerr.h"
#include "cssmport.h"
#include "cssmlock.h"
#include "maf_config.h"
#include "maf_collectn.h"
#include "maf_util.h"
#include "mds.h"
#include "mds_schema.h"
#include "mds_util.h"
#include "intel/installdefs.h"

/* Number of MDS update operations that will be remembered by mds_util for
 * error clean-up during installation.
 */
#define DUMMYMULTI_MDS_COMMIT_SIZE  500


CSSM_RETURN  FindRecordsAndClean( //CSSM_DB_RECORDTYPE RecordType, 
                                  CSSM_GUID module_guid,
                                  void *pPrimaryTemplate,
                                  void *pEncProdTemplate,
                                  MDSU_CONTEXT *pContext,
                                  const IfMdsuSchemaManipulator *pPrimaryManipulator,
                                  const IfMdsuSchemaManipulator *pEncProdManipulator,
                                  MDSU_TEMPLATE_MASK PrimaryMask,
                                  MDSU_TEMPLATE_MASK EncProdMask,
                                  void *pPrimaryData,
                                  void *pEncProdData)
{ 
    CSSM_RETURN rv = CSSM_OK;


    rv = MDSU_FindFirst( pContext,
                         pPrimaryManipulator,
                         pPrimaryTemplate,
                         PrimaryMask,
                         pPrimaryData,
                         NULL );
    while ( rv == CSSM_OK )
    {
        if ( ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimaryData)->Manifest.Data )
        {
            MDSU_free( pContext, ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimaryData)->Manifest.Data );
        }

        rv = MDSU_FindNext( pContext,
                            pPrimaryData,
                            NULL );
    }
    if ( rv == CSSMERR_DL_ENDOFDATA )
    {
        // Encapsulated Product Schema 
        ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProdTemplate)->ModuleID = ADDIN_GUID;
        rv = MDSU_FindFirst( pContext,
                             pEncProdManipulator,
                             pEncProdTemplate,
                             EncProdMask,
                             pEncProdData,
                             NULL );
        while ( rv == CSSM_OK )
        {
            rv = MDSU_FindNext( pContext,
                                pEncProdData,
                                NULL );
        }
        if ( rv == CSSMERR_DL_ENDOFDATA )
        {
            // Flush everything from the MDS 
            rv = MDSU_Revert( pContext );
        } // Encapsulated Product removed 
    } //  Primary removed 

    return rv;
}


static CSSM_RETURN __CleanMDS( MDSU_CONTEXT *pContext )
{
    CSSM_RETURN rv = CSSM_OK;
    MDSU_CDSA_COMMON_SCHEMA CommonTemplate;
    MDSU_CDSA_COMMON_SCHEMA CommonData;
    
    MDSU_CDSA_CSP_PRIMARY_SCHEMA CSPPrimaryTemplate;
    MDSU_CDSA_CSP_PRIMARY_SCHEMA CSPPrimaryData;
    MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA CSPEncProdTemplate;
    MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA CSPEncProdData;
    
    MDSU_CDSA_CL_PRIMARY_SCHEMA CLPrimaryTemplate;
    MDSU_CDSA_CL_PRIMARY_SCHEMA CLPrimaryData;
    MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA CLEncProdTemplate;
    MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA CLEncProdData;
    
    MDSU_CDSA_DL_PRIMARY_SCHEMA DLPrimaryTemplate;
    MDSU_CDSA_DL_PRIMARY_SCHEMA DLPrimaryData;
    MDSU_CDSA_DL_ENC_PRODUCT_SCHEMA DLEncProdTemplate;
    MDSU_CDSA_DL_ENC_PRODUCT_SCHEMA DLEncProdData;
    
    MDSU_CDSA_TP_PRIMARY_SCHEMA TPPrimaryTemplate;
    MDSU_CDSA_TP_PRIMARY_SCHEMA TPPrimaryData;
    MDSU_CDSA_TP_ENC_PRODUCT_SCHEMA TPEncProdTemplate;
    MDSU_CDSA_TP_ENC_PRODUCT_SCHEMA TPEncProdData;
   
    MDSU_CDSA_AC_PRIMARY_SCHEMA ACPrimaryTemplate;
    MDSU_CDSA_AC_PRIMARY_SCHEMA ACPrimaryData;
    
    /* Execute a search on every Dummymulti record type to get a handle of all
    * records and then delete them.
    */
    /* Common Schema */
    CommonTemplate.ModuleID = ADDIN_GUID;
    rv = MDSU_FindFirst( pContext,
        &IfiMdsuCdsadirCommonSchema,
        &CommonTemplate,
        MDSU_COM_MODULE_ID,
        &CommonData,
        NULL );
    while ( rv == CSSM_OK )
    {
        if ( CommonData.Manifest.Data )
        {
            MDSU_free( pContext, CommonData.Manifest.Data );
        }
        
        rv = MDSU_FindNext( pContext,
            &CommonData,
            NULL );
    }
    if ( rv == CSSMERR_DL_ENDOFDATA )
    {
        // AC Primary Schema 
        ACPrimaryTemplate.ModuleID = ADDIN_GUID;
        rv = MDSU_FindFirst( pContext,
                             &IfiMdsuCdsadirACPrimarySchema,
                             &ACPrimaryTemplate,
                             MDSU_ACP_MODULE_ID,
                             &ACPrimaryData,
                             NULL );
        while ( rv == CSSM_OK )
        {
            if ( ACPrimaryData.Manifest.Data )
            {
                MDSU_free( pContext, ACPrimaryData.Manifest.Data );
            }
            
            rv = MDSU_FindNext( pContext,
                &ACPrimaryData,
                NULL );
        }
        if ( rv == CSSMERR_DL_ENDOFDATA )
        {
            // Flush AC primary records from the MDS 
            rv = MDSU_Revert( pContext );
        } // AC Primary removed 

        if ( rv == CSSM_OK )
        {
            CSPPrimaryTemplate.ModuleID = ADDIN_GUID;        
            rv = FindRecordsAndClean( //MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE,
                ADDIN_GUID,
                &CSPPrimaryTemplate,
                &CSPEncProdTemplate,
                pContext,
                &IfiMdsuCdsadirCSPPrimarySchema,
                &IfiMdsuCdsadirCSPEncProductSchema,
                MDSU_CSPP_MODULE_ID,
                MDSU_CSPEP_MODULE_ID,
                &CSPPrimaryData,
                &CSPEncProdData);
            if ( rv == CSSM_OK )
            {
				DLPrimaryTemplate.ModuleID = ADDIN_GUID;        

                rv = FindRecordsAndClean( //MDS_CDSADIR_DL_PRIMARY_RECORDTYPE,
                    ADDIN_GUID,
                    &DLPrimaryTemplate,
                    &DLEncProdTemplate,
                    pContext,
                    &IfiMdsuCdsadirDLPrimarySchema,
                    &IfiMdsuCdsadirDLEncProductSchema,
                    MDSU_DLP_MODULE_ID,
                    MDSU_DLEP_MODULE_ID,
                    &DLPrimaryData,
                    &DLEncProdData);
                if ( rv == CSSM_OK )
                {
					CLPrimaryTemplate.ModuleID = ADDIN_GUID;        

                    rv = FindRecordsAndClean( //MDS_CDSADIR_CL_PRIMARY_RECORDTYPE,
                        ADDIN_GUID,
                        &CLPrimaryTemplate,
                        &CLEncProdTemplate,
                        pContext,
                        &IfiMdsuCdsadirCLPrimarySchema,
                        &IfiMdsuCdsadirCLEncProductSchema,
                        MDSU_CLP_MODULE_ID,
                        MDSU_CLEP_MODULE_ID,
                        &CLPrimaryData,
                        &CLEncProdData);
                    if ( rv == CSSM_OK )
                    {
						TPPrimaryTemplate.ModuleID = ADDIN_GUID;        

                        rv = FindRecordsAndClean( //MDS_CDSADIR_TP_PRIMARY_RECORDTYPE,
                            ADDIN_GUID,
                            &TPPrimaryTemplate,
                            &TPEncProdTemplate,
                            pContext,
                            &IfiMdsuCdsadirTPPrimarySchema,
                            &IfiMdsuCdsadirTPEncProductSchema,
                            MDSU_TPP_MODULE_ID,
                            MDSU_TPEP_MODULE_ID,
                            &TPPrimaryData,
                            &TPEncProdData);
                    }
                }
            }
        }
    }
    // Make sure nothing gets removed if there is an error 
    if ( ( rv != CSSMERR_DL_ENDOFDATA ) &&
         ( rv != CSSM_OK ) ) 
    {
        MDSU_Commit( pContext );
    }
    return rv;
}


void FillPrimaryData ( void *pPrimary,
                       CSSM_GUID addin_guid,
                       uint32 addin_ssid,
                       const char *szModuleName )
{
    /* Fill in the Primary Schema data for all common items */
    memcpy( &((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->ModuleID, &addin_guid, sizeof(addin_guid) );
    ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->SSID = addin_ssid;
    ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->Manifest.Data = NULL;
    ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->Manifest.Length = 0;
    strcpy( ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->ModuleName, szModuleName );
    ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->ProductVersion.Major = ADDIN_PRODUCT_VERSION_MAJOR;
    ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->ProductVersion.Minor = ADDIN_PRODUCT_VERSION_MINOR;
    strcpy( ((MDSU_CDSA_CSP_PRIMARY_SCHEMA *)pPrimary)->Vendor, ADDIN_VENDOR_NAME );
}

void FillEncProdData ( void *pEncProd,
                       CSSM_GUID addin_guid,
                       uint32 addin_ssid,
                       const char *szProdDesc )
{
    /* Fill in the Encapsulated Product Schema data for all common items */
    memcpy( &((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ModuleID, &addin_guid, sizeof(addin_guid) );
    ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->SSID = addin_ssid;
    strcpy( ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ProductVendor, ADDIN_VENDOR_NAME );
    ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ProductVersion.Major = ADDIN_PRODUCT_VERSION_MAJOR;
    ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ProductVersion.Minor = ADDIN_PRODUCT_VERSION_MINOR;
    strcpy( ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ProductDesc, szProdDesc );
    ((MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA *)pEncProd)->ProductFlags = 0;
}

CSSM_RETURN UpdateMDS( MDSU_CONTEXT *pContext,
                       const IfMdsuSchemaManipulator *pManipulator,
                       void *pTemplate,
                       INSTALL_ERROR *pError)
{
    CSSM_RETURN rv = CSSM_OK;

    /* Write the data to MDS */
    rv = MDSU_UpdateSchema( pContext,
                            pManipulator,
                            pTemplate,
                            NULL );
    if ( rv != CSSM_OK )
    {
        sprintf( pError->ErrorString, "MDS Error : %X\n", rv );
        pError->ErrorCode = rv;
        MDSU_Revert( pContext );
        MDSU_Term( pContext );
    }
    return rv;
}

CSSM_RETURN CSSMAPI INSTALL_ENTRY_NAME(
                        const char *szModuleName,
                        const char *szSearchPath,
                        CSSM_DATA *pCredential,
                        INSTALL_ACTION Action,
                        INSTALL_ERROR *pError )
{
    CSSM_RETURN rv = CSSM_OK;
    MDSU_CONTEXT MDSUContext;
    MDSU_CDSA_COMMON_SCHEMA Common;
    MDSU_CDSA_CSP_PRIMARY_SCHEMA CSPPrimary;
    MDSU_CDSA_CSP_ENC_PRODUCT_SCHEMA CSPEncProduct;

    MDSU_CDSA_DL_PRIMARY_SCHEMA DLPrimary;
    MDSU_CDSA_DL_ENC_PRODUCT_SCHEMA DLEncProduct;

    MDSU_CDSA_CL_PRIMARY_SCHEMA CLPrimary;
    MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA CLEncProduct;

    MDSU_CDSA_TP_PRIMARY_SCHEMA TPPrimary;
    MDSU_CDSA_TP_ENC_PRODUCT_SCHEMA TPEncProduct;

    MDSU_CDSA_AC_PRIMARY_SCHEMA ACPrimary;


    /* Initialize the MDS update */
    if ( ( rv = MDSU_Init( &MDSUContext, 
                           &Addin_APIMemFuncs,
                           &ADDIN_GUID,
                           MDS_CDSA_DIRECTORY,
                           ( CSSM_DB_ACCESS_READ | 
                             CSSM_DB_ACCESS_WRITE ),
                           DUMMYMULTI_MDS_COMMIT_SIZE ) ) != CSSM_OK )
    {
        sprintf( pError->ErrorString, "MDS Error (Init): %X\n", rv );
        pError->ErrorCode = rv;
        return rv;
    }

    if ( ( rv = __CleanMDS( &MDSUContext ) ) != CSSM_OK )
    {
        sprintf( pError->ErrorString, "MDS Error (Clean): %X\n", rv );
        pError->ErrorCode = rv;
        MDSU_Term( &MDSUContext );
        return rv;
    }

    if ( Action == INSTALL_ACTION_UNINSTALL )
    {
        MDSU_Term( &MDSUContext );
        return rv;
    }
    /* Fill in the "easy" common schema structure */
    memcpy( &Common.ModuleID, &ADDIN_GUID, sizeof(ADDIN_GUID) );
    Common.Manifest = *pCredential;
    strcpy( Common.ModuleName, szModuleName );
    strcpy( Common.Path, szSearchPath );
    Common.CDSAVersion.Major = CSSM_MAJOR;
    Common.CDSAVersion.Minor = CSSM_MINOR;
    strcpy( Common.Desc, ADDIN_PRINT_NAME );
    Common.DynamicFlag = CSSM_FALSE;
    Common.MultiThreadFlag = CSSM_FALSE;
    Common.ServiceMask = CSSM_SERVICE_CSP|CSSM_SERVICE_CL|CSSM_SERVICE_DL|CSSM_SERVICE_TP|CSSM_SERVICE_AC;

    /* Insert the common schema record into the MDS */
    rv = MDSU_UpdateSchema( &MDSUContext,
                            &IfiMdsuCdsadirCommonSchema,
                            &Common,
                            NULL );
    if ( rv != CSSM_OK )
    {
        sprintf( pError->ErrorString, "MDS Error (common): %X\n", rv );
        pError->ErrorCode = rv;
        MDSU_Term( &MDSUContext );
        return rv;
    }

    /* Filling in first 6 fields of primary templates for CSP, CL, DL, TP, AC */
    FillPrimaryData ( &CSPPrimary, ADDIN_GUID, 0, szModuleName);
    FillPrimaryData ( &DLPrimary, ADDIN_GUID, 0, szModuleName);
    FillPrimaryData ( &CLPrimary, ADDIN_GUID, 0, szModuleName);
    FillPrimaryData ( &TPPrimary, ADDIN_GUID, 0, szModuleName);
    FillPrimaryData ( &ACPrimary, ADDIN_GUID, 0, szModuleName);

    /* Filling in first 6 fields of Encapsulated Product templates for CSP, CL, DL, TP */
    /* AC does not have an Encapsulated Product ! */
    FillEncProdData ( &CSPEncProduct, ADDIN_GUID, 0, "CSP for Dummy Multi-Service Module");
    FillEncProdData ( &DLEncProduct, ADDIN_GUID, 0, "DL for Dummy Multi-Service Module");
    FillEncProdData ( &CLEncProduct, ADDIN_GUID, 0, "CL Encoder for Dummy Multi-Service Module");
    FillEncProdData ( &TPEncProduct, ADDIN_GUID, 0, "TP for Dummy Multi-Service Module");

/********************* AC ************************/
    /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirACPrimarySchema,
                    &ACPrimary,
                    pError );

    // Commit the records
    MDSU_Commit( &MDSUContext );

/******************** CSP ************************/
    CSPPrimary.CspType = CSSM_CSP_SOFTWARE;
    CSPPrimary.CspFlags = 0;
    CSPPrimary.CspCustomFlags = 0;
    CSPPrimary.UseeTags[0] = CSSM_USEE_NONE;
    CSPPrimary.NumUseeTags = 1;
    memset(CSPPrimary.SampleTypes, 0,
                sizeof(CSSM_SAMPLE_TYPE)*MDSU_MAX_VALUE_COUNT); 
    CSPPrimary.NumSampleTypes = 0;
    memset(CSPPrimary.AclSubjectTypes, 0, 
                  sizeof(CSSM_ACL_SUBJECT_TYPE)*MDSU_MAX_VALUE_COUNT);
    CSPPrimary.NumAclSubjectTypes = 0;
    memset(CSPPrimary.AclAuthTags, 0,
                sizeof(CSSM_ACL_AUTHORIZATION_TAG)*MDSU_MAX_VALUE_COUNT);
    CSPPrimary.NumAclAuthTags = 0;    

    /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirCSPPrimarySchema,
                    &CSPPrimary,
                    pError );

    /* Fill in the CSP encapsulated product schema data */
    strcpy( CSPEncProduct.StandardDesc, ADDIN_STANDARD_DESCRIPTION );
    CSPEncProduct.StandardVersion.Major = ADDIN_STANDARD_VERSION_MAJOR;
    CSPEncProduct.StandardVersion.Minor = ADDIN_STANDARD_VERSION_MINOR;

    CSPEncProduct.CustomFlags = 0;
    strcpy( CSPEncProduct.ReaderDesc, "" );
    strcpy( CSPEncProduct.ReaderVendor, "" );
    CSPEncProduct.ReaderVersion.Major = 0;
    CSPEncProduct.ReaderVersion.Minor = 0;
    CSPEncProduct.ReaderFirmwareVersion.Major = 0;
    CSPEncProduct.ReaderFirmwareVersion.Minor = 0;
    CSPEncProduct.ReaderFlags = 0;
    CSPEncProduct.ReaderCustomFlags = 0;
    strcpy( CSPEncProduct.ReaderSerialNumber, "" );

    /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirCSPEncProductSchema,
                    &CSPEncProduct,
                    pError );

    /* Commit the records */
    MDSU_Commit( &MDSUContext );
/**************** DL *******************/
    DLPrimary.DLType = CSSM_DL_UNKNOWN;
    DLPrimary.QueryLimitsFlag = CSSM_FALSE;
    memset(DLPrimary.SampleTypes, 0,
                sizeof(CSSM_SAMPLE_TYPE)*MDSU_MAX_VALUE_COUNT); 
    DLPrimary.NumSampleTypes = 0;
    memset(DLPrimary.AclSubjectTypes, 0, 
                  sizeof(CSSM_ACL_SUBJECT_TYPE)*MDSU_MAX_VALUE_COUNT);
    DLPrimary.NumAclSubjectTypes = 0;
    memset(DLPrimary.AclAuthTags, 0,
                sizeof(CSSM_ACL_AUTHORIZATION_TAG)*MDSU_MAX_VALUE_COUNT);
    DLPrimary.NumAclAuthTags = 0;    
    memset(DLPrimary.ConjunctiveOps, 0,
                sizeof(CSSM_DB_CONJUNCTIVE)*MDSU_MAX_VALUE_COUNT);
    DLPrimary.NumConjunctiveOps = 0;
    memset(DLPrimary.RelationalOps, 0,
                sizeof(CSSM_DB_OPERATOR)*MDSU_MAX_VALUE_COUNT);
    DLPrimary.NumRelationalOps = 0;

    // Write the data to MDS 
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirDLPrimarySchema,
                    &DLPrimary,
                    pError );
    strcpy( DLEncProduct.StandardDesc, ADDIN_STANDARD_DESCRIPTION );
    DLEncProduct.StandardVersion.Major = ADDIN_STANDARD_VERSION_MAJOR;
    DLEncProduct.StandardVersion.Minor = ADDIN_STANDARD_VERSION_MINOR;

    DLEncProduct.Protocol = CSSM_NET_PROTO_NONE;
    DLEncProduct.RetrievalMode = 0;

    // Write the data to MDS 
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirDLEncProductSchema,
                    &DLEncProduct,
                    pError );

    // Commit the records
    MDSU_Commit( &MDSUContext );
/****************** CL *******************/
    //CertTypeFormat = CSSM_CERT_X_509v3 << 16;
    CLPrimary.CertTypeFormat = 0; //CertTypeFormat | CSSM_CERT_ENCODING_DER;

    //CrlTypeFormat = CSSM_CRL_TYPE_X_509v2 << 16;
    CLPrimary.CrlTypeFormat = 0;  //CrlTypeFormat | CSSM_CRL_ENCODING_DER;

    CLPrimary.CertFieldNames.Data = NULL;
    CLPrimary.CertFieldNames.Length = 0;

    memset(CLPrimary.BundleTypeFormat, 0, 
                  sizeof(CSSM_CERT_BUNDLE_TYPE)*MDSU_MAX_VALUE_COUNT);
    //BundleTypeFormat = CSSM_CERT_BUNDLE_PKCS7_SIGNED_DATA << 16;
    //CLPrimary.BundleTypeFormat[0] = BundleTypeFormat | CSSM_CERT_BUNDLE_ENCODING_DER;
    CLPrimary.NumBundleTypeFormat = 0;

    memset(CLPrimary.XlationTypeFormat, 0, 
                  sizeof(uint32)*MDSU_MAX_VALUE_COUNT);
    CLPrimary.NumXlationTypeFormat = 0;

    CLPrimary.DefaultTemplateType = 0;

    CLPrimary.TemplateFieldNames.Data = NULL;
    CLPrimary.TemplateFieldNames.Length = 0;

    /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirCLPrimarySchema,
                    &CLPrimary,
                    pError );

    strcpy( CLEncProduct.StandardDesc, ADDIN_STANDARD_DESCRIPTION );
    CLEncProduct.StandardVersion.Major = ADDIN_STANDARD_VERSION_MAJOR;
    CLEncProduct.StandardVersion.Minor = ADDIN_STANDARD_VERSION_MINOR;

    // Write the data to MDS 
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirCLEncProductSchema,
                    &CLEncProduct,
                    pError );

    MDSU_Commit( &MDSUContext );

/********************* TP *******************/
    TPPrimary.CertTypeFormat =  0;
    memset(TPPrimary.SampleTypes, 0,
                sizeof(CSSM_SAMPLE_TYPE)*MDSU_MAX_VALUE_COUNT); 
    TPPrimary.NumSampleTypes = 0;
    memset(TPPrimary.AclSubjectTypes, 0, 
                  sizeof(CSSM_ACL_SUBJECT_TYPE)*MDSU_MAX_VALUE_COUNT);
    TPPrimary.NumAclSubjectTypes = 0;
    memset(TPPrimary.AclAuthTags, 0,
                sizeof(CSSM_ACL_AUTHORIZATION_TAG)*MDSU_MAX_VALUE_COUNT);
    TPPrimary.NumAclAuthTags = 0;    

        /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirTPPrimarySchema,
                    &TPPrimary,
                    pError );

    strcpy( TPEncProduct.StandardDesc, ADDIN_STANDARD_DESCRIPTION );
    TPEncProduct.StandardVersion.Major = ADDIN_STANDARD_VERSION_MAJOR;
    TPEncProduct.StandardVersion.Minor = ADDIN_STANDARD_VERSION_MINOR;

    memset(TPEncProduct.AuthorityRequestType, 0, 
                  sizeof(CSSM_TP_AUTHORITY_REQUEST_TYPE)*MDSU_MAX_VALUE_COUNT);
    TPEncProduct.NumAuthorityRequestType = 0;
    strcpy( TPEncProduct.ProtocolDesc, "" );
    TPEncProduct.ProtocolFlags = CSSM_NET_PROTO_NONE;
    strcpy( TPEncProduct.CertClassName, "" );
    TPEncProduct.RootCertificate.Data = NULL;
    TPEncProduct.RootCertificate.Length = 0;
    TPEncProduct.RootCertTypeFormat = CSSM_CERT_UNKNOWN;

        /* Write the data to MDS */
    rv = UpdateMDS( &MDSUContext,
                    &IfiMdsuCdsadirTPEncProductSchema,
                    &TPEncProduct,
                    pError );

    /* Commit the records */
    MDSU_Commit( &MDSUContext );
    /* Shut down the MDS connection */
    MDSU_Term( &MDSUContext );

    return CSSM_OK;
}
