/*-----------------------------------------------------------------------
 * File: hrsdynamicschema.c
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------

/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */


#include "precomp.h"
#include "hrsaddin_schema.h"

#define _PRINTABLE_GUID_LENGTH     ( 40 )
#define _PRINTABLE_VERSION_LENGTH  ( 8 )

#define __ModuleId			( 0 )
#define __SSId              ( 1 )
#define __BspName			( 2 )
#define __SupportedFormats  ( 3 )
#define __SupportedEvents   ( 4 )
#define __DeviceVendor      ( 5 )
#define __DeviceDescription ( 6 )
#define __DeviceSerialNumber ( 7 )
#define __DeviceHWVersion   ( 8 )
#define __DeviceFirmwareVersion  ( 9 )
#define __AuthenticatedDevice    ( 10 )

typedef struct hrs_bsp_dynamic_attr_state
{
    CSSM_DB_ATTRIBUTE_DATA DbRecordAttributes[MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES];
    CSSM_DATA AttributeValues[MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_PRINTABLE_GUID_LENGTH];
    uint8 PrintableDeviceHWVersion[_PRINTABLE_VERSION_LENGTH];
    uint8 PrintableDeviceFirmwareVersion[_PRINTABLE_VERSION_LENGTH];
}
HRS_BSP_DYNAMIC_ATTR_STATE;

static CSSM_RETURN CSSMAPI ConstructHrsBspDynamicSchemaAttributes(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaData,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFillValues,
                            void **ppAttrState )
{
    CSSM_RETURN rv = CSSM_OK;
    const CDSA_HRS_BSP_DYNAMIC_SCHEMA *pHrsBspDynamic = 
                            (const CDSA_HRS_BSP_DYNAMIC_SCHEMA*)pSchemaData;
    HRS_BSP_DYNAMIC_ATTR_STATE *pState = NULL;
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( pContext && DbRecord && ppAttrState );
    assert( ( bFillValues == CSSM_FALSE ) ||
            ( ( bFillValues == CSSM_TRUE ) && pSchemaData ) );

    /* Allocate the memory for the attributes */
    pState = MDSU_malloc( pContext, sizeof(HRS_BSP_DYNAMIC_ATTR_STATE) );
    if ( pState == NULL )
    {
        ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
    }
    else
    {
        /* Clear the structure */
        memset( pState, 0, sizeof(HRS_BSP_DYNAMIC_ATTR_STATE) );
        DbRecordAttributes = pState->DbRecordAttributes;

        /* Fill in the record information */
        DbRecord->DataRecordType = MDS_CDSADIR_HRS_BSP_DYNAMIC_RECORDTYPE;
        DbRecord->SemanticInformation = 0;
        DbRecord->NumberOfAttributes = MDS_CDSADIR_HRS_BSP_DYNAMIC_NUM_ATTRIBUTES;
        DbRecord->AttributeData = DbRecordAttributes;


        /* Fill in attribute info */
        DbRecordAttributes[__ModuleId].Info = s_MdsAttrInfo_Hrs_ModuleID;
        DbRecordAttributes[__SSId].Info = s_MdsAttrInfo_Hrs_SSID;
        DbRecordAttributes[__BspName].Info = s_MdsAttrInfo_Hrs_BspName;
        DbRecordAttributes[__SupportedFormats].Info = s_MdsAttrInfo_Hrs_SupportedFormats;
        DbRecordAttributes[__SupportedEvents].Info = s_MdsAttrInfo_Hrs_SupportedEvents;
        DbRecordAttributes[__DeviceVendor].Info = s_MdsAttrInfo_Hrs_DeviceVendor;
        DbRecordAttributes[__DeviceDescription].Info = s_MdsAttrInfo_Hrs_DeviceDescription;
        DbRecordAttributes[__DeviceSerialNumber].Info = s_MdsAttrInfo_Hrs_DeviceSerialNumber;
        DbRecordAttributes[__DeviceHWVersion].Info = s_MdsAttrInfo_Hrs_DeviceHWVersion;
        DbRecordAttributes[__DeviceFirmwareVersion].Info = s_MdsAttrInfo_Hrs_DeviceFirmwareVersion;
        DbRecordAttributes[__AuthenticatedDevice].Info = s_MdsAttrInfo_Hrs_AuthenticatedDevice;

        if ( bFillValues == CSSM_TRUE )
        {
            /* Connect the attribute value array to the attribute array */
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = &pState->AttributeValues[i];
                DbRecordAttributes[i].NumberOfValues = 1;
            }

            /* Fill in the record contents */
            /* ModuleID */
            _MDSU_GetPrintableGUID( &pHrsBspDynamic->ModuleID,
                                    (char*)pState->PrintableGuid );
            DbRecordAttributes[__ModuleId].Value->Data = 
                                                    pState->PrintableGuid;
            DbRecordAttributes[__ModuleId].Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;

            /* SSID */
            DbRecordAttributes[__SSId].Value->Length = sizeof(uint32);
            DbRecordAttributes[__SSId].Value->Data = (uint8*)&pHrsBspDynamic->SSID;

            /* BspName */
            DbRecordAttributes[__BspName].Value->Length = 
                                        strlen( pHrsBspDynamic->BSPName ) + 1 ;
            DbRecordAttributes[__BspName].Value->Data =
                                                (uint8*)pHrsBspDynamic->BSPName;



            /* SupportedFormats */
            DbRecordAttributes[__SupportedFormats].Value->Length =
                                            sizeof(uint32) * pHrsBspDynamic->NumSupportedformats;
            DbRecordAttributes[__SupportedFormats].Value->Data = (uint8*)&pHrsBspDynamic->SupportedFormats;

            /* SupportedEvents */
            DbRecordAttributes[__SupportedEvents].Value->Length =
                                            sizeof(uint32);
            DbRecordAttributes[__SupportedEvents].Value->Data = (uint8*)&pHrsBspDynamic->SupportedEvents;

            /* DeviceVendor */
            DbRecordAttributes[__DeviceVendor].Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceVendor ) + 1 ;
            DbRecordAttributes[__DeviceVendor].Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceVendor;

            /* DeviceDescription */
            DbRecordAttributes[__DeviceDescription].Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceDescription ) + 1 ;
            DbRecordAttributes[__DeviceDescription].Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceDescription;

            /* DeviceSerialNumber */
            DbRecordAttributes[__DeviceSerialNumber].Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceSerialNumber ) + 1 ;
            DbRecordAttributes[__DeviceSerialNumber].Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceSerialNumber;

            /* DeviceHWVersion */
            _MDSU_GetPrintableVersion( &pHrsBspDynamic->DeviceHWVersion,
                                       (char*)pState->PrintableDeviceHWVersion );
            DbRecordAttributes[__DeviceHWVersion].Value->Data = 
                                                    pState->PrintableDeviceHWVersion;
            DbRecordAttributes[__DeviceHWVersion].Value->Length = 
                                strlen( (char*)pState->PrintableDeviceHWVersion ) + 1;

            /* DeviceFirmwareVersion */
            _MDSU_GetPrintableVersion( &pHrsBspDynamic->DeviceFirmwareVersion,
                                       (char*)pState->PrintableDeviceFirmwareVersion );
            DbRecordAttributes[__DeviceFirmwareVersion].Value->Data = 
                                                    pState->PrintableDeviceFirmwareVersion;
            DbRecordAttributes[__DeviceFirmwareVersion].Value->Length = 
                                strlen( (char*)pState->PrintableDeviceFirmwareVersion ) + 1;


            /* AuthenticatedDevice */
            DbRecordAttributes[__AuthenticatedDevice].Value->Length = sizeof(uint32);
            DbRecordAttributes[__AuthenticatedDevice].Value->Data = (uint8*)&pHrsBspDynamic->AuthenticatedDevice;



        } /* bFillValues */
        else
        {
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = NULL;
                DbRecordAttributes[i].NumberOfValues = 0;
            }
        }

        if ( rv != CSSM_OK )
        {
            /* Deallocate memory if there was an error */
            MDSU_free( pContext,
                       pState );
            pState = NULL;
        }
    } /* allocate DbRecordAttributes */

    *ppAttrState = (void*)pState;

    return rv;
}

static CSSM_RETURN CSSMAPI DestructHrsBspDynamicSchemaAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFreeAll,
                            void *pAttrState )
{
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( pContext && DbRecord && pAttrState );

    /* Free the buffers allocated for the attribute values */
    DbRecordAttributes = DbRecord->AttributeData;
    if ( bFreeAll == CSSM_TRUE )
    {
        for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
        {
            if ( DbRecordAttributes[i].Value )
            {
                if ( DbRecordAttributes[i].Value->Data )
                {
                    MDSU_free( pContext,
                               DbRecordAttributes[i].Value->Data );
                }
                MDSU_free( pContext,
                           DbRecordAttributes[i].Value );
            }
        }
    }

    /* Free the state structure */
    MDSU_free( pContext,
               pAttrState );
    DbRecord->AttributeData = NULL;

    return CSSM_OK;
}


typedef struct hrs_bsp_capabilties_pred_state
{
    CSSM_SELECTION_PREDICATE Predicates[MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES];
    CSSM_DATA PredicateValues[MDS_CDSADIR_HRS_BSP_CAPABILITY_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_PRINTABLE_GUID_LENGTH];
    uint8 PrintableDeviceHWVersion[_PRINTABLE_VERSION_LENGTH];
    uint8 PrintableDeviceFirmwareVersion[_PRINTABLE_VERSION_LENGTH];
}
HRS_BSP_DYNAMIC_PRED_STATE;



static CSSM_RETURN CSSMAPI ConstructHrsBspDynamicPredicate(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaTemplate,
                            MDSU_TEMPLATE_MASK ValidTemplates,
                            CSSM_QUERY *pQuery,
                            void **ppPredState )
{
    CSSM_RETURN rv = CSSM_OK;
    const CDSA_HRS_BSP_DYNAMIC_SCHEMA *pHrsBspDynamic= 
                            (const CDSA_HRS_BSP_DYNAMIC_SCHEMA*)pSchemaTemplate;
    HRS_BSP_DYNAMIC_PRED_STATE *pState = NULL;
    CSSM_SELECTION_PREDICATE *pPredicates = NULL;
    uint32 uNumPredicates;
    uint32 uIndex = 0;
    uint32 i;

    assert( pContext && 
            pQuery &&
            ppPredState &&
            ( ( pSchemaTemplate && ValidTemplates ) ||
              ( !pSchemaTemplate && !ValidTemplates ) ) );

    /* Count the predicates */
    _CountBits( ValidTemplates, uNumPredicates );

    /* Allocate the predicate state and link the value structures */
    if ( uNumPredicates != 0 )
    {
        pState = MDSU_malloc( pContext, sizeof(HRS_BSP_DYNAMIC_PRED_STATE) );
        if ( pState == NULL )
        {
            ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
        }
        else
        {
            memset( pState,
                    0,
                    sizeof(HRS_BSP_DYNAMIC_PRED_STATE) );
            pPredicates = pState->Predicates;

            /* Allocate a single attribute value struct per attribute */
            for ( i = 0; i < uNumPredicates; i++ )
            {
                pPredicates[i].Attribute.Value = &pState->PredicateValues[i];
                pPredicates[i].Attribute.NumberOfValues = 1;
            }
        } /* Allocate and link predicate values */
    } /* uNumPredicates != 0 */

    if ( rv == CSSM_OK )
    {
        /* Fill in the query parameters */
        pQuery->RecordType = MDS_CDSADIR_HRS_BSP_DYNAMIC_RECORDTYPE;
        pQuery->Conjunctive = CSSM_DB_AND;
        pQuery->NumSelectionPredicates = uNumPredicates;
        pQuery->QueryLimits.TimeLimit = CSSM_QUERY_TIMELIMIT_NONE;
        pQuery->QueryLimits.SizeLimit = CSSM_QUERY_SIZELIMIT_NONE;
        pQuery->QueryFlags = 0;
        pQuery->SelectionPredicate = pPredicates;
        for ( uIndex = 0; uIndex < uNumPredicates; uIndex++ )
        {
            pPredicates[uIndex].DbOperator = CSSM_DB_EQUAL;
        }


        /* Fill in the predicate data */
        uIndex = 0;
        if ( ValidTemplates & MDSU_HRS_DYN_MODULE_ID )
        {
            /* ModuleID */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_ModuleID;
            _MDSU_GetPrintableGUID( &pHrsBspDynamic->ModuleID,
                                    (char*)pState->PrintableGuid );
            pPredicates[uIndex].Attribute.Value->Data = pState->PrintableGuid;
            pPredicates[uIndex].Attribute.Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_SSID )
        {
            /* SSID */
            pPredicates[uIndex].Attribute.Info = 
                                                s_MdsAttrInfo_Hrs_SSID;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(uint32);
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pHrsBspDynamic->SSID;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_BSP_NAME )
        {
            /* ModuleName */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_BspName;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pHrsBspDynamic->BSPName ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pHrsBspDynamic->BSPName;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_SUPPORTEDFORMATS )
        {
            /* SampleTypes */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_SupportedFormats;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        sizeof(uint32) * pHrsBspDynamic->NumSupportedformats;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pHrsBspDynamic->SupportedFormats;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_SUPPORTEDEVENTS )
        {
            /* SSID */
            pPredicates[uIndex].Attribute.Info = 
                                                s_MdsAttrInfo_Hrs_SupportedEvents;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(uint32);
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pHrsBspDynamic->SupportedEvents;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_HRS_DYN_DEVICEVENDOR )
        {
            /* Vendor */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_DeviceVendor;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceVendor ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceVendor;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_HRS_DYN_DEVICEDESCRIPTION )
        {
            /* Vendor */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_DeviceDescription;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceDescription ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceDescription;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_DEVICESERALNUMBER )
        {
            /* Vendor */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_DeviceSerialNumber;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pHrsBspDynamic->DeviceSerialNumber ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pHrsBspDynamic->DeviceSerialNumber;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_HRS_DYN_DEVICEHWVERSION )
        {
            /* Version */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_DeviceHWVersion;
            _MDSU_GetPrintableVersion( &pHrsBspDynamic->DeviceHWVersion,
                                       (char*)pState->PrintableDeviceHWVersion );
            pPredicates[uIndex].Attribute.Value->Data =
                                                pState->PrintableDeviceHWVersion;
            pPredicates[uIndex].Attribute.Value->Length = 
                            strlen( (char*)pState->PrintableDeviceHWVersion ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_DEVICEFIRMWAREVERSION )
        {
            /* Version */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Hrs_DeviceFirmwareVersion;
            _MDSU_GetPrintableVersion( &pHrsBspDynamic->DeviceFirmwareVersion ,
                                       (char*)pState->PrintableDeviceFirmwareVersion );
            pPredicates[uIndex].Attribute.Value->Data =
                                                pState->PrintableDeviceFirmwareVersion;
            pPredicates[uIndex].Attribute.Value->Length = 
                            strlen( (char*)pState->PrintableDeviceFirmwareVersion ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_HRS_DYN_AUTHENTICATEDDEVICE )
        {
            /* AuthenticatedDevice */
            pPredicates[uIndex].Attribute.Info = 
                                                s_MdsAttrInfo_Hrs_AuthenticatedDevice;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(uint32);
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pHrsBspDynamic->AuthenticatedDevice;
            uIndex++;
        }


    } /* Initial allocation successful */

    *ppPredState = (void*)pState;

    return rv;
}

static CSSM_RETURN CSSMAPI DestructHrsBspDynamicPredicate(
                            MDSU_CONTEXT *pContext,
                            CSSM_QUERY *pQuery,
                            void *pPredState )
{
    assert( pContext && pQuery );

    /* Free the attribute array */
    if ( pPredState != NULL )
    {
        MDSU_free( pContext, pPredState );
    }

    return CSSM_OK;
}

static CSSM_RETURN CSSMAPI ConvertHrsBspDynamicAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            void *pSchemaData )
{
    CSSM_RETURN crReturn;
    CDSA_HRS_BSP_DYNAMIC_SCHEMA *pHrsBspDynamic = (CDSA_HRS_BSP_DYNAMIC_SCHEMA*)pSchemaData;

    const CSSM_DB_ATTRIBUTE_DATA *DbAttr = NULL;

    assert( pSchemaData && DbRecord && pContext );

    DbAttr = DbRecord->AttributeData;


     /* ModuleID */
    crReturn = _MDSU_GetStructuredGUID( 
                                (char*)DbAttr[__ModuleId].Value->Data,
                                &pHrsBspDynamic->ModuleID );
    if ( crReturn != CSSM_OK )
    {
        return crReturn;
    }

    /* SSID */
    pHrsBspDynamic->SSID = *(uint32*)DbAttr[__SSId].Value->Data;

    /* BspName */
    strcpy( pHrsBspDynamic->BSPName,
            (char*)DbAttr[__BspName].Value->Data );


    /* SupportedFormats */
    pHrsBspDynamic->NumSupportedformats = (DbAttr[__SupportedFormats].Value->Length / sizeof(uint32));
    if((0 != pHrsBspDynamic->NumSupportedformats) &&
       (16 >= pHrsBspDynamic->NumSupportedformats))
    {
        memcpy(pHrsBspDynamic->SupportedFormats,
               DbAttr[__SupportedFormats].Value->Data,
               (sizeof(uint32) * pHrsBspDynamic->NumSupportedformats));
    }
    else
    {
        /* we can't handle more than 16 SupportedFormats */
        if(16 < pHrsBspDynamic->NumSupportedformats)
        {
            pHrsBspDynamic->NumSupportedformats = 0;
            return CSSMERR_CSSM_INTERNAL_ERROR;
        }
    }
    /* AuthFactors */
    pHrsBspDynamic->SupportedEvents = *(uint32*)DbAttr[__SupportedEvents].Value->Data;
    /* DeviceVendor */
    strcpy( pHrsBspDynamic->DeviceVendor,
            (char*)DbAttr[__DeviceVendor].Value->Data );

    /* DeviceDescription */
    strcpy( pHrsBspDynamic->DeviceDescription,
            (char*)DbAttr[__DeviceDescription].Value->Data );

     /* DeviceSerialNumber */
    strcpy( pHrsBspDynamic->DeviceSerialNumber,
            (char*)DbAttr[__DeviceSerialNumber].Value->Data );
   
    /* DeviceHWVersion */
    crReturn = _MDSU_GetStructuredVersion( 
                                    (char*)DbAttr[__DeviceHWVersion].Value->Data,
                                    &pHrsBspDynamic->DeviceHWVersion );
    /* DeviceFirmwareVersion */
    crReturn = _MDSU_GetStructuredVersion( 
                                    (char*)DbAttr[__DeviceFirmwareVersion].Value->Data,
                                    &pHrsBspDynamic->DeviceFirmwareVersion );

    /* AuthenticatdDevice */
    pHrsBspDynamic->AuthenticatedDevice = *(uint32*)DbAttr[__AuthenticatedDevice].Value->Data;

    return CSSM_OK;
}

const IfMdsuSchemaManipulator IfiMdsuCdsadirHrsBspDynamicSchema =
{
    ConstructHrsBspDynamicSchemaAttributes,
    DestructHrsBspDynamicSchemaAttributes,
    ConstructHrsBspDynamicPredicate,
    DestructHrsBspDynamicPredicate,
    ConvertHrsBspDynamicAttributes,
    MDS_CDSADIR_HRS_BSP_DYNAMIC_RECORDTYPE
};

