/*-----------------------------------------------------------------------
 * File: MAF_CONFIG.H
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */


#ifndef MAF_CONFIG_H
#define MAF_CONFIG_H

#include "hrstype.h"
#include "guids.h"
#include "maf_api.h"


#define ADDIN_VENDOR_NAME                   "Intel Corporation"

#define ADDIN_STANDARD_DESCRIPTION          "HRS Password BSP v1.0"

/* GUID of the addin module */
#define ADDIN_GUID_INITIALIZER \
	{ 0x4b302e32, 0x4d50, 0x11d4, { 0xba, 0x80, 0x0, 0x6, 0x29, 0xe2, 0x61, 0x71 } }

/* Manifest section name of the service provider */
#ifdef VMS
#define ADDIN_SELF_CHECK_SECTION            "cdsa$inhrspwbsp_shr"
#else
#define ADDIN_SELF_CHECK_SECTION            "inhrspwbsp100"
#endif

/* Name of the module. */
#define ADDIN_NAME                          "HRS Password BSP"

/* Major version of the cdsa interface */
#define ADDIN_VERSION_MAJOR                 CSSM_MAJOR

/* Minor version of the cdsa interface */
#define ADDIN_VERSION_MINOR                 CSSM_MINOR

/* Major and minor versions of the product */
#define ADDIN_PRODUCT_VERSION_MAJOR         1
#define ADDIN_PRODUCT_VERSION_MINOR         0

/* Is the addin module thread safe? */
#define ADDIN_THREAD_SAFE                   1

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_HRS_SUBSERVICE_ID             0

/* Does the addin module require the writer lock durring SPI functions on the load 
tracker node (is this node being modified?) */
#define ADDIN_NEED_LOAD_WRITER_LOCK         0

/* Does the addin module require the writer lock durring SPI functions on the attach 
tracker node (is this node being modified?) */
#define ADDIN_NEED_ATTACH_WRITER_LOCK       1

/* Does the addin module require the custom ADDIN_LOAD_DATA structure (see below) */
/* The signature key will be cached here */
#define ADDIN_NEED_ADDIN_LOAD_STRUCT        1

/* Does the addin module require the custom ADDIN_ATTACH_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_ATTACH_STRUCT      1

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_IS_VALID_SUBSERVICE(SubServiceId, SubServiceType) \
    (((SubServiceId) == 0) && ( ((SubServiceType) == CSSM_SERVICE_HRS) \
                              ) \
    )

/* Defining additional constants and data structs relating to the 
 * format of the biometric data that will be used in creating BIRs */
#define BIO_DATA_FORMAT_OWNER               0
#define BIO_DATA_FORMAT_ID                  0
#define BIR_HEADER_VERSION                  1

#define HASH_SALT_SIZE		8
#define PBE_SALT_SIZE		8
#define INIT_VECTOR_SIZE	8 
typedef struct BioDataHeader 
{
	uint32 PWLength;
	uint32 PayloadLength;
	uint8  HashSalt[HASH_SALT_SIZE];
	uint8  PBESalt[PBE_SALT_SIZE];
	uint8  InitVector[INIT_VECTOR_SIZE];

} BIO_DATA_HEADER;

/* Define default timeout values that will be used when 
 * capturing the password */
#define DEFAULT_TIMEOUT 20000 /* 20 seconds */

/* Declare a mutex that may be created when the module is 
 * initialized and used to lock the capture function so
 * the password buffer isn't wiped out in a thread race.  */
void *BSPMutex;

#if (ADDIN_NEED_ADDIN_LOAD_STRUCT)
/*------------------------------------------------------------------------------*
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_LOAD_TRACKER structure
 *------------------------------------------------------------------------------*/
struct addin_load_data
{
    /* Guid of the CSP that is tracked in order to 
     * detach and unload the module later           */
	CSSM_GUID		CSPGuid;

    /* Handle to the CSP that will be used to
     * perform the needed operations in the SPI     */
	CSSM_CSP_HANDLE CSPHandle;

    /* The static contexts that will be used to
     * perform many of those CSP operations.  The
     * dynamic ones such as the key derivation  
     * context are generated during the SPI calls.  */
	CSSM_CC_HANDLE  DigestContext;
	CSSM_CC_HANDLE  HashRandomContext;
	CSSM_CC_HANDLE  KeyDeriveRandomContext;
	CSSM_CC_HANDLE  InitRandomContext;

    /* The CSSM API entry points required to perform
     * the CSP operations.                          */
    _CSSM_ModuleLoad_PTR                  ModuleLoad;
    _CSSM_ModuleUnload_PTR                ModuleUnload;
    _CSSM_ModuleAttach_PTR                ModuleAttach;
    _CSSM_ModuleDetach_PTR                ModuleDetach;
    _CSSM_GenerateRandom_PTR              GenerateRandom;
    _CSSM_CSP_CreateDeriveKeyContext_PTR  CreateDeriveKeyContext;
    _CSSM_DeriveKey_PTR                   DeriveKey;
    _CSSM_CSP_CreateSymmetricContext_PTR  CreateSymmetricContext;
    _CSSM_EncryptData_PTR                 EncryptData;
    _CSSM_DecryptData_PTR                 DecryptData;
    _CSSM_DeleteContext_PTR               DeleteContext;
    _CSSM_FreeKey_PTR                     FreeKey;
    _CSSM_DigestData_PTR                  DigestData;
};
typedef struct addin_load_data ADDIN_LOAD_DATA;

#endif /* if ADDIN_NEED_ADDIN_LOAD_STRUCT */


#if (ADDIN_NEED_ADDIN_ATTACH_STRUCT)

typedef struct BIRHandleNode 
{
	CSSM_HRS_BIR_HANDLE BIRHandle;
	CSSM_HRS_BIR        BIR;
	void               *NextNode;

} BIR_LIST_NODE;

/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_ATTACH_TRACKER structure
 *
 * BIRList     - points to the list of linked list nodes which contain the
 *               BIRs which currently reside in the BSPs memory space\
 *
 * HandleIndex - counter which is used to ensure the handles given to the 
 *               BIRs do not collide
 *---------------------------------------------------------------------------*/
struct addin_attach_data
{
   BIR_LIST_NODE       *BIRList;
   CSSM_HRS_BIR_HANDLE  HandleIndex;
};
typedef struct addin_attach_data ADDIN_ATTACH_DATA;

#endif /* if ADDIN_NEED_ADDIN_ATTACH_STRUCT */

#endif
