/*-----------------------------------------------------------------------
 * File: LISTALLOC.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "maf_include.h"

CSSM_RETURN CSSMAPI MAF_AllocateCSSMList(
                        CSSM_HANDLE hModule,
                        CSSM_LIST_PTR pNewList,
                        uint32 NumElements )
{
    CSSM_RETURN rv = CSSM_OK;
    uint32 i;
    CSSM_LIST_ELEMENT *pNewElement = NULL;

    assert( pNewList );

    /* Fill in the basic list parameters */
    pNewList->ListType = CSSM_LIST_TYPE_SEXPR;
    pNewList->Head = pNewList->Tail = NULL;

    /* Allocate the list nodes */
    for ( i = 0; i < NumElements; i++ )
    {
        pNewElement = MAF_MALLOC( hModule, sizeof(CSSM_LIST_ELEMENT) );
        if ( pNewElement == NULL )
        {
            ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
        }
        else
        {
            /* Clear the structure */
            memset( pNewElement, 0, sizeof(CSSM_LIST_ELEMENT) );

            /* Insert the structure at the end of the list */
            if ( pNewList->Head == NULL )
            {
                pNewList->Head = pNewList->Tail = pNewElement;
            }
            else
            {
                pNewList->Tail->NextElement = pNewElement;
                pNewList->Tail = pNewElement;
            }
        }
    }

    /* Allocate any partial lists created before the error */
    if ( rv != CSSM_OK )
    {
        MAF_DeallocateCSSMList( hModule, pNewList );
    }

    return rv;
}

