/*-----------------------------------------------------------------------
 * File: LISTDEALLOC.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "maf_include.h"

CSSM_RETURN CSSMAPI MAF_DeallocateCSSMList(
                        CSSM_HANDLE hModule,
                        CSSM_LIST *pList )
{
    CSSM_RETURN rv = CSSM_OK;
    CSSM_LIST_ELEMENT *pElement = NULL;
    CSSM_LIST_ELEMENT *pDoomedElement = NULL;

    assert( pList );
    
    pElement = pList->Head;
    while ( pElement )
    {
        switch ( pElement->ElementType )
        {
        case CSSM_LIST_ELEMENT_WORDID:
        case CSSM_LIST_ELEMENT_DATUM:
            {
                /* Wipe and free the word value if it is present */
                if ( pElement->Element.Word.Data != NULL )
                {
                    memset( pElement->Element.Word.Data,
                            0,
                            pElement->Element.Word.Length );
                    MAF_FREE( hModule, pElement->Element.Word.Data );
                }

                break;
            }
        case CSSM_LIST_ELEMENT_SUBLIST:
            {
                /* Free the sublist */
                rv = MAF_DeallocateCSSMList( hModule,
                                             (CSSM_LIST*)&pElement->Element.Sublist );
            }
        default:
            {
                assert( 0 );
                ERR( rv = CSSMERR_CSSM_INTERNAL_ERROR );
            }
        }

        /* Save the current pointer */ 
        pDoomedElement = pElement;
        pElement = pElement->NextElement;

        /* Wipe and free the element structure */
        memset( pDoomedElement, 0, sizeof(CSSM_LIST_ELEMENT) );
        MAF_FREE( hModule, pDoomedElement );

    }

    /* Wipe the list structure itself */
    memset( pList, 0, sizeof(CSSM_LIST) );

    return rv;
}
