/***************************************************************************/
/* tagint.h                                                                */
/*                                                                         */
/* Tag intersection code.                                                  */
/***************************************************************************/

#if !defined( __TAGINT_H__ )
#define __TAGINT_H__

#include "sexp.h"

#define ST_NONE (10)            /* not a known *-form */
#define ST_STAR (11)            /* (*) */
#define ST_SET (12)             /* (* set ...) */
#define ST_PREFIX (13)          /* (* prefix ...) */
#define ST_RANGE (14)           /* (* range ...) */
#define ST_AND (15)             /* (* and ...) */

/***************************************************************************/
/* sexp_compare                                                            */
/*                                                                         */
/* Compare two S-expressions.                                              */
/*                                                                         */
/* EQUAL: a == b if they are equal throughout.                             */
/* LESSER: a < b -- if b != a and a <= b                                   */
/*         a <= b -- if a is not longer than b and each sub-expression     */
/*                 as <= bs.                                               */
/* GREATER: a > b -- if b < a                                              */
/* INCOMPARABLE: otherwise                                                 */
/***************************************************************************/

int sexp_compare( pSexp a, pSexp b ) ;

/***************************************************************************/
/* pull_star                                                               */
/*                                                                         */
/* Grab the type of * node from the S-expression, s.                       */
/***************************************************************************/

int pull_star( pSexp s ) ;

/***************************************************************************/
/* tag_intersect                                                           */
/*                                                                         */
/* Compute the intersection of two tags and return the intersection.       */
/* This can be either of the two inputs, or a new, constructed Sexp.       */
/*                                                                         */
/* The pSexp returned by tag_intersect needs to have its refcnt            */
/* bumped if it is stored.  That is, you can call                          */
/*   store_sexp( &loc, tag_intersect( ) )                                  */
/* without adjusting the refcnt.                                           */
/*                                                                         */
/* This also means that it is possible for tag_intersect to return         */
/* a pSexp with a refcnt of 0, implying that if it isn't stored            */
/* it needs to be dropped.                                                 */
/*                                                                         */
/***************************************************************************/

pSexp tag_intersect( pSexp u, pSexp v ) ;

#endif /* __TAGINT_H__ */
