/*-----------------------------------------------------------------------
 * File: AALINTRN.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _AALINTRN_H
#define _AALINTRN_H    

#include "cssmport.h"

#ifdef ISL_INCLUDED
#include "eislapi.h"
#endif

#if defined(WIN32)
#define AAL_MAX_REG REGSTR_MAX_VALUE_LENGTH + 1
#else
#define AAL_MAX_REG 256
#endif  /* WIN32 */

#ifdef WIN32
#define CSSMSECTION "Software\\CDSA\\"  
#else
#define CSSMSECTION "Software/CDSA/"  
#endif

#define AAL_PREFERENCE_PERSONAL    1
#define AAL_PREFERENCE_GLOBAL      0
#define AAL_VALUE_BINARY           0
#define AAL_VALUE_STRING           1

typedef char **AAL_REG_LIST, ***AAL_REG_LIST_PTR;

#ifdef VMS
#define aal_LoadCSSM          aalint_LoadCSSM
#define aal_GetCSSMList       aalint_GetCSSMList
#define aal_GetModuleLocation aalint_GetModuleLocation
#endif

AAL_RETURN_CODE aal_GetFunctionPtrsFromLoadedCSSM(AAL_LIB_HANDLE_PTR libHandle,  
                                                  AAL_FUNC_TABLE_PTR FuncNameAddrPtr, 
                                                  uint32 Numfns);
AAL_RETURN_CODE aal_LoadCSSM(AAL_FUNC_TABLE_PTR FuncNameAddrPtr, 
                                uint32 NumOfFuncNameAddr,
                                CSSM_GUID_PTR Guid,
                                AAL_LIB_HANDLE_PTR libHandle,
                                CSSM_DATA_PTR TrustedKeys,
                                uint32 NumTrustedKeys);

AAL_RETURN_CODE aal_GetCSSMList(AAL_VERSION_VENDOR_PTR VersionVendorPtr,
                                    uint32* NumOfVersionVendorInfo);

AAL_RETURN_CODE aal_GetCredentialAndSection(CSSM_GUID_PTR ModuleGuid,
                                CSSM_DATA_PTR Manifest,
                                CSSM_DATA_PTR ModuleSerachPath,
                                AAL_PATHNAME_PTR ManifestSection);
AAL_RETURN_CODE aal_GetModuleLocation(CSSM_GUID_PTR ModuleGuid,
                                      AAL_PATHNAME_PTR Location);
AAL_RETURN_CODE
aal_GetModuleInfo (CSSM_GUID_PTR guid, CSSM_DATA_PTR Credential, CSSM_DATA_PTR ModuleName, 
                                CSSM_DATA_PTR ModulePath);

AAL_RETURN_CODE
aal_GetModulePath (CSSM_DATA ModuleName, CSSM_DATA ModuleSearchPath, AAL_PATHNAME ModulePath);

#ifdef CSSM_MEMTRACK_ON

#define aal_malloc_func(s, t) _aal_malloc_func((s), (t), __FILE__, __LINE__)
#define aal_calloc_func(s, t, u) _aal_calloc_func((s), (t) ,(u), __FILE__, __LINE__)
#define aal_realloc_func(s, t, u) _aal_realloc_func((s), (t), (u), __FILE__, __LINE__)
#define aal_free_func(s, t) _aal_free_func((s), (t), __FILE__, __LINE__)

void* CSSMAPI
_aal_malloc_func(uint32 size,
                 void *allocRef,
                 const char * szFilename,
                 uint32 u32LineNumber);

void* CSSMAPI
_aal_calloc_func(uint32 num_elements,
                 uint32 size_element, 
                 void *allocRef,
                 const char * szFilename,
                 uint32 u32LineNumber);

void* CSSMAPI
_aal_realloc_func(void * ptr, 
                  uint32 new_size, 
                  void *allocRef,
                  const char * szFilename,
                  uint32 u32LineNumber);

void CSSMAPI
_aal_free_func(void *MemPtr, 
               void *allocRef,
               const char * szFilename,
               uint32 u32LineNumber);
#else

#define aal_malloc_func(s, t) _aal_malloc_func((s), (t))
#define aal_calloc_func(s, t, u) _aal_calloc_func((s), (t) ,(u))
#define aal_realloc_func(s, t, u) _aal_realloc_func((s), (t), (u))
#define aal_free_func(s, t) _aal_free_func((s), (t))

void* CSSMAPI
_aal_malloc_func(uint32 size,
                 void *allocRef);

void* CSSMAPI
_aal_calloc_func(uint32 num_elements,
                 uint32 size_element, 
                 void *allocRef);

void* CSSMAPI
_aal_realloc_func(void * ptr, 
                  uint32 new_size, 
                  void *allocRef);

void CSSMAPI
_aal_free_func(void *MemPtr, 
               void *allocRef);

#endif

#endif
