/*-----------------------------------------------------------------------
 * File: INTERNAL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


#ifndef INTERNAL_H
#define INTERNAL_H

#include "cssmtype.h"

#ifdef ISL_INCLUDED
#ifdef VMS
#include <isl_internal.h>
#endif
#include "isltype.h"
#endif

#ifdef VMS
#define cssm_GetSubserviceUIDFromHandle cssm_GetSSUIDFromHandle
#define cssm_ModuleLoad   cssmint_ModuleLoad
#define cssm_ModuleUnload cssmint_ModuleUnload
#define cssm_ModuleAttach cssmint_ModuleAttach
#define cssm_ModuleDetach cssmint_ModuleDetach
#endif

#ifdef __cplusplus
extern "C"
{
#endif



typedef struct cssm_thread_data
{
    CSSM_GUID ModuleGuid;
    void* CssmNotifyCallbackCtx;
    uint32 SubserviceID;
    CSSM_SERVICE_TYPE ServiceType;
    CSSM_MODULE_EVENT EventType;
} cssm_THREAD_DATA, *cssm_THREAD_DATA_PTR;



typedef CSSM_RETURN  (CSSMAPI *SPI_ATTACH_FUNC_PTR) (
                            const CSSM_GUID *ModuleGuid,
                            const CSSM_VERSION *Version,
                            uint32 SubServiceId,
                            CSSM_SERVICE_TYPE SubServiceType,
                            CSSM_ATTACH_FLAGS AttachFlags,
                            CSSM_MODULE_HANDLE ModuleHandle,
                            CSSM_KEY_HIERARCHY KeyHierarchy,
                            const CSSM_GUID *CssmGuid,
                            const CSSM_GUID *ModuleManagerGuid,
                            const CSSM_GUID *CallerGuid,
                            CSSM_UPCALLS_PTR Upcalls,
                            CSSM_MODULE_FUNCS_PTR *FuncTbl);
typedef CSSM_RETURN (CSSMAPI *SPI_DETACH_FUNC_PTR)(
                            CSSM_MODULE_HANDLE Handle);

typedef CSSM_RETURN  (CSSMAPI *MANAGER_AUTH_FUNC_PTR) ( 
                                      CSSM_KEY_HIERARCHY KeyHierarchy,
                                      const CSSM_GUID *CssmGuid,
                                      const CSSM_GUID *CallerGuid,
                                      CSSM_MANAGER_REGISTRATION_INFO_PTR RegInfo);
typedef CSSM_RETURN (CSSMAPI * LOAD_MODULE_FUNC_PTR)
                               (const CSSM_GUID *CssmGuid,
                                const CSSM_GUID *ModuleGuid,
                                CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
                                const void* CssmNotifyCallbackCtx);
typedef CSSM_RETURN (CSSMAPI * UNLOAD_MODULE_FUNC_PTR)
                               (const CSSM_GUID *CssmGuid,
                                const CSSM_GUID *ModuleGuid, 
                                CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
                                const void* CssmNotifyCallbackCtx);

extern CSSM_RETURN CSSMAPI
CSSM_CRYPTO_DATA_API_CALLBACK(CSSM_DATA_PTR OutData,
                              void *CallbackCtx);


#include "cssmspi.h" /* Moved after the above typedef's because 
                      * of possible circular inclusion
                      */
 
extern CSSM_GUID cssm30_GUID;
extern CSSM_PRIVILEGE_SCOPE PrivilegeScope;

#ifdef ISL_INCLUDED
extern ISL_VERIFIED_MODULE_PTR *cssm30_Credentials;
extern uint32 cssm30_NumSections;
#else
extern CSSM_LIB_HANDLE cssm30_Library;
#endif

extern CSSM_FREE app_free;
extern CSSM_MALLOC app_malloc; 
extern CSSM_REALLOC app_realloc; 
extern CSSM_CALLOC app_calloc;
extern void *memRef;

extern uint32 cssm_NumUseetags;
extern uint32 *cssm_UseeTagList;
extern uint32 cssm_NumCssmTOGPrivs;
extern uint32* cssm_CssmTOGPrivs;
extern CSSM_PVC_MODE PvcMode;
extern CSSM_PRIVILEGE cssm_GlobalPrivilege;
extern PORT_TLS_INDEX hThreadContext;
extern cssm_SWMRLock_PTR hPrivSelectionLock;

/* privilege masks */
#define CSSM_USEE_MASK 0x000000FF
#define CSSM_TOGVENDOR_MASK 0xFFFF0000
#define CSSM_TOGMASK 0x80000000

/* internal locks available */
typedef uint32 CSSM_LOCK_TYPE, *CSSM_LOCK_TYPE_PTR;
#define CSSM_NO_LOCK     0
#define CSSM_READER_LOCK 1
#define CSSM_WRITER_LOCK 2



typedef struct cssm_vendor_privileges
{
    uint32 NumVendorPrivs;
    uint32 *VendorPrivs;
    CSSM_KEY_HIERARCHY KeyHierarchy;

    struct cssm_vendor_privileges *Next;
}
cssm_VENDOR_PRIVILEGES, *cssm_VENDOR_PRIVILEGES_PTR;

extern cssm_VENDOR_PRIVILEGES_PTR cssm_CssmVendorPrivs;


typedef struct cssm_module_components
{
    CSSM_PROC_ADDR StartAddr;
    CSSM_PROC_ADDR EndAddr;

#ifdef ISL_INCLUDED
    uint32 NumComponents;
    ISL_VERIFIED_MODULE_PTR *VerifiedModule;
#endif

    struct cssm_module_components *Next;
}
cssm_MODULE_COMPONENTS, *cssm_MODULE_COMPONENTS_PTR;


typedef struct cssm_caller_list_entry
{
    uint32 RefCount;

    /* privileges this caller is authorized to use */
    uint32 NumTOGPrivs;
    uint32 *TOGPrivs;
    uint32 NumUSEETags;
    uint32 *USEETags;
    cssm_VENDOR_PRIVILEGES_PTR VendorPrivs;

    /* key hierarchies this caller can use */
    CSSM_KEY_HIERARCHY KeyHierarchy;

    CSSM_GUID CallerGuid;
    CSSM_BOOL bAddin;
    /* start and end addresses for module for PVC */
    CSSM_PROC_ADDR ModuleStartAddr;
    CSSM_PROC_ADDR ModuleEndAddr;
    cssm_MODULE_COMPONENTS_PTR ModuleComponents;
}
cssm_CALLER_LIST_ENTRY, *cssm_CALLER_LIST_ENTRY_PTR;



typedef struct cssm_caller_list
{
    struct cssm_caller_list *Next;
    cssm_SWMRLock_PTR EntryLock;

    cssm_CALLER_LIST_ENTRY_PTR ListEntry;
}
cssm_CALLER_LIST, *cssm_CALLER_LIST_PTR;



typedef struct cssm_index_entry
{
    CSSM_PROC_ADDR IndexKey;
    cssm_CALLER_LIST_PTR CallerList;
}
cssm_INDEX_ENTRY, *cssm_INDEX_ENTRY_PTR;



typedef struct cssm_caller_list_index
{
    cssm_SWMRLock_PTR CallerListLock;
    uint32 NumEntries;
    cssm_INDEX_ENTRY *IndexArray;
    cssm_CALLER_LIST_PTR CallerList;
}
cssm_CALLER_LIST_INDEX, *cssm_CALLER_LIST_INDEX_PTR;



/*
 * Structure to define the information about an EMM module Manager.
 */
typedef struct internalmanagerlist
{
    CSSM_GUID EMMGuid;                /* GUID of EMM */
    cssm_SWMRLock_PTR hEMMRecordSWMRLock;
    CSSM_SERVICE_TYPE ServiceType;  /* type of service supported */
    uint32 attachRefCount;          /* Number of emm module attaches */    

    CSSM_BOOL EMMInitialized;
    CSSM_KEY_HIERARCHY VerifiedHierarchy;

    CSSM_MANAGER_REGISTRATION_INFO_PTR EMMJT;   /* Entry point into the EMM
                                                   functions
                                                */
#ifdef ISL_INCLUDED
    uint32 NumEmmSections;
    ISL_VERIFIED_MODULE_PTR* EMMCredentialPtr;  /*FillIn at attach time*/
#else
    CSSM_LIB_HANDLE LibHandle;      /* Handle to actual library */
#endif

    struct internalmanagerlist *Next;     /* Next EMM in list */
} cssm_INTERNALMANAGERLIST, *cssm_INTERNALMANAGERLIST_PTR;



typedef struct cssm_app_notify_callback_list
{
    uint32 RefCount;
    CSSM_API_ModuleEventHandler AppCallbackFunc;
    void *AppCallbackCtx;
    struct cssm_app_notify_callback_list *Next;
} cssm_APP_NOTIFY_CALLBACK_LIST, *cssm_APP_NOTIFY_CALLBACK_LIST_PTR;



typedef struct cssm_challenge_callback_proxy_modulehandle
{
    CSSM_MODULE_HANDLE ModuleHandle;
    CSSM_CHALLENGE_CALLBACK Callback;
    void* CallbackCtx;
}CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE, *CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR;


typedef struct cssm_challenge_callback_proxy
{
    CSSM_CC_HANDLE CCHandle;
    CSSM_CHALLENGE_CALLBACK Callback;
    void* CallbackCtx;
}CSSM_CHALLENGE_CALLBACK_PROXY, *CSSM_CHALLENGE_CALLBACK_PROXY_PTR;

typedef struct cssm_subject_callback_proxy
{
    CSSM_MODULE_HANDLE ModuleHandle;
    CSSM_ACL_SUBJECT_CALLBACK Callback;
    void* CallbackCtx;
}CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE, *CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR;

typedef struct cssm_passphrase_proxy
{
    CSSM_CC_HANDLE CCHandle;
    CSSM_CALLBACK Callback;
    void* CallbackCtx;
}CSSM_PASSPHRASE_PROXY, *CSSM_PASSPHRASE_PROXY_PTR;

typedef struct cssm_seed_callback_proxy
{
    CSSM_CC_HANDLE CCHandle;
    CSSM_CALLBACK Callback;
    void* CallbackCtx;
}CSSM_SEED_CALLBACK_PROXY, *CSSM_SEED_CALLBACK_PROXY_PTR;



typedef struct cssm_internal_context
{
    CSSM_CONTEXT_PTR Context;
    CSSM_CSP_HANDLE CSPHandle;
    CSSM_CC_HANDLE ContextHandle;
    CSSM_SEED_CALLBACK_PROXY_PTR SeedProxy;
    CSSM_PASSPHRASE_PROXY_PTR PassphraseProxy;
    CSSM_CHALLENGE_CALLBACK_PROXY_PTR ChallengeCbProxy;
    cssm_SWMRLock_PTR hContextSWMRLock; /* 
                                         * Single writer multiple readers lock for
                                         * synchronization between various context.
                                         */
    struct cssm_internal_context *Next;
} cssm_INTERNAL_CONTEXT, *cssm_INTERNAL_CONTEXT_PTR;



typedef struct cssm_internal_attach_list
{
    cssm_SWMRLock_PTR hAttachSWMRLock;

    /* used for non-threadsafe modules - hopefully this will go away */
    CSSM_BOOL ThreadSafe;
    HANDLE MultiTaskMutex;            /* Mutex for functions in Addin */

    /* used to find EMM record if this is an EMM attach */
    cssm_INTERNALMANAGERLIST_PTR pEMMRecord;

    CSSM_HANDLE AttachHandle;

    CSSM_BOOL AttachComplete;

    CSSM_SERVICE_TYPE ServiceType;
    CSSM_ATTACH_FLAGS AttachFlags;

    CSSM_VERSION Version;
    CSSM_API_MEMORY_FUNCS_PTR MemoryFunctions;
    CSSM_MODULE_FUNCS_PTR AddInFunctions;   /* the Add-in functions */

    CSSM_KEY_HIERARCHY KeyHierarchy;
    uint32 NumVendorPrivs;
    uint32 *VendorPrivs;

    cssm_INTERNAL_CONTEXT_PTR ContextList;

    struct cssm_internal_attach_list *Next;
} cssm_INTERNAL_ATTACH_LIST, *cssm_INTERNAL_ATTACH_LIST_PTR;



typedef struct cssm_internal_service_list
{
    cssm_SWMRLock_PTR hServiceSWMRLock;

    uint32 ServiceHandleIndex;
    uint32 SubServiceID;
    CSSM_SERVICE_MASK ServiceMask;

#ifdef ISL_INCLUDED
    uint32 NumSPCreds;
    ISL_VERIFIED_MODULE_PTR* SPCredentialPtr;
    CSSM_BOOL bDiffPrimaryManifest;
#else
    CSSM_LIB_HANDLE ModuleLibHandle;
#endif
    CSSM_KEY_HIERARCHY KeyHierarchy;

    /* privilege and USEE tags */
    uint32 NumSPTags;
    uint32 *SPTags;
    uint32 NumTOGPrivs;
    uint32 *TOGPrivs;
    cssm_VENDOR_PRIVILEGES_PTR VendorPrivs;

    cssm_INTERNAL_ATTACH_LIST_PTR AttachList;

    struct cssm_internal_service_list *Next;
} cssm_INTERNAL_SERVICE_LIST, *cssm_INTERNAL_SERVICE_LIST_PTR;



typedef struct cssm_internal_module_list
{
    uint32 ModuleHandleIndex;   /* top 9 bits used */

    CSSM_GUID GUID;             /* GUID of service provider or
                                   docking module as appropriate
                                */

    cssm_SWMRLock_PTR hModuleSWMRLock;  /*
                                         * Single writer multiple readers lock
                                         * for synchronization between various
                                         * load/attach instances of a module.
                                         */
    CSSM_SERVICE_MASK ServiceSummary;
    cssm_INTERNALMANAGERLIST_PTR* pEMMRecord;
    uint32 NumEmms;
    /* used to determine if CSSM needs to do locking */
    CSSM_BOOL ThreadSafe;
    HANDLE MultiTaskMutex;  /* Mutex for functions in service provider */

#ifdef ISL_INCLUDED
    uint32 NumSections;
    ISL_VERIFIED_MODULE_PTR* ModuleCredentialPtr;   /* possibly docking module,
                                                       possible addin module.
                                                    */
#else
    CSSM_LIB_HANDLE LibHandle;     /* Handle to actual library */
#endif  /* ISL_INCLUDED */

    cssm_INTERNAL_SERVICE_LIST_PTR ServiceList;

    cssm_APP_NOTIFY_CALLBACK_LIST_PTR EventNotificationCallbackList;

    struct cssm_internal_module_list* Next;     /* Next Plug-in in list */
} cssm_INTERNAL_MODULE_LIST, *cssm_INTERNAL_MODULE_LIST_PTR;



extern CSSM_UPCALLS InternalUpcalls;


extern cssm_SWMRLock_PTR hModuleListSWMRLock; /* for syncronising the
                                                 allocation and deallocation of
                                                 attached addinInfo structure
                                              */

extern cssm_SWMRLock_PTR hEMMListSWMRLock;

/* Linked list tracking pointers */
extern cssm_INTERNAL_MODULE_LIST_PTR hModuleListHead;
extern cssm_INTERNAL_MODULE_LIST_PTR hModuleListTail;

extern cssm_INTERNALMANAGERLIST_PTR EMMManagerListHead;
extern cssm_INTERNALMANAGERLIST_PTR EMMManagerListTail;

/* lock and structure for the global caller list */
extern cssm_SWMRLock_PTR hCallerListIndexLock;
extern cssm_CALLER_LIST_INDEX_PTR hCallerListIndex;


/* Module information tree search functions. */
CSSM_RETURN
cssm_GetModuleListLock(CSSM_LOCK_TYPE ModuleListLock);

CSSM_RETURN
cssm_FindModuleAndMultiLock(CSSM_MODULE_HANDLE hModule,
                            CSSM_LOCK_TYPE ModuleListLock,
                            cssm_INTERNAL_MODULE_LIST_PTR *ppModule,
                            CSSM_LOCK_TYPE ModuleEntryLock);

CSSM_RETURN
cssm_FindModuleByGUIDAndMultiLock(const CSSM_GUID *GuidPtr,
                                  CSSM_LOCK_TYPE ModuleListLock,
                                  cssm_INTERNAL_MODULE_LIST_PTR *ppModule,
                                  CSSM_LOCK_TYPE ModuleEntryLock);

CSSM_RETURN
cssm_FindServiceAndMultiLock(CSSM_MODULE_HANDLE hService,
                             CSSM_LOCK_TYPE ModuleListLock,
                             cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                             CSSM_LOCK_TYPE ModuleEntryLock,
                             cssm_INTERNAL_SERVICE_LIST_PTR *ppService,
                             CSSM_LOCK_TYPE ServiceEntryLock);

CSSM_RETURN
cssm_FindServiceByGUIDAndSSIDAndMultiLock(const CSSM_GUID *pGUID,
                                          uint32 SubServiceID,
                                          CSSM_LOCK_TYPE ModuleListLock,
                                          cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                                          CSSM_LOCK_TYPE ModuleEntryLock,
                                          cssm_INTERNAL_SERVICE_LIST_PTR *ppService,
                                          CSSM_LOCK_TYPE ServiceEntryLock);

CSSM_RETURN
cssm_FindAttachAndMultiLock(CSSM_MODULE_HANDLE hAttach,
                            CSSM_LOCK_TYPE ModuleListLock,
                            cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                            CSSM_LOCK_TYPE ModuleEntryLock,
                            cssm_INTERNAL_SERVICE_LIST_PTR *ServiceRecord,
                            CSSM_LOCK_TYPE ServiceEntryLock,
                            cssm_INTERNAL_ATTACH_LIST_PTR *ppAttach,
                            CSSM_LOCK_TYPE AttachEntryLock);

CSSM_RETURN
cssm_FindContextAndMultiLock(CSSM_CC_HANDLE hContext,
                             CSSM_LOCK_TYPE ModuleListLock,
                             cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                             CSSM_LOCK_TYPE ModuleEntryLock,
                             cssm_INTERNAL_SERVICE_LIST_PTR *ServiceRecord,
                             CSSM_LOCK_TYPE ServiceEntryLock,
                             cssm_INTERNAL_ATTACH_LIST_PTR *AttachRecord,
                             CSSM_LOCK_TYPE AttachEntryLock,
                             cssm_INTERNAL_CONTEXT_PTR *ppContext,
                             CSSM_LOCK_TYPE ContextEntryLock);


void cssm_ReleaseContextLock(cssm_INTERNAL_CONTEXT_PTR pContext,
                             CSSM_LOCK_TYPE ltLockType);
void cssm_ReleaseAttachLock(cssm_INTERNAL_ATTACH_LIST_PTR pAttach,
                            CSSM_LOCK_TYPE ltLockType);
void cssm_ReleaseServiceLock(cssm_INTERNAL_SERVICE_LIST_PTR pService,
                             CSSM_LOCK_TYPE ltLockType);
void cssm_ReleaseModuleLock(cssm_INTERNAL_MODULE_LIST_PTR pModule,
                            CSSM_LOCK_TYPE ltLockType);
void cssm_ReleaseModuleListLock(CSSM_LOCK_TYPE ltLockType);


/* 9 most significant bits */
#define NUM_MODULE_INDEX_BITS   ( 9 )
#define cssm_ExtractModuleIndex(handle)  ( handle & 0xFF800000 )

/* Next 6 most significant bits */
#define NUM_SERVICE_INDEX_BITS  ( 6 )
#define cssm_ExtractServiceIndex(handle) ( handle & 0xFFFE0000 )

/* 17 least significant bits */
#define NUM_ATTACH_INDEX_BITS   ( 17 )
#define cssm_ExtractAttachIndex(handle)  ( handle & 0xFFFFFFFF )


/* getting the high and low parts of a 64-bit integer */
#define CSSM_UINT64_HIGH(v) ( (uint32)( v >> 32 ) )
#define CSSM_UINT64_LOW(v)  ( (uint32)v )
#define CSSM_UINT64_SETHIGH(v,w) { v = ( ( v & 0xFFFFFFFF ) | ( (uint64)w << 32 ) ); }



/* Macro to count the "1" bits in a word */
#define cssm_CountBits( _inword_, _count_ ) \
    { \
        uint32 _tempword_ = _inword_; \
        _count_ = 0; \
        while ( _tempword_ ) \
        { \
            if ( _tempword_ & 1 ) \
            { \
                _count_++; \
            } \
            _tempword_ >>= 1; \
        } \
    }\

/* caller list locking/searching functions */
CSSM_RETURN
cssm_GetCallerListIndexLock(CSSM_LOCK_TYPE ltLockType);

CSSM_RETURN
cssm_GetCallerListAndMultiLock(CSSM_LOCK_TYPE CallerListIndexLock,
                               cssm_CALLER_LIST_INDEX_PTR *CallerListIndex,
                               CSSM_LOCK_TYPE CallerListLock,
                               cssm_CALLER_LIST_PTR *CallerList);

CSSM_RETURN
cssm_FindCallerListEntryAndMultiLock(CSSM_PROC_ADDR CallerAddress,
                                     CSSM_LOCK_TYPE CallerListIndexLock,
                                     cssm_CALLER_LIST_INDEX_PTR *CallerListIndex,
                                     CSSM_LOCK_TYPE CallerListLock,
                                     cssm_CALLER_LIST_PTR *CallerList,
                                     CSSM_LOCK_TYPE CallerListEntryLock,
                                     cssm_CALLER_LIST_ENTRY_PTR *CallerListEntry);

CSSM_RETURN
cssm_FindCallerListEntryByGuidAndMultiLock(const CSSM_GUID *GuidToFind,
                                           CSSM_LOCK_TYPE CallerListIndexLock,
                                           cssm_CALLER_LIST_INDEX_PTR *CallerListIndex,
                                           CSSM_LOCK_TYPE CallerListLock,
                                           cssm_CALLER_LIST_PTR *CallerList,
                                           CSSM_LOCK_TYPE CallerListEntryLock,
                                           cssm_CALLER_LIST_ENTRY_PTR *CallerListEntry);

void
cssm_ReleaseCallerListEntry(cssm_CALLER_LIST_PTR CallerList,
                            CSSM_LOCK_TYPE ltLockType);

void
cssm_ReleaseCallerList(cssm_CALLER_LIST_INDEX_PTR CallerListIndex,
                       CSSM_LOCK_TYPE ltLockType);

void
cssm_ReleaseCallerListIndex(CSSM_LOCK_TYPE ltLockType);



/* caller list insert/remove and create/destroy functions */
CSSM_RETURN
cssm_InsertCallerListEntry(cssm_CALLER_LIST_ENTRY_PTR NewEntry);

CSSM_RETURN
cssm_RemoveCallerListEntry(CSSM_PROC_ADDR CallerAddress,
                           cssm_CALLER_LIST_ENTRY_PTR *RemovedEntry);

CSSM_RETURN
cssm_CreateGlobalCallerList();  /* should only be called on "1st" Init */

CSSM_RETURN
cssm_DestroyGlobalCallerList(); /* Must be called only after module list is
                                   destroyed or a risk of deadlock exists.
                                   Should only be called after "last" Term.
                                */

#ifdef ISL_INCLUDED
CSSM_BOOL cssm_PrivilegeMatchForCallerAndSP( CSSM_PRIVILEGE Privilege,
                                             cssm_CALLER_LIST_ENTRY_PTR CallerListEntry,
                                             cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord,
                                             cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord);

CSSM_RETURN
cssm_VerifyCallerAndFillInPrivileges(cssm_CALLER_LIST_ENTRY_PTR Entry,
                                     const CSSM_DATA *Credential,
                                     const CSSM_DATA *Path,
                                     CSSM_KEY_HIERARCHY KeyHierarchy,
                                     ISL_VERIFIED_MODULE_PTR **VerModPtr,
                                     uint32 *NumSections);
CSSM_RETURN cssm_ExtractUseeAndPrivFromCssmManifest( ISL_VERIFIED_MODULE_PTR *VerModPtr, uint32 NumSections,
                                                    uint32 *ModNumUseeTags, uint32** ModUseeTags, 
                                                    uint32 *ModNumTogPrivs, uint32** ModTogPrivs,
                                                    cssm_VENDOR_PRIVILEGES_PTR* ModVendorPrivilegePtr,
                                                    CSSM_KEY_HIERARCHY KeyHierarchy);

#endif

CSSM_RETURN
cssm_DestroyCallerListEntry(cssm_CALLER_LIST_ENTRY_PTR *Entry);



/* privilege setting/getting functions */
CSSM_RETURN
cssm_SetThreadPrivilege(CSSM_PRIVILEGE Privilege);

CSSM_RETURN
cssm_GetThreadPrivilege(CSSM_PRIVILEGE* Privilege);

CSSM_RETURN
cssm_SetGlobalPrivilege(CSSM_PRIVILEGE Privilege);

CSSM_RETURN
cssm_GetGlobalPrivilege(CSSM_PRIVILEGE* Privilege);
void 
CSSMAPI cssm_FreeTLSPrivilege ( void *pValue );


/* PVC policy setting funcs */
#ifdef ISL_INCLUDED
CSSM_RETURN cssm_CheckPVCTagsAndSetPVCPolicy(const CSSM_GUID* CallerGuid, 
                                             CSSM_PVC_MODE *PvcPolicy,
                                             CSSM_KEY_HIERARCHY KeyHierarchy,
                                             ISL_VERIFIED_MODULE_PTR* CssmExpVerifiedCredentials,
                                             uint32 NumExpSections);
#endif

CSSM_RETURN
cssm_SetPVCPolicy(CSSM_PVC_MODE PvcPolicy);

CSSM_PVC_MODE
cssm_GetPVCPolicy();

CSSM_RETURN
cssm_GetCSSMNativeServices(CSSM_SERVICE_MASK *NativeServices);


CSSM_RETURN CSSMAPI
cssm_GetUpcallsTable(CSSM_MODULE_HANDLE hAddIn,
                     CSSM_UPCALLS_PTR UpcallTable);

CSSM_RETURN CSSMAPI
cssm_GetAppMemoryFunctions(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord, CSSM_API_MEMORY_FUNCS_PTR *MemFuncs);

CSSM_RETURN CSSMAPI
cssm_GetAddInCallBackByType(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord,
                            CSSM_SERVICE_MASK AddInType, void** CallBack);



/* setup/cleanup for standard API functions which are serviced by service
   providers
*/

CSSM_RETURN
cssm_SetupStandardSPICall(CSSM_PROC_ADDR retAddress,
                          CSSM_HANDLE ModuleHandle,
                          CSSM_SERVICE_TYPE ModuleType,
                          void** CallBack,
                          cssm_INTERNAL_ATTACH_LIST_PTR *AttachRecord);

CSSM_RETURN
cssm_CleanupStandardSPICall(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord);



/* load/attach related internal functions */
CSSM_RETURN CSSMAPI
cssm_ModuleUnload(const CSSM_GUID *ModuleGuid,
                  CSSM_API_ModuleEventHandler AppNotifyCallback,
                  const void* AppNotifyCallbackCtx);
CSSM_RETURN CSSMAPI
cssm_ModuleLoad(const CSSM_GUID *ModuleGuid,
                CSSM_KEY_HIERARCHY KeyHierarchy,
                CSSM_API_ModuleEventHandler AppNotifyCallback,
                const void* AppNotifyCallbackCtx);
CSSM_RETURN CSSMAPI
cssm_ModuleAttach(const CSSM_GUID *GUID,
                  const CSSM_VERSION *Version,
                  const CSSM_API_MEMORY_FUNCS *MemoryFuncs,
                  uint32 SubserviceID,
                  CSSM_SERVICE_TYPE SubserviceType,
                  CSSM_ATTACH_FLAGS AttachFlags,
                  CSSM_KEY_HIERARCHY KeyHierarchy,
                  CSSM_FUNC_NAME_ADDR_PTR FunctionTable,
                  uint32 NumFunctionTable,
                  const void * Reserved,
                  CSSM_MODULE_HANDLE_PTR NewModuleHandle,
                  const CSSM_GUID* CallerGuid
                  );

CSSM_RETURN CSSMAPI
cssm_ModuleDetach(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord,
                  cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord);

CSSM_RETURN CSSMAPI cssm_CheckInit (void);


CSSM_RETURN CSSMAPI CSSMVerifyComponents (void);


/* locking functions */
CSSM_RETURN CSSMAPI cssm_ReaderLock(cssm_SWMRLock_PTR SWMRLock);
CSSM_RETURN CSSMAPI cssm_ReaderUnlock (cssm_SWMRLock_PTR SWMRLock);
CSSM_RETURN CSSMAPI cssm_WriterLock (cssm_SWMRLock_PTR SWMRLock);
CSSM_RETURN CSSMAPI cssm_WriterUnlock (cssm_SWMRLock_PTR SWMRLock);
CSSM_RETURN CSSMAPI cssm_WriterToReader (cssm_SWMRLock_PTR SWMRLock);
CSSM_RETURN CSSMAPI cssm_SWMRInit (cssm_SWMRLock_PTR *SWMRLock);
CSSM_RETURN CSSMAPI cssm_SWMRDelete (cssm_SWMRLock_PTR SWMRLock);

CSSM_RETURN CSSMAPI
cssm_LockMultiTaskMutex(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord);
CSSM_RETURN CSSMAPI
cssm_UnlockMultiTaskMutex(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord);
CSSM_RETURN CSSMAPI
cssm_LockMultiTaskMutexByHandle(CSSM_MODULE_HANDLE ModuleHandle);
CSSM_RETURN CSSMAPI
cssm_UnlockMultiTaskMutexByHandle(CSSM_MODULE_HANDLE ModuleHandle);
CSSM_RETURN CSSMAPI
cssm_GetAttachRecordAndLockMultiTask(CSSM_HANDLE hAddIn,
                                     CSSM_SERVICE_TYPE AddinType,
                                     void **CallBack,
                                     cssm_INTERNAL_ATTACH_LIST_PTR *AttachInfoPtr);
CSSM_RETURN CSSMAPI
cssm_ReleaseAttachRecordAndUnlockMultiTask(cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord);


CSSM_BOOL CSSMAPI cssm_IsValidAddin (CSSM_MODULE_HANDLE ModuleHandle, 
                                     CSSM_SERVICE_TYPE ServiceType); 

/* internal list additions/deletions */
CSSM_RETURN CSSMAPI
cssm_CleanInternalModuleRecord(cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                               CSSM_LIB_HANDLE *MainLibHandle);

void CSSMAPI
cssm_CleanInternalServiceRecord(cssm_INTERNAL_MODULE_LIST_PTR ModuleRecord,
                                cssm_INTERNAL_SERVICE_LIST_PTR *ServiceRecord);

void CSSMAPI
cssm_CleanInternalAttachRecord(cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord,
                               cssm_INTERNAL_ATTACH_LIST_PTR *AttachRecord);


CSSM_RETURN
cssm_InsertAttachRecord(const CSSM_API_MEMORY_FUNCS *MemoryFunc,
                        const CSSM_VERSION *Version,
                        CSSM_SERVICE_TYPE SubServiceType,
                        CSSM_ATTACH_FLAGS AttachFlags,
                        CSSM_KEY_HIERARCHY KeyHierarchy,
                        const cssm_INTERNAL_MODULE_LIST *ModuleRecord,
                        cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord,
                        CSSM_HANDLE *NewAttachHandle,
                        cssm_INTERNAL_ATTACH_LIST_PTR *NewAttachRecord);

CSSM_RETURN
cssm_InsertServiceRecord(uint32 SubserviceID,
                         CSSM_SERVICE_MASK ServiceType,
                         cssm_INTERNAL_MODULE_LIST_PTR pModuleEntry,
                         cssm_INTERNAL_SERVICE_LIST_PTR *pServiceEntry);

CSSM_RETURN cssm_DeleteServiceRecord( cssm_INTERNAL_MODULE_LIST_PTR pModuleEntry,
                               cssm_INTERNAL_SERVICE_LIST_PTR pServiceEntry );

CSSM_RETURN CSSMAPI cssm_UnloadModule(CSSM_GUID_PTR ModuleGuid, 
                                      void* AppNotifyCallbackCtx);


CSSM_RETURN CSSMAPI CSSM_ModuleEventHandler(
                            const CSSM_GUID *ModuleGuid,
                            void* CssmNotifyCallbackCtx,
                            uint32 SubserviceID,
                            CSSM_SERVICE_TYPE ServiceType,
                            CSSM_MODULE_EVENT EventType);
                                    



/* NOTE - The following five functions are intended to be used only by EMMs */

CSSM_RETURN CSSMAPI 
cssm_GetAttachFunctions(CSSM_HANDLE hAddIn,
                        CSSM_SERVICE_MASK AddinType,
                        void **CallBack,
                        CSSM_GUID_PTR Guid);

void CSSMAPI
cssm_ReleaseAttachFunctions(CSSM_HANDLE hAddIn);



typedef CSSM_RETURN ( CSSMAPI* cssm_IsFuncCallValid) (CSSM_MODULE_HANDLE hAddin,
                                            CSSM_PROC_ADDR SrcAddress, /* application */
                                            CSSM_PROC_ADDR DestAddress,
                                            CSSM_PRIVILEGE InPriv,
                                            CSSM_PRIVILEGE *OutPriv,
                                            CSSM_BITMASK Hints,
                                            CSSM_BOOL *IsOK);

CSSM_RETURN CSSMAPI 
cssm_IsFuncCallOK(CSSM_MODULE_HANDLE hAddin,
                  CSSM_PROC_ADDR SrcAddress, /* application */
                  CSSM_PROC_ADDR DestAddress,/* addin */
                  CSSM_PRIVILEGE InPriv,
                  CSSM_PRIVILEGE *OutPriv,
                  CSSM_BITMASK Hints,
                  CSSM_BOOL *IsOK); 

#ifdef ISL_INCLUDED
CSSM_RETURN
GetPrivTagInCredentialAttribute(ISL_CONST_DATA_PTR AttribValue,
                                uint32 *NumPrivs,
                                uint32 **Privs);

CSSM_RETURN
cssm_ExtractPrivilege(ISL_MANIFEST_SECTION_PTR ManifestSectionptr,
                      uint32 *NumPriv,
                      uint32** Privilege,
                      ISL_CONST_DATA AttribName);

CSSM_RETURN
cssm_FilterTOGAndVendorPrivs(uint32 NumPrivs,
                             uint32* Privs,
                             uint32* NumVendorPrivs,
                             uint32** VendorPrivs,
                             uint32* NumTOGPrivs,
                             uint32** TOGPrivs);
#endif

/* Management interface functions */
void CSSMAPI cssm_CleanModuleRecord (const CSSM_GUID_PTR GUID);



CSSM_RETURN CSSMAPI cssm_RemoveModuleRecord (const CSSM_GUID_PTR GUID);
CSSM_RETURN CSSMAPI
cssm_NewModuleRecord(const CSSM_GUID *GUID,
                     cssm_INTERNAL_MODULE_LIST_PTR *NewModuleRecord);

CSSM_RETURN CSSMAPI cssm_UpdateModuleRecord (
                                 const CSSM_GUID_PTR GUID,
                                 const CSSM_MODULE_FUNCS_PTR FunctionTable);

CSSM_RETURN CSSMAPI cssm_LoadAddInByName(char * pAddInName,
                            uint32 AddInType,
                            CSSM_HANDLE *phAddIn);
CSSM_RETURN CSSMAPI cssm_UnloadAddInByHandle(CSSM_HANDLE hAddIn,
                                CSSM_SERVICE_MASK AddInType);



CSSM_RETURN CSSMAPI cssm_AddInDetach (CSSM_HANDLE AddInHandle);
                                   

uint32 cssm_Rand();
CSSM_RETURN cssm_CheckAPIMemFuncs (
                              const CSSM_API_MEMORY_FUNCS *MemoryFunctions);
CSSM_RETURN CSSMAPI cssm_GetUsageMask(CSSM_HANDLE ModuleHandle, 
                                      CSSM_SERVICE_TYPE *ServiceTypePtr);
CSSM_RETURN CSSMAPI cssm_GetGUIDFromHandle(CSSM_HANDLE ModuleHandle, 
                                           CSSM_GUID_PTR GuidPtr);

CSSM_RETURN CSSMAPI cssm_GetSubserviceUIDFromHandle(CSSM_HANDLE ModuleHandle, 
                                                    CSSM_SUBSERVICE_UID_PTR UidPtr);

HANDLE CSSMAPI cssm_GetSWMRLockHandle (void *ModuleInfo);

CSSM_RETURN cssm_CleanUp (void);



/* event notify callback list functions */
CSSM_RETURN cssm_PruneCallbackList(
                            cssm_INTERNAL_MODULE_LIST_PTR pModuleEntry,
                            CSSM_API_ModuleEventHandler AppCallback,
                            const void *AppCallbackCtx );
CSSM_RETURN cssm_UpdateCallbackList(
                            cssm_INTERNAL_MODULE_LIST_PTR pModuleEntry,
                            CSSM_API_ModuleEventHandler AppCallback,
                            const void *AppCallbackCtx );
CSSM_RETURN cssm_BroadcastModuleEvent(const CSSM_GUID *ModuleGuid,
                                uint32 SubserviceID,
                                CSSM_SERVICE_MASK ServiceType,
                                CSSM_MODULE_EVENT EventType );

CSSM_RETURN cssm_GetFunctionTable(CSSM_FUNC_NAME_ADDR_PTR FuncNameAddrPtr, 
                                  uint32 NumOfFuncNameAddre, 
                                  CSSM_HANDLE hAddIn);



/* attach to a module without verification */
CSSM_RETURN
cssm_AttachModuleNoKeys(const CSSM_GUID *ModuleGuid,
                        const CSSM_VERSION *Version,
                        uint32 SubServiceId,
                        CSSM_SERVICE_TYPE SubServiceType,
                        CSSM_ATTACH_FLAGS AttachFlags,
                        CSSM_MODULE_HANDLE ModuleHandle,
                        CSSM_KEY_HIERARCHY KeyHierarchy,
                        const CSSM_GUID *CallerGuid,
                        CSSM_MODULE_FUNCS_PTR *AttachFuncTbl,
                        cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord,
                        cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord);

/* FOR integrity*/
#ifdef ISL_INCLUDED
CSSM_BOOL
cssm_CheckFunctionTableInverifiedModules(uint32 NumVerifiedModules,
                                         ISL_VERIFIED_MODULE_PTR *pVerifiedModules,
                                         CSSM_MODULE_FUNCS_PTR AddInJT);


CSSM_RETURN
cssm_CheckIfAttachRecordWithMatchingKeyHierarchyExists(CSSM_MODULE_HANDLE hAttach,
                                               CSSM_KEY_HIERARCHY KeyHierarchy,
                                               cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord,
                                               CSSM_BOOL* bMatch);


CSSM_RETURN cssm_AttachModuleUsingExportOrIntegrityKeys(const CSSM_GUID *ModuleGuid,
                                            const CSSM_VERSION *Version,
                                            uint32 SubServiceId,
                                            CSSM_SERVICE_TYPE SubServiceType,
                                            CSSM_ATTACH_FLAGS AttachFlags,
                                            CSSM_MODULE_HANDLE ModuleHandle,
                                            CSSM_KEY_HIERARCHY KeyHierarchy,
                                            CSSM_MODULE_FUNCS_PTR *FuncTbl,
                                            cssm_INTERNAL_SERVICE_LIST_PTR ServiceRecord,
                                            cssm_INTERNAL_ATTACH_LIST_PTR AttachRecord,
                                            const CSSM_GUID *CallerGuid,
                                            CSSM_DATA_PTR Manifest,
                                            CSSM_DATA_PTR ModulePath);


CSSM_RETURN cssm_ExtractUSEETags(
                             CSSM_GUID_PTR Guid,
                             ISL_MANIFEST_SECTION_PTR** ManifestSectionPtrArr,
                             uint32* NumSections, 
                             uint32* NumUseeTags,
                             CSSM_PRIVILEGE** UseeTags,
                             CSSM_BOOL bAddin);

CSSM_BOOL cssm_GetUSEEString(CSSM_PRIVILEGE USEERequest, 
                               char* szUSEERequest);
CSSM_RETURN cssm_ExtractUseeTagsFromCredentialAttribute( CSSM_PRIVILEGE** UseeTags, 
                                                       uint32* NumTags,
                                                       ISL_CONST_DATA_PTR AttribValue);

CSSM_RETURN cssm_ExtractUSEETagAndCompareWithRequest(CSSM_GUID_PTR Guid,
                                                     CSSM_PRIVILEGE USEERequest, 
                                                     uint32* NumSections, 
                                                     CSSM_BOOL bAddin, 
                                                     ISL_MANIFEST_SECTION_PTR** pManifestSections );

CSSM_RETURN cssm_AddinLoadAndCheckIntegriy(CSSM_GUID *ModuleGuid,
                               CSSM_GUID *CallerGuid,
                               ISL_VERIFIED_MODULE_PTR** VerifiedModulePtr,
                               uint32 *NumberOfVerifiedModuleSections);


CSSM_RETURN cssm_AddInLoadUsingExportKeys(CSSM_GUID_PTR ModuleGuid,
                              ISL_VERIFIED_MODULE_PTR** VerifiedModulePtrArr,
                              uint32* NumSections,
                              CSSM_BOOL bLoadModule,
                              uint32* NumUseeTags, /* Number of useeTags in the addin manifest*/
                              CSSM_PRIVILEGE** UseeTags); /* Usee Tags in addin manifest. We can cache it in Attach record*/

CSSM_RETURN cssm_VerifyCaller(uint32 KeyType, 
                  uint32* NumAppSections, 
                  ISL_VERIFIED_MODULE_PTR** VerifiedAppPtrArr,
                  const CSSM_GUID* CallerGuid);

CSSM_RETURN cssm_VerifyLoadedCSSMUsingExportRoot(CSSM_DATA_PTR ModuleCredential, 
                                                CSSM_DATA_PTR SearchPath,
                                                ISL_VERIFIED_MODULE_PTR** VerifiedModulePtrArr,
                                                uint32* NumSections);

CSSM_RETURN cssm_FreeVerifiedCredentials( ISL_VERIFIED_MODULE_PTR* VerifiedModulePtrArr, 
                                      uint32 NumSections);

CSSM_RETURN CSSMAPI
cssm_ModuleImplicitUnintroduce( const CSSM_GUID* ModuleGuid );
CSSM_RETURN CSSMAPI 
cssm_ModuleImplicitIntroduce( const CSSM_GUID *ModuleGuid, 
                              CSSM_KEY_HIERARCHY KeyHierarchy,
                              uint32 NumSections,
                              ISL_VERIFIED_MODULE_PTR* VerifiedModulePtr);

#endif
void CSSMAPI
cssm_FreeAttributes(uint32 NumberOfAttributes,
                    CSSM_CONTEXT_ATTRIBUTE_PTR Attributes,
                    CSSM_FREE free_func,
                    void *mem_ref,
                    uint32 Free);
CSSM_RETURN CSSMAPI cssm_CheckCopy (void *Dest, void *Src, uint32 size);
CSSM_RETURN CSSMAPI cssm_SetAccessCredProxyModuleHandle( CSSM_MODULE_HANDLE ModuleHandle,
                                             const CSSM_ACCESS_CREDENTIALS* AccessCred,
                                             CSSM_ACCESS_CREDENTIALS_PTR ProxyCallback,
                                             CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ProxyCtx);
CSSM_RETURN CSSMAPI cssm_SetResourceControlProxyModuleHandle(CSSM_MODULE_HANDLE ModuleHandle,
                                                 const CSSM_RESOURCE_CONTROL_CONTEXT* CredAndAclEntry,
                                                 CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ChallengeProxyCtx,
                                                 CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR SubjectProxyCtx,
                                                 CSSM_RESOURCE_CONTROL_CONTEXT* ProxyCallback);

CSSM_RETURN CSSMAPI 
cssm_GUIDToStr( const CSSM_GUID *Guid,  char *ID);

CSSM_DATA_PTR CSSMAPI
Base64_Decode(CSSM_HANDLE AppHandle, 
              const CSSM_UPCALLS *MemoryFunctions, 
              const CSSM_DATA *pEncodedData);

CSSM_RETURN CSSMAPI cssm_CcToHandle(CSSM_CC_HANDLE Cc,
                                    CSSM_MODULE_HANDLE_PTR ModuleHandle);


CSSM_RETURN CSSMAPI
cssm_GetModuleInfo_func(CSSM_MODULE_HANDLE Module,
                        CSSM_GUID *Guid,
                        CSSM_VERSION *Version,
                        uint32 *SubServiceId,
                        CSSM_SERVICE_TYPE *SubServiceType,
                        CSSM_ATTACH_FLAGS *AttachFlags,
                        CSSM_KEY_HIERARCHY *KeyHierarchy,
                        CSSM_FUNC_NAME_ADDR_PTR FunctionTable,
                        uint32 *NumFunctions);

#ifdef __cplusplus
}
#endif

#endif
