/*-----------------------------------------------------------------------
 * File: CDSACSPPRIMARY.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


#include <assert.h>
#include <string.h>
#ifdef VMS
#include "cssmport.h"
#else
#include "port/cssmport.h"
#endif
#include "cssmtype.h"
#include "cssmerr.h"
#include "mds_schema.h"

#include "mds.h"
#include "mds_util.h"
#include "convert.h"
#include "internal.h"

#define __ModuleID          ( 0 )
#define __SSID              ( 1 )
#define __Manifest          ( 2 )
#define __ModuleName        ( 3 )
#define __ProductVersion    ( 4 )
#define __Vendor            ( 5 )
#define __CspType           ( 6 )
#define __CspFlags          ( 7 )
#define __CspCustomFlags    ( 8 )
#define __UseeTags          ( 9 )
#define __SampleTypes       ( 10 )
#define __AclSubjectTypes   ( 11 )
#define __AclAuthTags       ( 12 )

typedef struct _mdsu_cdsa_csp_primary_attr_state
{
    CSSM_DB_ATTRIBUTE_DATA DbRecordAttributes[MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES];
    CSSM_DATA AttributeValues[MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_MDSU_PRINTABLE_GUID_LENGTH];
    uint8 PrintableVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
}
_MDSU_CDSA_CSP_PRIMARY_ATTR_STATE;

static CSSM_RETURN CSSMAPI __MDSU_ConstructCDSACSPPrimaryAttributes(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaData,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFillValues,
                            void **ppAttrState )
{
    CSSM_RETURN rv = CSSM_OK;
    const MDSU_CDSA_CSP_PRIMARY_SCHEMA *pPrimary = 
                        (const MDSU_CDSA_CSP_PRIMARY_SCHEMA*)pSchemaData;
    _MDSU_CDSA_CSP_PRIMARY_ATTR_STATE *pState = NULL;
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( pContext && DbRecord && ppAttrState );
    assert( ( bFillValues == CSSM_FALSE ) ||
            ( ( bFillValues == CSSM_TRUE ) && pSchemaData ) );

    /* Allocate the memory for the attributes */
    pState = MDSU_malloc( pContext, sizeof(*pState) );
    if ( pState == NULL )
    {
        ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
    }
    else
    {
        /* Clear the structure */
        memset( pState,
                0,
                sizeof(*pState) );
        DbRecordAttributes = pState->DbRecordAttributes;

        /* Fill in the record information */
        DbRecord->DataRecordType = MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE;
        DbRecord->SemanticInformation = 0;
        DbRecord->NumberOfAttributes = MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES;
        DbRecord->AttributeData = DbRecordAttributes;

        /* Fill in the attribute info */
        DbRecordAttributes[__ModuleID].Info = s_MdsAttrInfo_ModuleId;
        DbRecordAttributes[__SSID].Info = s_MdsAttrInfo_SSID;
        DbRecordAttributes[__Manifest].Info = s_MdsAttrInfo_Manifest;
        DbRecordAttributes[__ModuleName].Info = s_MdsAttrInfo_ModuleName;
        DbRecordAttributes[__ProductVersion].Info = s_MdsAttrInfo_ProductVersion;
        DbRecordAttributes[__Vendor].Info = s_MdsAttrInfo_Vendor;
        DbRecordAttributes[__CspType].Info = s_MdsAttrInfo_CspType;
        DbRecordAttributes[__CspFlags].Info = s_MdsAttrInfo_CspFlags;
        DbRecordAttributes[__CspCustomFlags].Info = 
                                                s_MdsAttrInfo_CspCustomFlags;
        DbRecordAttributes[__UseeTags].Info = s_MdsAttrInfo_UseeTags;
        DbRecordAttributes[__SampleTypes].Info = s_MdsAttrInfo_SampleTypes;
        DbRecordAttributes[__AclSubjectTypes].Info = s_MdsAttrInfo_AclSubjectTypes;
        DbRecordAttributes[__AclAuthTags].Info = s_MdsAttrInfo_AuthTags;

        if ( bFillValues == CSSM_TRUE )
        {
            /* Connect the attribute value array to the attribute array */
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = &pState->AttributeValues[i];
                DbRecordAttributes[i].NumberOfValues = 1;
            }

            /* Fill in the record contents */
            /* ModuleID */
            _MDSU_GetPrintableGUID( &pPrimary->ModuleID,
                                    (char*)pState->PrintableGuid );
            DbRecordAttributes[__ModuleID].Value->Data =
                                                    pState->PrintableGuid;
            DbRecordAttributes[__ModuleID].Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;

            /* SSID */
            DbRecordAttributes[__SSID].Value->Length = sizeof(uint32);
            DbRecordAttributes[__SSID].Value->Data = (uint8*)&pPrimary->SSID;

            /* Manifest */
            (*DbRecordAttributes[__Manifest].Value) = pPrimary->Manifest;

            /* ModuleName */
            DbRecordAttributes[__ModuleName].Value->Length = 
                                        strlen( pPrimary->ModuleName ) + 1 ;
            DbRecordAttributes[__ModuleName].Value->Data =
                                                (uint8*)pPrimary->ModuleName;

            /* Version */
            _MDSU_GetPrintableVersion( &pPrimary->ProductVersion,
                                       (char*)pState->PrintableVersion );
            DbRecordAttributes[__ProductVersion].Value->Data =
                                                pState->PrintableVersion;
            DbRecordAttributes[__ProductVersion].Value->Length = 
                            strlen( (char*)pState->PrintableVersion ) + 1;

            /* Vendor */
            DbRecordAttributes[__Vendor].Value->Length = 
                                                strlen( pPrimary->Vendor ) + 1 ;
            DbRecordAttributes[__Vendor].Value->Data = (uint8*)pPrimary->Vendor;

            /* CspType */
            DbRecordAttributes[__CspType].Value->Length = sizeof(uint32);
            DbRecordAttributes[__CspType].Value->Data = (uint8*)&pPrimary->CspType;

            /* CspFlags */
            DbRecordAttributes[__CspFlags].Value->Length = sizeof(uint32);
            DbRecordAttributes[__CspFlags].Value->Data =    (uint8*)&pPrimary->CspFlags;

            /* CspCustomFlags */
            DbRecordAttributes[__CspCustomFlags].Value->Length = sizeof(uint32);
            DbRecordAttributes[__CspCustomFlags].Value->Data =
                                                (uint8*)&pPrimary->CspCustomFlags;

            /* UseeTags */
            DbRecordAttributes[__UseeTags].Value->Length = 
                                            sizeof(uint32) * pPrimary->NumUseeTags;
            DbRecordAttributes[__UseeTags].Value->Data =    (uint8*)&pPrimary->UseeTags;

            /* SampleTypes */
            DbRecordAttributes[__SampleTypes].Value->Length =
                                            sizeof(uint32) * pPrimary->NumSampleTypes;
            DbRecordAttributes[__SampleTypes].Value->Data = (uint8*)&pPrimary->SampleTypes;

            /* AclSubjectTypes */
            DbRecordAttributes[__AclSubjectTypes].Value->Length =
                                            sizeof(uint32) * pPrimary->NumAclSubjectTypes;
            DbRecordAttributes[__AclSubjectTypes].Value->Data = (uint8*)&pPrimary->AclSubjectTypes;
    
            /* AclAuthTags */
            DbRecordAttributes[__AclAuthTags].Value->Length =
                                            sizeof(uint32) * pPrimary->NumAclAuthTags;
            DbRecordAttributes[__AclAuthTags].Value->Data = (uint8*)&pPrimary->AclAuthTags;

        }
        else
        {
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = NULL;
                DbRecordAttributes[i].NumberOfValues = 0;
            }
        }
    } /* Allocate attributes array */

    *ppAttrState = (void*)pState;

    return rv;
}

static CSSM_RETURN CSSMAPI __MDSU_DestructCDSACSPPrimaryAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFreeAll,
                            void *pAttrState )
{
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( pContext && DbRecord );

    /* Free the buffers allocated for the attribute values */
    DbRecordAttributes = DbRecord->AttributeData;
    if ( bFreeAll == CSSM_TRUE )
    {
        for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
        {
            if ( DbRecordAttributes[i].Value )
            {
                if ( DbRecordAttributes[i].Value->Data )
                {
                    MDSU_free( pContext,
                               DbRecordAttributes[i].Value->Data );
                }
                MDSU_free( pContext,
                           DbRecordAttributes[i].Value );
            }
        }
    }

    /* Free the attribute array */
    MDSU_free( pContext, pAttrState );
    DbRecord->AttributeData = NULL;

    return CSSM_OK;
}

typedef struct _mdsu_cdsa_csp_primary_pred_state
{
    CSSM_SELECTION_PREDICATE Predicates[MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES];
    CSSM_DATA PredicateValues[MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_MDSU_PRINTABLE_GUID_LENGTH];
    uint8 PrintableVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
}
_MDSU_CDSA_CSP_PRIMARY_PRED_STATE;

static CSSM_RETURN CSSMAPI __MDSU_ConstructCDSACSPPrimaryPredicate(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaTemplate,
                            MDSU_TEMPLATE_MASK ValidTemplates,
                            CSSM_QUERY *pQuery,
                            void **ppPredState )
{
    CSSM_RETURN rv = CSSM_OK;
    const MDSU_CDSA_CSP_PRIMARY_SCHEMA *pPrimary = 
                        (const MDSU_CDSA_CSP_PRIMARY_SCHEMA*)pSchemaTemplate;
    _MDSU_CDSA_CSP_PRIMARY_PRED_STATE *pState = NULL;
    CSSM_SELECTION_PREDICATE *pPredicates = NULL;
    uint32 uNumPredicates;
    uint32 uIndex = 0;
    uint32 i;

    assert( pContext && 
            pQuery &&
            ppPredState &&
            ( ( pSchemaTemplate && ValidTemplates ) ||
              ( !pSchemaTemplate && !ValidTemplates ) ) );

    /* Count the predicates */
    /* Clear the manifest and multi_uint32 bits. Searching on manifest and
     * multi-value is not supported.
     */
    ValidTemplates &= ~( MDSU_CSPP_MANIFEST |
                         MDSU_CSPP_USEE_TAGS |
                         MDSU_CSPP_SAMPLE_TYPES |
                         MDSU_CSPP_ACL_SUBJ_TYPES |
                         MDSU_CSPP_ACL_AUTH_TAGS );
    _MDSU_CountBits( ValidTemplates, uNumPredicates );

    /* Allocate the predicate state and link the value structures */
    if ( uNumPredicates != 0 )
    {
        pState = MDSU_malloc( pContext, sizeof(*pState) );
        if ( pState == NULL )
        {
            ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
        }
        else
        {
            memset( pState,
                    0,
                    sizeof(*pState) );
            pPredicates = pState->Predicates;

            /* Allocate a single attribute value struct per attribute */
            for ( i = 0; i < uNumPredicates; i++ )
            {
                pPredicates[i].Attribute.Value =
                                            &pState->PredicateValues[i];
                pPredicates[i].Attribute.NumberOfValues = 1;
            }
        } /* Allocate and link predicate values */
    } /* uNumPredicates != 0 */

    if ( rv == CSSM_OK )
    {
        /* Fill in the query parameters */
        pQuery->RecordType = MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE;
        pQuery->Conjunctive = CSSM_DB_AND;
        pQuery->NumSelectionPredicates = uNumPredicates;
        pQuery->QueryLimits.TimeLimit = CSSM_QUERY_TIMELIMIT_NONE;
        pQuery->QueryLimits.SizeLimit = CSSM_QUERY_SIZELIMIT_NONE;
        pQuery->QueryFlags = 0;
        pQuery->SelectionPredicate = pPredicates;
        for ( uIndex = 0; uIndex < uNumPredicates; uIndex++ )
        {
            pPredicates[uIndex].DbOperator = CSSM_DB_EQUAL;
        }

        /* Fill in the predicate data */
        uIndex = 0;
        if ( ValidTemplates & MDSU_CSPP_MODULE_ID )
        {
            /* ModuleID */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ModuleId;
            _MDSU_GetPrintableGUID( &pPrimary->ModuleID,
                                    (char*)pState->PrintableGuid );
            pPredicates[uIndex].Attribute.Value->Data = pState->PrintableGuid;
            pPredicates[uIndex].Attribute.Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_SSID )
        {
            /* SSID */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_SSID;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(pPrimary->SSID);
            pPredicates[uIndex].Attribute.Value->Data = 
                                                    (uint8*)&pPrimary->SSID;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_MODULE_NAME )
        {
            /* ModuleName */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ModuleName;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pPrimary->ModuleName ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pPrimary->ModuleName;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_PRODUCT_VERSION )
        {
            /* ProductVersion */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ProductVersion;
            _MDSU_GetPrintableVersion( &pPrimary->ProductVersion,
                                       (char*)pState->PrintableVersion );
            pPredicates[uIndex].Attribute.Value->Data =
                                                pState->PrintableVersion;
            pPredicates[uIndex].Attribute.Value->Length = 
                            strlen( (char*)pState->PrintableVersion ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_VENDOR )
        {
            /* Vendor */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_Vendor;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pPrimary->Vendor ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pPrimary->Vendor;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_CSP_TYPE )
        {
            /* CspType */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_CspType;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(pPrimary->CspType);
            pPredicates[uIndex].Attribute.Value->Data = 
                                                    (uint8*)&pPrimary->CspType;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_CSP_FLAGS )
        {
            /* CspFlags */
            pPredicates[uIndex].Attribute.Info = 
                                                s_MdsAttrInfo_CspFlags;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(pPrimary->CspFlags);
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pPrimary->CspFlags;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CSPP_CSP_CUSTOM_FLAGS )
        {
            /* CspCustomFlags */
            pPredicates[uIndex].Attribute.Info = 
                                                s_MdsAttrInfo_CspCustomFlags;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(pPrimary->CspCustomFlags);
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)&pPrimary->CspCustomFlags;
            uIndex++;
        }
        
        /* Searching on remaining MULTI_UINT32 fields not supported */
    } /* Initial allocation successful */

    *ppPredState = (void*)pState;

    return rv;
}

static CSSM_RETURN CSSMAPI __MDSU_DestructCDSACSPPrimaryPredicate(
                            MDSU_CONTEXT *pContext,
                            CSSM_QUERY *pQuery,
                            void *pPredState )
{
    assert( pContext && pQuery );

    /* Free the attribute array */
    if ( pPredState != NULL )
    {
        MDSU_free( pContext, pPredState );
    }

    return CSSM_OK;
}

static CSSM_RETURN CSSMAPI __MDSU_ConvertCDSACSPPrimaryAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            void *pSchemaData )
{
    MDSU_CDSA_CSP_PRIMARY_SCHEMA *pPrimary = 
                                (MDSU_CDSA_CSP_PRIMARY_SCHEMA*)pSchemaData;
    const CSSM_DB_ATTRIBUTE_DATA *DbAttr = NULL;

    assert( pSchemaData && DbRecord && pContext );

    DbAttr = DbRecord->AttributeData;

    /* ModuleID */
    _MDSU_GetStructuredGUID( (char*)DbAttr[__ModuleID].Value->Data,
                             &pPrimary->ModuleID );

    /* SSID */
    pPrimary->SSID = *(uint32*)DbAttr[__SSID].Value->Data;

    /* ModuleName */
	if ( DbAttr[__ModuleName].Value->Data != NULL )
        strncpy( pPrimary->ModuleName,
                 (char*)DbAttr[__ModuleName].Value->Data,
				 DbAttr[__ModuleName].Value->Length );
	else
		strcpy( pPrimary->ModuleName, "" );

    /* ProductVersion */
    _MDSU_GetStructuredVersion( (char*)DbAttr[__ProductVersion].Value->Data,
                                &pPrimary->ProductVersion );

    /* Vendor */
	if ( DbAttr[__Vendor].Value->Data != NULL )
        strncpy( pPrimary->Vendor,
                 (char*)DbAttr[__Vendor].Value->Data,
				 DbAttr[__Vendor].Value->Length );
	else
		strcpy( pPrimary->Vendor, "" );

    /* CspType */
    pPrimary->CspType = *(CSSM_CSPTYPE*)DbAttr[__CspType].Value->Data;

    /* CspFlags */
    pPrimary->CspFlags = *(uint32*)DbAttr[__CspFlags].Value->Data;

    /* CspCustomFlags */
    pPrimary->CspCustomFlags = *(uint32*)DbAttr[__CspCustomFlags].Value->Data;

    /* UseeTags */
    memcpy( pPrimary->UseeTags,
            DbAttr[__UseeTags].Value->Data,
            DbAttr[__UseeTags].Value->Length );
    pPrimary->NumUseeTags = DbAttr[__UseeTags].Value->Length / 
                                sizeof(pPrimary->UseeTags[0]);

    /* SampleTypes */
    memcpy( pPrimary->SampleTypes,
            DbAttr[__SampleTypes].Value->Data,
            DbAttr[__SampleTypes].Value->Length );
    pPrimary->NumSampleTypes = DbAttr[__SampleTypes].Value->Length / 
                                sizeof(pPrimary->SampleTypes[0]);

    /* AclSubjectTypes */
    memcpy( pPrimary->AclSubjectTypes,
            DbAttr[__AclSubjectTypes].Value->Data,
            DbAttr[__AclSubjectTypes].Value->Length );
    pPrimary->NumAclSubjectTypes = DbAttr[__AclSubjectTypes].Value->Length / 
                                sizeof(pPrimary->AclSubjectTypes[0]);

    /* AclAuthTags */
    memcpy( pPrimary->AclAuthTags,
            DbAttr[__AclAuthTags].Value->Data,
            DbAttr[__AclAuthTags].Value->Length );
    pPrimary->NumAclAuthTags = DbAttr[__AclAuthTags].Value->Length / 
                                sizeof(pPrimary->AclAuthTags[0]);

    /* Manifest */
    if ( DbAttr[__Manifest].Value->Data != NULL )
    {
        pPrimary->Manifest.Data = MDSU_malloc( 
                                        pContext,
                                        DbAttr[__Manifest].Value->Length );
        if ( pPrimary->Manifest.Data == NULL )
        {
            return CSSMERR_CSSM_MEMORY_ERROR;
        }
        pPrimary->Manifest.Length = DbAttr[__Manifest].Value->Length;
        memcpy( pPrimary->Manifest.Data,
                DbAttr[__Manifest].Value->Data,
                DbAttr[__Manifest].Value->Length );
    }
    else
    {
        pPrimary->Manifest.Data = NULL;
        pPrimary->Manifest.Length = 0;
    }

    return CSSM_OK;
}

const IfMdsuSchemaManipulator IfiMdsuCdsadirCSPPrimarySchema =
{
    __MDSU_ConstructCDSACSPPrimaryAttributes,
    __MDSU_DestructCDSACSPPrimaryAttributes,
    __MDSU_ConstructCDSACSPPrimaryPredicate,
    __MDSU_DestructCDSACSPPrimaryPredicate,
    __MDSU_ConvertCDSACSPPrimaryAttributes,
    MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE
};
