/*-----------------------------------------------------------------------
 * File: CONVERT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
** OpenVMS History
 *
 *    02-Nov-1999	Carol Kelly
 *		o Merge in changes from Unix code base
 *
 */

#include <stdio.h>

#include "cssmtype.h"
#include "cssmerr.h"

#include "mds.h"
#include "mds_util.h"
#include "convert.h"

CSSM_RETURN _MDSU_GetPrintableVersion( const CSSM_VERSION *pVersion,
                                       char *PrintableVersion )
{
    /* Parameter check */
    if ( ( pVersion == NULL ) ||
         ( PrintableVersion == NULL ) )
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }

    /* format the output */
    sprintf( PrintableVersion, "%d.%d", pVersion->Major, pVersion->Minor );

    return CSSM_OK;
}

CSSM_RETURN _MDSU_GetStructuredVersion( const char *PrintableVersion,
                                        CSSM_VERSION *pVersion )
{
    int nCount;

    /* Parameter check */
    if ( ( pVersion == NULL ) ||
         ( PrintableVersion == NULL ) )
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }

    /* format the output */
    nCount = sscanf( PrintableVersion, "%d.%d", 
                     &pVersion->Major,
                     &pVersion->Minor );

    return CSSM_OK;
}

CSSM_RETURN _MDSU_GetPrintableGUID( const CSSM_GUID *pGUID,
                                    char *PrintableGUID )
{
    /* Parameter checks */
    if ( ( pGUID == NULL ) ||
         ( PrintableGUID == NULL ) )
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }

    /* Format the output */
    sprintf( PrintableGUID,
             GUID_FORMAT_STRING, 
             pGUID->Data1,
             pGUID->Data2,
             pGUID->Data3,
             pGUID->Data4[0], pGUID->Data4[1],
             pGUID->Data4[2], pGUID->Data4[3], pGUID->Data4[4],
             pGUID->Data4[5], pGUID->Data4[6], pGUID->Data4[7] );

    return CSSM_OK;
}

CSSM_RETURN _MDSU_GetStructuredGUID( const char *PrintableGUID,
                                     CSSM_GUID *pGUID )
{
    int nCount;
#ifdef VMS  /* COMPAQ_FIX */
    /*
     * This fix is based on mdsutil_StrToGUID() in mds_util_query.c
     * and cssm_StrToGUID() in aalmds.c, both of which scan the guid
     * correctly.  The original code here corrupted the stack on all
     * platforms.  On Tru64 UNIX, it also produced unaligned access
     * error messages and on Solaris, it also dumped core.
     */
    uint32 i, Data1;
    uint32 Data2, Data3;
    uint32 Data4[8];
#endif /* COMPAQ_FIX */

    /* Parameter checks */
    if ( ( pGUID == NULL ) ||
         ( PrintableGUID == NULL ) )
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }

    /* Format the output */
    nCount = sscanf( PrintableGUID,
                     GUID_FORMAT_STRING, 
#ifdef VMS /* COMPAQ_FIX */
                     &Data1,
                     &Data2,
                     &Data3,
                     &Data4[0], &Data4[1],
                     &Data4[2], &Data4[3], &Data4[4],
                     &Data4[5], &Data4[6], &Data4[7] );
#else  /* COMPAQ_FIX */
                     &pGUID->Data1,
                     &pGUID->Data2,
                     &pGUID->Data3,
                     &pGUID->Data4[0], &pGUID->Data4[1],
                     &pGUID->Data4[2], &pGUID->Data4[3], &pGUID->Data4[4],
                     &pGUID->Data4[5], &pGUID->Data4[6], &pGUID->Data4[7] );
#endif /* COMPAQ_FIX */
    if ( nCount != 11 )
    {
        return CSSMERR_CSSM_INVALID_GUID;
    }
#ifdef VMS /* COMPAQ_FIX */
    pGUID->Data1 = Data1;
    pGUID->Data2 = (uint16) Data2;
    pGUID->Data3 = (uint16) Data3;
    for ( i = 0; i < 8; i++ )
        pGUID->Data4[i] = (uint8) Data4[i];
#endif /* COMPAQ_FIX */

    return CSSM_OK;
}
