/*-----------------------------------------------------------------------
 * File: PORTINIT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Portions:
 * (C) COPYRIGHT International Business Machines Corp. 1996, 1997
 * All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#include "internal.h"


/*-----------------------------------------------------------------------------
 * Variables keeping track of initialized state
 *---------------------------------------------------------------------------*/
static CSSM_BOOL port_initialized = CSSM_FALSE;

/*-----------------------------------------------------------------------------
 * Name: port_init
 *
 * Description: Prepares this library for use by an application. 
 * 
 * Parameters: None. 
 *
 * Returns:
 * CSSM_OK if the library was successfully initialized
 * CSSM_FAIL if an error occured
 *
 * Error Codes: None.
 *---------------------------------------------------------------------------*/
CSSM_RETURN CSSMAPI port_init()
{

#if defined (AIX)

    char *envptr = getenv(CSSM_ODMDIR);   
    TRACE(port_trace_info, FNCODE_cssm_port_init);
    tzset();
    odm_initialize();

    if (envptr == NULL)
            envptr = getenv("HOME");

    odm_set_path(envptr); 
   
    if (odm_open_class(CssmData_CLASS) == (CLASS_SYMBOL) -1) 
    {
        if  ((odmerrno != ODMI_CLASS_DNE) || 
            (odm_create_class(CssmData_CLASS) < 0))
        {
            RETURN(CSSM_CANT_CREATE_KEY);
        }
    }

    if (odm_open_class(SectionMap_CLASS) == (CLASS_SYMBOL) -1) 
    {
        if ((odmerrno != ODMI_CLASS_DNE) ||
          (odm_create_class(SectionMap_CLASS) < 0))
        {
            RETURN(CSSM_CANT_CREATE_KEY);
        }
    }

#else /* AIX */
    TRACE(port_trace_info, FNCODE_cssm_port_init);
#endif  

   port_initialized = CSSM_TRUE;
   RETURN(CSSM_OK);
}


/*-----------------------------------------------------------------------------
 * Name: port_terminate
 *
 * Description: Performs the necessary cleanup. 
 * 
 * Parameters: None. 
 *
 * Returns:
 * CSSM_OK if successfully completed
 * CSSM_FAIL if an error occured
 *
 * Error Codes: None.
 *---------------------------------------------------------------------------*/
CSSM_RETURN CSSMAPI port_terminate()
{
#if defined (AIX)
    odm_terminate(); 
#elif defined (LINUX)
    char	cmd[128];
    strcpy(cmd, "/bin/rm -rf ");
    strcat(cmd, CDSA_SYSDIR);
    if (system(cmd) != 0)
    {
      RETURN(CSSMERR_CSSM_OS_ACCESS_DENIED);
    }
#endif
    port_initialized = CSSM_FALSE;

    RETURN(CSSM_OK);
}
