/*-----------------------------------------------------------------------
 * File: CRLLIST.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* This file declares the data structures and functions used to manage
    a CRL revoked certificates list.  For information on the list data
    structures, see file X509defs.h
*/


#ifndef _CRLLIST_H
#define _CRLLIST_H

#include "cssm.h"
#include "x509defs.h"    /* List data structures */


/* Information needed for traversing to the next node in the list */
typedef struct revoked_traverse_info {
    X509_REVOKED_LIST_NODE_PTR nextNode;
} REVOKED_TRAVERSE_INFO, *REVOKED_TRAVERSE_INFO_PTR;


/* Creates a new empty list */
CSSM_RETURN CSSM_CLUTIL_X509_RevokedListCreate(const CSSM_CLUTIL_CONTEXT *pContext,
													   CSSM_HANDLE AppHandle,
                                                       X509_REVOKED_CERT_LIST_PTR *result_val);

/* Returns the number of nodes in the list */
uint32 CSSM_CLUTIL_X509_RevokedListNumNodes(const CSSM_CLUTIL_CONTEXT *pContext,
											X509_REVOKED_CERT_LIST_PTR RCList);

/* Inserts a revoked cert entry into the list */
CSSM_BOOL CSSM_CLUTIL_X509_InsertRevokedCertEntry(const CSSM_CLUTIL_CONTEXT *pContext,
													   CSSM_HANDLE AppHandle, 
                                                  X509_REVOKED_CERT_LIST *RCList,
                                                  CSSM_X509_REVOKED_CERT_ENTRY_PTR RCEntry);

/* Removes and frees the list entry with the specified serial number */
CSSM_RETURN CSSM_CLUTIL_X509_RemoveRevokedCertEntry(const CSSM_CLUTIL_CONTEXT *pContext,
													   CSSM_HANDLE AppHandle, 
                                                  X509_REVOKED_CERT_LIST *RCList,
                                                  CSSM_DATA_PTR serial_num);

/* Searches for a list entry with the specified serial number */
CSSM_RETURN CSSM_CLUTIL_X509_FindRevokedCertEntry(const CSSM_CLUTIL_CONTEXT *pContext,
						                          CSSM_HANDLE AppHandle, 
                                                  X509_REVOKED_CERT_LIST_PTR RCList,
                                                  CSSM_DATA_PTR serial_num,
                                                  CSSM_X509_REVOKED_CERT_ENTRY_PTR *result_val);

/* This function returns a pointer to the last entry inserted into the revoked cert list */
CSSM_X509_REVOKED_CERT_ENTRY_PTR CSSM_CLUTIL_X509_GetLastRevokedEntryPtr(X509_REVOKED_CERT_LIST_PTR RCList);

/* Initializes a list traversal, and provides a handle for traversal */
CSSM_RETURN CSSM_CLUTIL_X509_RevokedListTraverseInitialize(CSSM_CLUTIL_CONTEXT *pContext,
														   X509_REVOKED_CERT_LIST_PTR RCList,
                                                           CSSM_HANDLE *result);

/* Provides the next node in an initialized traversal */
CSSM_RETURN CSSM_CLUTIL_X509_RevokedListTraverseNext(CSSM_CLUTIL_CONTEXT *pContext,
													 CSSM_HANDLE AppHandle, 
                                                     X509_REVOKED_CERT_LIST_PTR RCList,
                                                     CSSM_HANDLE results_handle,
                                                     CSSM_X509_REVOKED_CERT_ENTRY_PTR *result_val);

/* Aborts a list traversal, freeing up any related traversal info. */
CSSM_BOOL CSSM_CLUTIL_X509_RevokedListTraverseAbort(CSSM_CLUTIL_CONTEXT *pContext,
													CSSM_HANDLE results_handle);


#endif /* ndef _CRLLIST_H */
