/*-----------------------------------------------------------------------
 * File: CSSMCLF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Certificate Library Service Provider module.
 * The CSSM_DEFINE_FUNC, __CSSM_PASTE2 and __CSSM_PASTE3 macros must be 
 * defined by the file that includes this header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( CertCreateTemplate, (
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						const CSSM_FIELD *CertFields, 
						CSSM_DATA_PTR CertTemplate ) );

CSSM_DEFINE_FUNCTION( CertGetAllTemplateFields, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *CertTemplate,
						uint32 *NumberOfFields,
						CSSM_FIELD_PTR *CertFields ) );

CSSM_DEFINE_FUNCTION( CertSign, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_CC_HANDLE CCHandle,
						const CSSM_DATA *CertTemplate,
						const CSSM_FIELD *SignScope,
						uint32 ScopeSize,
						CSSM_DATA_PTR SignedCert ) );

CSSM_DEFINE_FUNCTION( CertVerify, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_DATA *CertToBeVerified, 
						const CSSM_DATA *SignerCert, 
						const CSSM_FIELD *VerifyScope,
						uint32 ScopeSize ) );

CSSM_DEFINE_FUNCTION( CertVerifyWithKey, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_CC_HANDLE CCHandle,
						const CSSM_DATA *CertToBeVerified ) );

/* Returns the field value */
CSSM_DEFINE_FUNCTION( CertGetFirstFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_DATA *Cert, 
						const CSSM_OID *CertField,
						CSSM_HANDLE_PTR ResultsHandle,
						uint32 *NumberOfMatchedFields,
						CSSM_DATA_PTR *Value ) );

/* Returns the field value */
CSSM_DEFINE_FUNCTION( CertGetNextFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle,
						CSSM_DATA_PTR *Value ) );

CSSM_DEFINE_FUNCTION( CertAbortQuery, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle ) );

CSSM_DEFINE_FUNCTION( CertGetKeyInfo, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Cert,
						CSSM_KEY_PTR *Key ) );

CSSM_DEFINE_FUNCTION( CertGetAllFields, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Cert,
						uint32 *NumberOfFields,
						CSSM_FIELD_PTR *FieldList ) );

CSSM_DEFINE_FUNCTION( FreeFields, (
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						CSSM_FIELD_PTR FieldArray ) );

CSSM_DEFINE_FUNCTION( FreeFieldValue, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_OID *CertOrCrlOid,
						CSSM_DATA_PTR Value ) );

CSSM_DEFINE_FUNCTION( CertCache, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Cert, 
						CSSM_HANDLE_PTR CertHandle ) );

CSSM_DEFINE_FUNCTION( CertGetFirstCachedFieldValue, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_HANDLE CertHandle, 
						const CSSM_OID *CertField,
						CSSM_HANDLE_PTR ResultsHandle,
						uint32 *NumberOfMatchedFields,
						CSSM_DATA_PTR *FieldValue ) );

CSSM_DEFINE_FUNCTION( CertGetNextCachedFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle,
						CSSM_DATA_PTR *FieldValue ) );

CSSM_DEFINE_FUNCTION( CertAbortCache, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_HANDLE CertHandle ) );

CSSM_DEFINE_FUNCTION( CertGroupToSignedBundle, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_CERTGROUP *CertGroupToBundle, 
						const CSSM_CERT_BUNDLE_HEADER *BundleInfo,
						CSSM_DATA_PTR SignedBundle ) );

CSSM_DEFINE_FUNCTION( CertGroupFromVerifiedBundle, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle,
						const CSSM_CERT_BUNDLE *CertBundle, 
						const CSSM_DATA *SignerCert,
						CSSM_CERTGROUP_PTR *CertGroup ) ); 

CSSM_DEFINE_FUNCTION( CertDescribeFormat, (
						CSSM_CL_HANDLE CLHandle,
						uint32 *NumberOfOids,
						CSSM_OID_PTR *OidList ) );

/*
 * 5.4 Certificate Revocation List Operations API
 */
/* Returns the new CRL */
CSSM_DEFINE_FUNCTION( CrlCreateTemplate, (
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						const CSSM_FIELD *CrlTemplate, 
						CSSM_DATA_PTR NewCrl ) );

/* Set general attribute fields in an unsigned CRL */
CSSM_DEFINE_FUNCTION( CrlSetFields, (
						CSSM_CL_HANDLE CLHandle,
						uint32 NumberOfFields,
						const CSSM_FIELD *CrlTemplate,
						const CSSM_DATA *OldCrl,
						CSSM_DATA_PTR ModifiedCrl ) );

/* Returns the updated CRL */
CSSM_DEFINE_FUNCTION( CrlAddCert, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_DATA *Cert, 
						uint32 NumberOfFields,
						const CSSM_FIELD *CrlEntryFields,
						const CSSM_DATA *OldCrl,
						CSSM_DATA_PTR NewCrl ) );

/* Returns the updated CRL */
CSSM_DEFINE_FUNCTION( CrlRemoveCert, (
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_DATA *Cert, 
						const CSSM_DATA *OldCrl,
						CSSM_DATA_PTR NewCrl ) );

/* Returns the signed CRL */
CSSM_DEFINE_FUNCTION( CrlSign, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_DATA *UnsignedCrl, 
						const CSSM_FIELD *SignScope,
						uint32 ScopeSize,
						CSSM_DATA_PTR SignedCrl ) );

CSSM_DEFINE_FUNCTION( CrlVerify, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						const CSSM_DATA *CrlToBeVerified, 
						const CSSM_DATA *SignerCert,
						const CSSM_FIELD *VerifyScope,
						uint32 ScopeSize ) );

CSSM_DEFINE_FUNCTION( CrlVerifyWithKey, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_CC_HANDLE CCHandle,
						const CSSM_DATA *CrlToBeVerified ) ); 

CSSM_DEFINE_FUNCTION( IsCertInCrl, (
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_DATA *Cert, 
						const CSSM_DATA *Crl,
						CSSM_BOOL *CertFound ) );

/* Returns the first item in the CRL */
CSSM_DEFINE_FUNCTION( CrlGetFirstFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						const CSSM_DATA *Crl,
						const CSSM_OID *CrlField,
						CSSM_HANDLE_PTR ResultsHandle,
						uint32 *NumberOfMatchedFields,
						CSSM_DATA_PTR *Value ) );

/* Returns the next item in the CRL */
CSSM_DEFINE_FUNCTION( CrlGetNextFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle,
						CSSM_DATA_PTR *Value ) );

CSSM_DEFINE_FUNCTION( CrlAbortQuery, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle ) );

CSSM_DEFINE_FUNCTION( CrlGetAllFields, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Crl,
						uint32 *NumberOfCrlFields,
						CSSM_FIELD_PTR *CrlFields ) ); 

CSSM_DEFINE_FUNCTION( CrlCache, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Crl, 
						CSSM_HANDLE_PTR CrlHandle ) ); 

CSSM_DEFINE_FUNCTION( IsCertInCachedCrl, (
						CSSM_CL_HANDLE CLHandle,
						const CSSM_DATA *Cert,
						CSSM_HANDLE CrlHandle,
						CSSM_BOOL *CertFound,
						CSSM_DATA_PTR CrlRecordIndex ) );

CSSM_DEFINE_FUNCTION( CrlGetFirstCachedFieldValue, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_HANDLE CrlHandle, 
						const CSSM_DATA *CrlRecordIndex, 
						const CSSM_OID *CrlField,
						CSSM_HANDLE_PTR ResultsHandle,
						uint32 *NumberOfMatchedFields,
						CSSM_DATA_PTR *FieldValue ) );

CSSM_DEFINE_FUNCTION( CrlGetNextCachedFieldValue, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_HANDLE ResultsHandle,
						CSSM_DATA_PTR *FieldValue ) );

CSSM_DEFINE_FUNCTION( CrlGetAllCachedRecordFields, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_HANDLE CrlHandle, 
						const CSSM_DATA *CrlRecordIndex,
						uint32 *NumberOfFields,
						CSSM_FIELD_PTR *Fields ) );

CSSM_DEFINE_FUNCTION( CrlAbortCache, (
						CSSM_CL_HANDLE CLHandle,
						CSSM_HANDLE CrlHandle ) ); 

CSSM_DEFINE_FUNCTION( CrlDescribeFormat, (
						CSSM_CL_HANDLE CLHandle,
						uint32 *NumberOfOids,
						CSSM_OID_PTR *OidList ) );

/*
 * Extensibility Functions 
 */

CSSM_DEFINE_FUNCTION( PassThrough, (
						CSSM_CL_HANDLE CLHandle, 
						CSSM_CC_HANDLE CCHandle, 
						uint32 PassThroughId, 
						const void *InputParams,
						void **OutputParams ) );


