/*-----------------------------------------------------------------------
 * File: HRSERR.H
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _HRSERR_H_
#define _HRSERR_H_

#include "cssmerr.h"

#define CSSM_HRS_BASE_ERROR	    (CSSM_AC_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT)
#define CSSM_HRS_PRIVATE_ERROR	(CSSM_HRS_BASE_ERROR + CSSM_ERRORCODE_CUSTOM_OFFSET)


/* HRS Error Values Derived from Common Error Codes in cssmerr.h */
#define CSSMERR_HRS_INTERNAL_ERROR              \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INTERNAL_ERROR)

#define CSSMERR_HRS_MEMORY_ERROR                \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_MEMORY_ERROR)

#define CSSMERR_HRS_INVALID_POINTER             \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INVALID_POINTER)

#define CSSMERR_HRS_INVALID_INPUT_POINTER       \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INVALID_INPUT_POINTER)

#define CSSMERR_HRS_INVALID_OUTPUT_POINTER      \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INVALID_OUTPUT_POINTER)

#define CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED    \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_FUNCTION_NOT_IMPLEMENTED)

#define CSSMERR_HRS_OS_ACCESS_DENIED            \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_OS_ACCESS_DENIED)

#define CSSMERR_HRS_FUNCTION_FAILED             \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_FUNCTION_FAILED)

#define CSSMERR_HRS_INVALID_DATA                \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INVALID_DATA)

#define CSSMERR_HRS_INVALID_DB_HANDLE           \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRCODE_INVALID_DB_HANDLE)



/*
    HRS Error Values
    Values that can be returned by one or more HRS APIs.
*/

#define CSSM_HRS_BASE_HRS_ERROR \
    (CSSM_HRS_BASE_ERROR+CSSM_ERRORCODE_COMMON_EXTENT)

#define CSSMERR_HRS_UNABLE_TO_CAPTURE           (CSSM_HRS_BASE_HRS_ERROR+1)
/* BSP is unable to capture raw samples from the device */

#define CSSMERR_HRS_TOO_MANY_HANDLES            (CSSM_HRS_BASE_HRS_ERROR+2)
/* The BSP has no more space to allocate BIR handles */

#define CSSMERR_HRS_TIMEOUT_EXPIRED             (CSSM_HRS_BASE_HRS_ERROR+3)
/* The Function has been terminated because the timeout value has expired */

#define CSSMERR_HRS_INVALID_BIR                 (CSSM_HRS_BASE_HRS_ERROR+4)
/* The input BIR is invalid for the purpose required */

#define CSSMERR_HRS_BIR_SIGNATURE_FAILURE       (CSSM_HRS_BASE_HRS_ERROR+5)
/* The BSP could not validate the signature on the BIR */

#define CSSMERR_HRS_UNABLE_TO_WRAP_PAYLOAD      (CSSM_HRS_BASE_HRS_ERROR+6)
/* The BSP is unable to include the payload in the new BIR */

#define CSSMERR_HRS_ADAPTATION_NOT_SUPPORTED    (CSSM_HRS_BASE_HRS_ERROR+7)
/* The BSP does not support the requested adaptation of a BIR */

#define CSSMERR_HRS_NO_INPUT_BIRS               (CSSM_HRS_BASE_HRS_ERROR+8)
/* The identify population is NULL */

#define CSSMERR_HRS_UNSUPPORTED_FORMAT          (CSSM_HRS_BASE_HRS_ERROR+9)
/* The BSP does not support the data form for the Import function */

#define CSSMERR_HRS_UNABLE_TO_IMPORT            (CSSM_HRS_BASE_HRS_ERROR+10)
/* The BSP was unable to construct a BIR from the input data */

#define CSSMERR_HRS_AUDITDATA_NOT_SUPPORTED     (CSSM_HRS_BASE_HRS_ERROR+11)
/* The BSP does not support the return of audit data. */

#define CSSMERR_HRS_FUNCTION_NOT_SUPPORTED      (CSSM_HRS_BASE_HRS_ERROR+12)
/* The BSP does not support this operation. */

#define CSSMERR_HRS_INCONSISTENT_PURPOSE        (CSSM_HRS_BASE_HRS_ERROR+13)
/* The purpose recorded in the BIR, and the requested purpose are */
/* inconsistent with the function being performed.                */

#define CSSMERR_HRS_BIR_NOT_FULLY_PROCESSED     (CSSM_HRS_BASE_HRS_ERROR+14)
/* The function requires a fully-processed BIR. */

#define CSSMERR_HRS_PURPOSE_NOT_SUPPORTED       (CSSM_HRS_BASE_HRS_ERROR+15)
/* The BSP does not support the requested purpose. */

#define CSSMERR_HRS_INVALID_BIR_HANDLE (CSSM_HRS_BASE_HRS_ERROR + 16)
/* No BIR exists with the requested handle */

#define CSSMERR_HRS_UNABLE_TO_OPEN_DATABASE     (CSSM_HRS_BASE_HRS_ERROR+256)
/* BSP is unable to open specified database */

#define CSSMERR_HRS_DATABASE_IS_LOCKED          (CSSM_HRS_BASE_HRS_ERROR+257)
/* Database cannot be opened for the access requested because it is locked */

#define CSSMERR_HRS_DATABASE_DOES_NOT_EXIST     (CSSM_HRS_BASE_HRS_ERROR+258)
/* The specified database name does not exist */

#define CSSMERR_HRS_DATABASE_ALREADY_EXISTS     (CSSM_HRS_BASE_HRS_ERROR+259)
/* Create failed because the database already exists */

#define CSSMERR_HRS_INVALID_DATABASE_NAME       (CSSM_HRS_BASE_HRS_ERROR+260)
/* Invalid database name */

#define CSSMERR_HRS_RECORD_NOT_FOUND            (CSSM_HRS_BASE_HRS_ERROR+261)
/* No record exists with the requested key */

#define CSSMERR_HRS_CURSOR_IS_INVALID           (CSSM_HRS_BASE_HRS_ERROR+262)
/* The specified cursor is invalid */

#define CSSMERR_HRS_DATABASE_IS_OPEN            (CSSM_HRS_BASE_HRS_ERROR+263)
/* Database is already open */

#define CSSMERR_HRS_INVALID_ACCESS_REQUEST      (CSSM_HRS_BASE_HRS_ERROR+264)
/* Unrecognized access type */

#define CSSMERR_HRS_END_OF_DATABASE             (CSSM_HRS_BASE_HRS_ERROR+265)
/* End of database has been reached. */

#define CSSMERR_HRS_UNABLE_TO_CREATE_DATABASE   (CSSM_HRS_BASE_HRS_ERROR+266)
/* Database could not be created */

#define CSSMERR_HRS_UNABLE_TO_CLOSE_DATABASE    (CSSM_HRS_BASE_HRS_ERROR+267)
/* Database could not be closed */

#define CSSMERR_HRS_UNABLE_TO_DELETE_DATABASE   (CSSM_HRS_BASE_HRS_ERROR+268)
/* Database could not be deleted */


#endif /* _HRSERR_H_ */
