/*-----------------------------------------------------------------------
 * File: AAL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _AAL_H
#define _AAL_H    
#include "port/cssmport.h"

/* taken from registr.h*/
#define REGSTR_MAX_VALUE_LENGTH 256

typedef CSSM_STRING AAL_STRING_64;
typedef CSSM_VERSION AAL_VERSION, *AAL_VERSION_PTR;
typedef char AAL_PATHNAME[_MAX_PATH], *AAL_PATHNAME_PTR;
#define AAL_FUNC CSSM_FUNC
typedef struct aal_func_name_addr {
	AAL_STRING_64 Name;
    CSSM_PROC_ADDR Address;
}AAL_FUNC_TABLE, *AAL_FUNC_TABLE_PTR;

typedef struct aal_version_vendor {
	AAL_VERSION Version;
	AAL_STRING_64 Vendor;
	CSSM_GUID Guid;	
}AAL_VERSION_VENDOR, *AAL_VERSION_VENDOR_PTR;

typedef enum aal_return_code {
	AAL_OK,
    AAL_CANT_GET_VALUE,   // Failed to find a key in the system registry
	AAL_CANT_OPEN_KEY,    //Failed to open a key in the system registry
	AAL_CANT_ENUM_KEY,    // Failed to enumerate keys in the system registry
	AAL_CANT_QUERY_KEY,   // Failed to query key in the system registry
    AAL_CSSM_LOAD_FAIL,   // Could not load cssm
	AAL_CSSM_INTEGRITY_FAIL,  //Integrity check of cssm failed
	AAL_CSSM_FUNC_ADDR_INTEGRITY_FAIL, //Function address is not in loaded cssm's address range
	AAL_MEMORY_ERROR,     //Internal memory error
	AAL_NOT_ENOUGH_MEMORY, //The memory provided by app, is not large enough
	AAL_INVALID_PTR,        //Invalid inpu pointer
	AAL_INVALID_CREDENTIAL_INFO, // Invalid Credential PAth or Section
	AAL_NO_ATTRIBUTE,
	AAL_MEMORY_EROR,
	AAL_MANIFEST_OPERATION_FAIL,
	AAL_MDS_ERROR,
	AAL_MDS_INITIALISE_ERROR,
	AAL_MDS_OPEN_ERROR,
	AAL_MDS_DATA_GET_ERROR,
	AAL_FAIL
} AAL_RETURN_CODE;

typedef CSSM_LIB_HANDLE AAL_LIB_HANDLE, *AAL_LIB_HANDLE_PTR;
 
/* AAL functions */
#ifdef __cplusplus
extern "C" {
#endif
/*This function checks the integrity of cssm in the specified location and loads it.
Returns a tables of function / address pairs and LibHandle*/

AAL_RETURN_CODE AAL_FUNC AAL_LoadCssm(AAL_FUNC_TABLE_PTR CssmFuncTable, 
							uint32 NumCssmFuncTable,
							CSSM_GUID_PTR Guid,
							AAL_LIB_HANDLE_PTR CssmHandle,
							CSSM_DATA_PTR TrustedKeys,
							uint32 NumTrustedKeys  );

/*Unloads the loaded cssm*/
void AAL_FUNC AAL_UnloadCssm (AAL_LIB_HANDLE CssmHandle);

/*Returns a list of Version/vendor and location of installed cssms */ 

AAL_RETURN_CODE AAL_FUNC AAL_FUNC AAL_GetCssmList ( AAL_VERSION_VENDOR_PTR CssmInfo,
									uint32* NumOfVersionVendorInfo); 

AAL_RETURN_CODE AAL_FUNC AAL_GetModuleLocation(CSSM_GUID_PTR ModuleGuid,
									  AAL_PATHNAME Location);

AAL_RETURN_CODE AAL_FUNC AAL_CheckAddressWithinVerifiedModule( AAL_LIB_HANDLE LibHandle,
													 CSSM_PROC_ADDR Address);

/*Generates the CredentialPath and section name from the exe/dll path.
This is needed for CSSM_ModuleAttach call*/

AAL_RETURN_CODE AAL_FUNC AAL_GetManifestInfo(CSSM_GUID_PTR ModuleGuid,
								CSSM_DATA_PTR Manifest,
								CSSM_DATA_PTR ModuleSearchPath,
								AAL_PATHNAME ManifestSection);


#ifdef __cplusplus
}
#endif
#endif
