/*-----------------------------------------------------------------------
 * File: CSSMUTIL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _CSSMUTIL_H
#define _CSSMUTIL_H    

#include "cssmtype.h"


#ifdef __cplusplus
extern "C" {
#endif

/*** Error Handling Macros ***********************************************/
/*************************************************************************/

#define CSSM_IsCSPError( err ) \
			( ( err >= CSSM_CSP_BASE_ERROR ) && \
			  ( err <= CSSM_CSP_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_IsCLError( err ) \
			( ( err >= CSSM_CL_BASE_ERROR ) && \
			  ( err <= CSSM_CL_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_IsDLError( err ) \
			( ( err >= CSSM_DL_BASE_ERROR ) && \
			  ( err <= CSSM_DL_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_IsTPError( err ) \
			( ( err >= CSSM_TP_BASE_ERROR ) && \
			  ( err <= CSSM_TP_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_IsCSSMError( err ) \
			( ( err >= CSSM_BASE_ERROR ) && \
			  ( err <= CSSM_CSSM_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_IsACError( err ) \
			( ( err >= CSSM_AC_BASE_ERROR ) && \
			  ( err <= CSSM_AC_BASE_ERROR + CSSM_ERRORCODE_MODULE_EXTENT - 1 ) )

#define CSSM_CompareGuids( guid1, guid2 ) \
		( ( guid1.Data1 == guid2.Data1 ) && \
		  ( guid1.Data2 == guid2.Data2 ) && \
		  ( guid1.Data3 == guid2.Data3 ) && \
		  ( guid1.Data4[0] == guid2.Data4[0] ) && \
		  ( guid1.Data4[1] == guid2.Data4[1] ) && \
		  ( guid1.Data4[2] == guid2.Data4[2] ) && \
		  ( guid1.Data4[3] == guid2.Data4[3] ) && \
		  ( guid1.Data4[4] == guid2.Data4[4] ) && \
		  ( guid1.Data4[5] == guid2.Data4[5] ) && \
		  ( guid1.Data4[6] == guid2.Data4[6] ) && \
		  ( guid1.Data4[7] == guid2.Data4[7] ) )


/* base 64 encoding functions */
uint32 CSSMAPI
Base64_EncodedLength (uint32 bufLength);

void CSSMAPI
Base64_Encode(uint8 *buf, uint32 bufLength, char* encodedString);

CSSM_DATA_PTR CSSMAPI
Base64_Decode(CSSM_HANDLE AppHandle, 
              const CSSM_UPCALLS *MemoryFunctions, 
              const CSSM_DATA *EncodedData);


/* Time converstion utilities */
char*  CSSMAPI cssm_TimeToUtc( sint32 lTime, char* UTCBuffer );
char*  CSSMAPI cssm_TimeToGeneralizedTime( sint32 lTime, char* GTBuffer );
sint32 CSSMAPI cssm_UtcToTime (const char *szTime );
sint32 CSSMAPI cssm_GeneralizedTimeToTime( const char *szTime );
sint32 CSSMAPI cssm_GetSecondsSince1970();
char*  CSSMAPI cssm_GeneralizedTimeToUtc( const char *szGTime, char *szUTCTime );

/* Time type checking utilities */
CSSM_BOOL CSSMAPI cssm_IsBadUtc (const char* szTime, sint32 length);
CSSM_BOOL CSSMAPI cssm_IsBadGeneralizedTime (const char* szGTime, sint32 length);

#ifdef WIN32
char * CSSMAPI cssm_FiletimeToUtc( FILETIME FileTime, 
                           CSSM_BOOL IsLocal, char *pUTCTime);
CSSM_RETURN CSSMAPI cssm_UtcToFiletime( const char *pUTCTime, FILETIME *pTime); 
char * CSSMAPI cssm_FiletimeToMMDDYY( FILETIME FileTime, CSSM_BOOL IsLocal, 
                              char *szMMDDYYTime);
CSSM_RETURN CSSMAPI cssm_MMDDYYToFiletime( const char *szMMDDYYTime, FILETIME *pFiletime); 
#endif  /* WIN32 */


/* This function frees all of the certs contained within a CSSM_CERTGROUP but
   does not free the top level structure.
*/
CSSM_RETURN CSSMAPI
cssmutil_CleanCertGroup(CSSM_CERTGROUP_PTR CertGroup,
                        CSSM_FREE free_func,
                        void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreeDataCertList(uint32 NumCerts,
                          CSSM_DATA_PTR *CertList,
                          CSSM_FREE free_func,
                          void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreeEncodedCertList(uint32 NumCerts,
                             CSSM_ENCODED_CERT_PTR *EncodedCertList,
                             CSSM_FREE free_func,
                             void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreeParsedCertList(uint32 NumCerts,
                            CSSM_PARSED_CERT_PTR *ParsedCertList,
                            CSSM_FREE free_func,
                            void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreePairCertList(uint32 NumCerts,
                          CSSM_CERT_PAIR_PTR *PairCertList,
                          CSSM_FREE free_func,
                          void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreeCSSMSamples(uint32 NumberOfSamples,
                         CSSM_SAMPLE_PTR *SamplesToFree,
                         CSSM_FREE free_func,
                         void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_FreeCSSMListElements(CSSM_LIST_ELEMENT_PTR *HeadPtr,
                              CSSM_FREE free_func,
                              void *mem_ref);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateBaseCerts(const CSSM_BASE_CERTS *SrcCerts,
                            CSSM_MALLOC malloc_func,
                            CSSM_FREE free_func,
                            void *mem_ref,
                            CSSM_BASE_CERTS_PTR DestCerts);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateDataCertList(uint32 NumSrcCerts,
                               const CSSM_DATA *SrcCertList,
                               CSSM_MALLOC malloc_func,
                               CSSM_FREE free_func,
                               void *mem_ref,
                               uint32 *NumDestCerts,
                               CSSM_DATA_PTR *DestCertList);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateEncodedCertList(uint32 NumSrcCerts,
                                  const CSSM_ENCODED_CERT *SrcCertList,
                                  CSSM_MALLOC malloc_func,
                                  CSSM_FREE free_func,
                                  void *mem_ref,
                                  uint32 *NumDestCerts,
                                  CSSM_ENCODED_CERT_PTR *DestCertList);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateParsedCertList(uint32 NumSrcCerts,
                                 const CSSM_PARSED_CERT *SrcCertList,
                                 CSSM_MALLOC malloc_func,
                                 CSSM_FREE free_func,
                                 void *mem_ref,
                                 uint32 *NumDestCerts,
                                 CSSM_PARSED_CERT_PTR *DestCertList);


CSSM_RETURN CSSMAPI
cssmutil_DuplicatePairCertList(uint32 NumSrcCerts,
                               const CSSM_CERT_PAIR *SrcCertList,
                               CSSM_MALLOC malloc_func,
                               CSSM_FREE free_func,
                               void *mem_ref,
                               uint32 *NumDestCerts,
                               CSSM_CERT_PAIR_PTR *DestCertList);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateSampleGroup(const CSSM_SAMPLEGROUP *SrcSamples,
                              CSSM_MALLOC malloc_func,
                              CSSM_FREE free_func,
                              void *mem_ref,
                              CSSM_SAMPLEGROUP_PTR DestSamples);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateSample(const CSSM_SAMPLE *SrcSample,
                         CSSM_MALLOC malloc_func,
                         CSSM_FREE free_func,
                         void *mem_ref,
                         CSSM_SAMPLE_PTR DestSample);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateCSSMList(const CSSM_LIST *SrcList,
                           CSSM_MALLOC malloc_func,
                           CSSM_FREE free_func,
                           void *mem_ref,
                           CSSM_LIST_PTR DestList);


CSSM_RETURN CSSMAPI
cssmutil_DuplicateCSSMListElements(const CSSM_LIST_ELEMENT *SrcElement,
                                   CSSM_MALLOC malloc_func,
                                   CSSM_FREE free_func,
                                   void *mem_ref,
                                   CSSM_LIST_ELEMENT_PTR *DestElement);

void CSSMAPI
cssm_FreeAttributes(uint32 NumberOfAttributes,
                    CSSM_CONTEXT_ATTRIBUTE_PTR Attributes,
                    CSSM_FREE free_func,
                    void *mem_ref,
                    uint32 Free);

CSSM_RETURN CSSMAPI cssm_CheckCopy (void *Dest, void *Src, uint32 size);



#ifdef __cplusplus
}
#endif

#endif
