/*-----------------------------------------------------------------------
 * File: MDS_SHIMUTIL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "cssm.h"

#ifndef MDS_SHIMUTIL_H
#define MDS_SHIMUTIL_H

#ifdef __cplusplus
extern "C" {
#endif

typedef 
CSSM_RETURN (CSSMAPI *MDSInitialize) (const CSSM_GUID *pCallerGuid,
                                      const CSSM_DATA *pCallerManifest,
                                      const CSSM_MEMORY_FUNCS *pMemoryFuncs,
                                      MDS_FUNCS * pDLFunctions,
                                      MDS_HANDLE *hMds);
typedef
CSSM_RETURN ( CSSMAPI *MDSTerminate) (MDS_HANDLE MdsHandle);

#ifndef CSSM_MEMTRACK_ON
void * CSSMAPI
mds_local_malloc(uint32 size,
                 void* allocRef);
void * CSSMAPI
mds_local_calloc(uint32 num_elem,
                 uint32 num_bytes,
                 void* allocRef);
void CSSMAPI
mds_local_free(void * mem_ptr,
               void* allocRef);
void * CSSMAPI
mds_local_realloc(void * old_ptr,
                  uint32 num_bytes,
                  void* allocRef);
#else
void * CSSMAPI
mds_local_malloc(uint32 size,
                 void* allocRef,
                 const char * szFilename,
                 uint32 u32LineNumber);
void * CSSMAPI
mds_local_calloc(uint32 num_elem,
                 uint32 num_bytes,
                 void* allocRef,
                 const char * szFilename,
                 uint32 u32LineNumber);
void CSSMAPI
mds_local_free(void * mem_ptr,
               void* allocRef,
               const char * szFilename,
               uint32 u32LineNumber);
void * CSSMAPI
mds_local_realloc(void * old_ptr,
                  uint32 num_bytes,
                  void* allocRef,
                  const char * szFilename,
                  uint32 u32LineNumber);
#endif

#ifndef CSSM_MEMTRACK_ON
#define mds_malloc(s, t) mds_local_malloc((s), (t))
#define mds_calloc(r, s, t) mds_local_calloc((r), (s), (t))
#define mds_realloc(r, s, t) mds_local_realloc((r), (s), (t))
#define mds_free(s, t) mds_local_free((s), (t))
#else
#define mds_malloc(s, t) mds_local_malloc((s), (t), __FILE__, __LINE__)
#define mds_calloc(r, s, t) mds_local_calloc((r), (s), (t), __FILE__, __LINE__)
#define mds_realloc(r, s, t) mds_local_realloc((r), (s), (t), __FILE__, __LINE__)
#define mds_free(s, t) mds_local_free((s), (t), __FILE__, __LINE__)
#endif


CSSM_RETURN CSSMAPI mds_SetError( const CSSM_GUID *guid, uint32 error);

CSSM_RETURN mds_LoadFile (const char *FileName, CSSM_DATA_PTR FileData);

#ifdef __cplusplus
}
#endif                             

#endif /* MDS_SHIMUTIL_H */
