/*-----------------------------------------------------------------------
 * File: ADD.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
     
/* Module name: add.c
   Computes t = a + b
     
   The length of the result, t, is set to MODULUS.
*/
     
#include "eicl.h"
#include "dss.h"
     
void __stdcall EICL_Add (DSAData *a, DSAData *b, DSAData *t) 
{
  register EICLWord     *aval, *bval, *tval; 
  register long          pos, length;
  unsigned long          sum, carry, msb;
     
  aval=a->value;
  bval=b->value;
  tval=t->value;
/* perform addition by using the smaller's length, then .... */
  length = (a->length > b->length) ? b->length : a->length; 
  carry = 0;
  for (pos=0; pos<length; ++pos) {
    sum=(aval[pos] & (WORDMASK >> 1)) +
          (bval[pos] & (WORDMASK >> 1)) + carry;
    msb=(aval[pos] >> (WORDSIZE-1)) + (bval[pos] >> (WORDSIZE-1)) +
      (sum >> (WORDSIZE-1));
    carry = (msb & 2)!=0;
    if (msb & 1)
      tval[pos]=sum | ((EICLWord)1 << (WORDSIZE-1));
    else
      tval[pos]=sum & (WORDMASK >> 1);
  }
/* just add the carry-out (0/1) for the residue, if any */
  if (a->length > b->length)
    length = a->length;
  else {
    length = b->length;
    aval = bval;
  }
  for (; pos<length; ++pos) {
    if (carry==0)
      tval[pos]=aval[pos];
    else {
      sum=(aval[pos] & (WORDMASK >> 1)) + carry; 
      msb=(aval[pos] >> (WORDSIZE-1)) + (sum >> (WORDSIZE-1)); 
      carry = (msb & 2)!=0;
      if (msb & 1)
     tval[pos]=sum | ((EICLWord)1 << (WORDSIZE-1));
      else
     tval[pos]=sum & (WORDMASK >> 1);
    }
  }
  tval[pos]=carry;
/* leading 0s are not discarded */
  while (pos>0 && tval[pos]==0)
      --pos;
  t->length = pos+1;
}
