/*-----------------------------------------------------------------------
 * File: MODINV.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: modinv.c
   Modular Inverse Computation.
*/
#include <stdio.h>

#include "eicl.h"
#include "dss.h"
#include "dsskg.h"
#include "eiclproc.h"

/* the modular inverse of "Number" is computed where the modulus is "Modulus"
   The inverse is located in "inverse".
   This function returns 0 on success, -1 on failure to find the inverse.
   length of input and output is byte based.
*/
void __stdcall	EICL_ModularInverse (EICLData *Number,
			EICLData *Modulus, EICLData *Inverse)
{
  int		k;
  DSAData	s, t, g; /* , *inv; */
  EICLWord	sarray[MODULUS], tarray[MODULUS], garray[MODULUS];
  DSAData   n, m, in;
  EICLWord   narray[MODULUS], marray[MODULUS], iarray[MODULUS];

/* value initialization */
  s.value=sarray;
  t.value=tarray;
  g.value=garray;

  n.value=narray;
  m.value=marray;
  in.value=iarray;
  in.length=0;

  EICL_MoveByte2Word(Number,&n);
  EICL_MoveByte2Word(Modulus,&m);

#if 0
  inv = (DSAData *)Inverse;     /* an alias pointer for Inverse */

/* adjust the lengths of inputs to reflect their word numbers */
  NumberLength = Number->length;		/* first, save their lengths */
  ModulusLength = Modulus->length;

  Number->length = EICL_WordCount (NumberLength);
  Modulus->length = EICL_WordCount (ModulusLength);
/* pad any leading byte(s) */
  EICL_PadBytes0 ((DSAData *)Number, NumberLength);
  EICL_PadBytes0 ((DSAData *)Modulus, ModulusLength);
#endif

/* run the Extended Euclidean Algorithm */
  EICL_EEA (&m, &n, &s, &t, &g, &k);

  if ((g.length==1) && (g.value[0]==1)) {	/* if GCD(.)==1 */
    if ((k & 1)==0) {   /* if number of iterations is even, make a subtract */
      EICL_MoveWord (&m, &in);
      EICL_Subtract (&in, &t);	/* Inverse = Modulus - t */
    }
    else
      EICL_MoveWord (&t, &in);
  }
  else {	/* cannot find an inverse for this modulus, since GCD(.)!=1 */
    Inverse->length=1;	/* return 0 to indicate this */
    Inverse->value[0]=0;
  }
/* fill the higher words with 0's */
  for (k=in.length; k<MODULUS; ++k)
    in.value[k] = 0;
/* and find the actual length */
  k=in.length;
  while (k>1 && in.value[k-1]==0)
    --k;
  in.length = k;

#if 0
/* restore the lengths of the input variables */
  Number->length = NumberLength;
  Modulus->length = ModulusLength;
/* find the byte-length of the Inverse */
  Inverse->length = EICL_ByteCount (inv);
#endif

  EICL_MoveWord2Byte(&in, Inverse);
}
