/*-----------------------------------------------------------------------
 * File: MOVE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*	Module name: move.c
	Move related functions.
*/

#include "eicl.h"
#include "dss.h"
#include "eiclproc.h"
#include "misc.h"

void __stdcall EICL_Padding0 (DSAData *a);


/* copies src to dest and fills the higher order words of dest with 0's */
void __stdcall	EICL_MoveWord (DSAData *src, DSAData *dest)
{
  register long	cnt, length=src->length;
  EICLWord	*dval, *sval;

  dval = dest->value;
  sval = src->value;
  for (cnt=0; cnt<length; ++cnt)
    dval[cnt] = sval[cnt];
/* pad the higher leading words to 0 */
  while (cnt<MODULUS)
    dval[cnt++]=0;
  dest->length = length;
}


/* copies src to dest, both are byte-length data, donot fill the upper words with 0's */
void __stdcall EICL_MoveByte (EICLData *src, EICLData *dest)
{
  register long	cnt=src->length;
  EICLByte	*dval, *sval;

  dval = dest->value;
  sval = src->value;
  dest->length = cnt;
  while (cnt-->0)
    dval[cnt] = sval[cnt];
}


/* copies src to dest and fills the higher order words of dest with 0's */
void __stdcall	EICL_MoveByte2Word (EICLData *src, DSAData *dest)
{
  register long	cnt, length;
  EICLWord	*dval;
  EICLByte	*sval;

  dval = dest->value;		/* word aligned */
  sval = src->value;		/* byte aligned */
  length = cnt = src->length/sizeof (EICLWord);	/* Word.length */

/* copy the words except the highest one */
  while (cnt-->0)
#ifdef BIG_ENDIAN
    dval[cnt] = bswap(((EICLWord *)sval)[cnt]);
#else
    dval[cnt] = ((EICLWord *)sval)[cnt];
#endif
  dest->length = length;

/* copy the last word. cnt is the number of bits to pad */
  cnt = (sizeof (EICLWord) - (src->length & 0x03)) * 8;	/* 32, 24, 26, 8 */
  if (cnt<32) {			/* if the last word is not fully used */
#ifdef BIG_ENDIAN
    dval[length] = bswap(((EICLWord *)sval)[length]) & ((~(EICLWord)0) >> cnt);
#else
    dval[length] = ((EICLWord *)sval)[length] & ((~(EICLWord)0) >> cnt);
#endif
    dest->length++;
  }
  EICL_Padding0 (dest);
}

/* copies src to dest */
void __stdcall	EICL_MoveWord2Byte (DSAData *src, EICLData *dest)
{
  register long length;
  EICLWord	*sval;
  EICLByte	*dval;

  dval = dest->value;		/* byte aligned */
  sval = src->value;		/* word aligned */
/* number of bytes except the 0's in the higher bytes */
  dest->length = length = EICL_ByteCount (src);
#ifndef BIG_ENDIAN
  while (length-->0)
    dval[length] = ((EICLByte *)sval)[length];
#else
  /* May copy up to 3 extra bytes */
  length = src->length;
  while (length-->0)
    ((EICLWord*)dval)[length] = bswap(sval[length]);
#endif
}

