/*-----------------------------------------------------------------------
 * File: EICL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
   Module name: eicl.h
   Embedded Intel Cryptographic Library, Version v1.0

   This Intel Cryptographic Library contains:
   * SHA Message Digest Algorithm
   * DSA Verify
*/

#ifndef _EICL_INCLUDE_
#define _EICL_INCLUDE_

#define WORDSIZE				32
#define	MODULUSBITS				1024
#define MODULUS					(MODULUSBITS / WORDSIZE)
#define MODULUSBYTES			(MODULUSBITS / 8)

#ifndef WIN32
/* Remove the Microsoft compiler declarations for function calling conv.
*/
#define __stdcall
#define __cdecl
#define __fastcall
#endif

/***************************************************************/
/*         Embedded Intel Cryptographic Library Data Types     */
/***************************************************************/
/*                                                             */
/* Multi-precision integers are stored the least significant   */
/* byte first. This ensures that Intel 80x86 processors access */
/* the same data as bytes, words, dwords, or qwords without    */
/* reordering. The least significant byte of each integer is   */
/* stored in "value[0]" of  "EICLData" type.                   */
/***************************************************************/

/* One byte (8 bit) for EICL data types                        */
typedef unsigned char EICLByte;

/* One word for EICL data types                                */
typedef unsigned int EICLWord;

/* SHA digested data, 160 bit with low order byte first        */
typedef EICLByte EICLSHADigest[20];

/***************************************************************/
/* EICLData has any number of EICLWords, not pre-allocated.    */
/* Space must be reserved by the user.                         */
/***************************************************************/

typedef struct {
  long         length;    /* number of bytes in 'value'        */
  EICLByte      *value;    /* memory address of the array      */
} EICLData;

/* SHA State Structure                                         */
typedef struct {
  EICLWord      state[5];        /* SHA State                  */
  EICLWord      count[2];        /* length of Message in bits  */
  EICLByte      buffer[64];      /* temprorary buffer          */
} EICLSHAState;

/* DSS Public Key Structure */
typedef struct {
  EICLData      PrimeModulus;    /* p */
  EICLData      PrimeDivisor;    /* q */
  EICLData      OrderQ;          /* g */
  EICLData      PublicKey;       /* y */
} EICLPublicKey;

/***************************************************************/
/*                    Function Prototypes                      */
/***************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

int EICL_SHABegin
        (EICLSHAState        *SHAState);

int EICL_SHAProcess
        (EICLData            *Message,
        EICLSHAState         *SHAState);

int EICL_SHAEnd
        (EICLSHAState        *SHAState,
        EICLSHADigest        digest);

int EICL_DSSVerify
        (EICLData            *Message,
        EICLData             *Signature_r,
        EICLData             *Signature_s,
        EICLPublicKey        *PublicKey);

#ifdef __cplusplus
}
#endif

#endif     /* _EICL_INCLUDE_ */
