/*-----------------------------------------------------------------------
 * File: MD2.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
**	MD2 algorithm glue
*/
#include "integapi.h"
#include "cssm.h"
#include "guids.h"
#include "isl_cssm.h"
#include <isl_memtrack.h>
extern CSSM_CSP_HANDLE gCSPHandle;

static unsigned char md2oid[] = { 
	0x06, 0x08, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x02, 0x02 };
static
void md2identify(
	ISL_SERVICE_CLASS *algClass,				
	ISL_CONST_DATA *algID,				
	ISL_CONST_DATA *serviceName)
{
	*algClass = ISL_ServiceDigest;
	algID->Data = md2oid;
	algID->Length = sizeof(md2oid);
	serviceName->Data = (const unsigned char *)"MD2";        /* ifdef VMS apply cast */
	serviceName->Length = strlen((char *)serviceName->Data); /* ifdef VMS apply cast */
}
static
void jarmd2identify(
	ISL_SERVICE_CLASS *algClass,				
	ISL_CONST_DATA *algID,				
	ISL_CONST_DATA *serviceName)
{
	*algClass = ISL_ServiceDigest;
	algID->Data = (const unsigned char *)"MD2";            /* ifdef VMS apply cast */
	algID->Length = strlen((char *)algID->Data);           /* ifdef VMS apply cast */
	serviceName->Data = (const unsigned char *)"MD2";          /* ifdef VMS apply cast */
	serviceName->Length = strlen((char *)serviceName->Data);   /* ifdef VMS apply cast */
}

struct md2context {
	CSSM_CSP_HANDLE hCSP;
	CSSM_CC_HANDLE hCssmContext;
	ISL_CSSM_METHODS *pCssmMethods;
};

static ISL_SIZE md2contextsize()
{
	return sizeof(struct md2context);
}

static
ISL_STATUS md2init(
	ISL_DIGEST_SERVICE_CONTEXT *memory,
	ISL_CONST_DATA parameters,
	ISL_SIZE *sizeofDigest)
{
	return ISL_FAIL;
}

static
ISL_ERROR_CODE md2update(
	ISL_DIGEST_SERVICE_CONTEXT *context,
	ISL_CONST_DATA data)
{
	int digestLength = 16;
	CSSM_RETURN status;
	CSSM_DATA ptr;
	struct md2context *md2 = (struct md2context*) context;
	ptr.Data = (uint8 *)data.Data;
	ptr.Length = data.Length;
	status = md2->pCssmMethods->CSSM_DigestDataUpdate(md2->hCssmContext, &ptr, 1); 
	switch (status) {
		case CSSM_OK:
			break;
		default:
			return ISL_INVALID_INPUT;
	}
	return ISL_OK;
}


static
ISL_ERROR_CODE md2recycle(
	ISL_DIGEST_SERVICE_CONTEXT *context,
	ISL_DATA_PTR digest)
{
	struct md2context *md2 = (struct md2context*) context;
	CSSM_DATA ptr;
	CSSM_RETURN status;
	ptr.Data = digest->Data;
	ptr.Length = digest->Length;
	status = md2->pCssmMethods->CSSM_DigestDataFinal(md2->hCssmContext, &ptr);
	if (status != CSSM_OK)
		return ISL_INVALID_INPUT;
	if (md2->pCssmMethods->CSSM_DeleteContext(md2->hCssmContext) != CSSM_OK) 
		return ISL_INVALID_INPUT;
	md2->hCssmContext = 0;
	md2->hCSP = 0;
	return ISL_OK;
}

static
ISL_STATUS md2initializeWithClass(
		ISL_DIGEST_SERVICE_CONTEXT *Memory,
		ISL_CONST_DATA Parameters,
		ISL_SIZE *SizeofDigest,
		ISL_CLASS_PTR Class) {
		
	CSSM_RETURN status;
	ISL_CSSM_METHODS *pCssmMethods;
	ISL_CSSM_MODULE_HANDLE *pCspHandle;

	struct md2context *context = (struct md2context*)Memory;

	if (!Class)
		return ISL_FAIL;
	else if (!Class->ClassContext)
		return ISL_FAIL;
	else if(!((ISL_DIGEST_CLASS_CONTEXT *)Class->ClassContext)->CSPList.ModuleHandles)
		return ISL_FAIL;
	else if(!((ISL_DIGEST_CLASS_CONTEXT *)Class->ClassContext)->CSPList.NumberOfHandles)
		return ISL_FAIL;
	else if(!((ISL_DIGEST_CLASS_CONTEXT *)Class->ClassContext)->CssmMethods)
		return ISL_FAIL;

	pCssmMethods = ((ISL_DIGEST_CLASS_CONTEXT *)Class->ClassContext)->CssmMethods;

	*SizeofDigest = 16;

	//use the first csp handle in the list to perform the operation
	pCspHandle = ((ISL_DIGEST_CLASS_CONTEXT *)Class->ClassContext)->CSPList.ModuleHandles;
	context->hCSP = pCspHandle->Handle;
	if (context->hCSP == 0)
		return ISL_FAIL;

	context->pCssmMethods = pCssmMethods; 

	//create digest context handle
	status = context->pCssmMethods->CSSM_CSP_CreateDigestContext(
		context->hCSP, CSSM_ALGID_MD2, &context->hCssmContext);
	if ( (status != CSSM_OK) || (context->hCssmContext == 0) ) {
		return ISL_FAIL;
	}

	status = context->pCssmMethods->CSSM_DigestDataInit(context->hCssmContext);
	if (status != CSSM_OK)
		return ISL_FAIL;
	return ISL_OK;
}	

ISL_DIGEST_METHODS PKCS7MD2Methods = { 
	{ md2identify },	
	md2contextsize,
	md2init,
	md2update,
	md2recycle,
	md2initializeWithClass };

ISL_DIGEST_METHODS JarMD2Methods = { 
	{ jarmd2identify },	
	md2contextsize,
	md2init,
	md2update,
	md2recycle,
	md2initializeWithClass };
