/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: pk.c,v $
 * Revision 1.1.2.4  2000/08/31  18:06:25  Thomas_Woodburn
 * 	Merged in Intel CDSA Version 2.0 Release 3.11.
 *
 * Revision 1.1.2.3  2000/08/07  14:42:12  Thomas_Woodburn
 * 	Merged in Intel CDSA Version 2.0 Release 3.1.
 *
 * Revision 1.1.2.2  2000/06/28  21:00:09  Thomas_Woodburn
 * 	Initial port to Tru64 UNIX.
 *
 * Revision 1.1.2.1  2000/06/28  20:25:24  Thomas_Woodburn
 * 	Created from Intel CDSA Version 2.0 Release 3.0.
 *
 * $EndLog$
 */
#ifndef WIN32
#ifndef VMS
#pragma ident "@(#)$RCSfile: pk.c,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:06:25 $"
#endif
#endif
/*-----------------------------------------------------------------------
 * File: PK.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


/* This file implements parsing and reading of uncompressed
   PKWARE archive format files */

#include <stdio.h>
#include <sys/stat.h>
#include <time.h>
#include "isltype.h"
#include "pkapi.h"
#include <isl_memtrack.h>

#ifdef COMPAQ_FIX /* Intel bug 582 */
/*
 * Get uint16 and uint32 values from a byte stream without worrying
 * about unaligned access errors.
 *
 * XXX This doesn't work on big-endian systems because it assumes
 * little-endian byte order and pk_WriteNewArchive() uses the system
 * byte order.
 */
#define GETUINT16(p) \
    ((uint16) (((uint8 *)(p))[0] | ((uint8 *)(p))[1] << 8))
#define GETUINT32(p) \
    ((uint32) (GETUINT16(p) | GETUINT16((uint8 *)(p) + 2) << 16))
#endif /* COMPAQ_FIX */

static void
CRC32_Init(
	uint32					table[256]);

static uint32
CRC32(
	uint32					table[256],
	PK_CONST_DATA_PTR		pData);

static PK_STATUS
ParseLocalFile(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**					pos,
	PK_ARCHIVE_PTR			pArchive);

static PK_STATUS
ParseCentralFile(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**			pos,
	PK_ARCHIVE_PTR	pArchive);

static PK_STATUS
ParseCentralDirEnd(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**			pos,
	PK_ARCHIVE_PTR	pArchive);

static PK_STATUS
ParseLocalFileHeader(
	uint8**						pos,
	PK_LOCAL_FILE_HEADER_PTR	pHeader);

static PK_STATUS
ParseCentralFileHeader(
	uint8**						pos,
	PK_CENTRAL_FILE_HEADER_PTR	pHeader);

static PK_STATUS
ParseCentralDirEndHeader(
	uint8**							pos,
	PK_CENTRAL_DIR_END_HEADER_PTR	pHeader);

PK_ARCHIVE_PTR
pk_InitializeFromMemory(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	PK_CONST_DATA	Fileimage)
{
	PK_ARCHIVE_PTR		pArchive = NULL;
	uint8*				pos = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x00);

	pArchive = isl_AllocateMemory(pMem, sizeof(PK_ARCHIVE));
	if (!pArchive) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, sizeof(PK_ARCHIVE));
		return NULL;
	}
	memset(pArchive, 0, sizeof(PK_ARCHIVE));

	pArchive->pMem = pMem;
	memcpy(&pArchive->Archive, &Fileimage, sizeof(PK_CONST_DATA));

	/* Start parsing */
	pos = (uint8*)Fileimage.Data;

	/* We look for all of the local files first */
	while ((uint32)(pos - Fileimage.Data + 4) < Fileimage.Length &&
#ifdef COMPAQ_FIX /* Intel bug 582 */
		   LOCAL_FILE_HEADER_SIGNATURE == GETUINT32(pos))
#else  /* COMPAQ_FIX */
		   LOCAL_FILE_HEADER_SIGNATURE == *(uint32*)pos)
#endif /* COMPAQ_FIX */
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x01);
		if (PK_OK != ParseLocalFile(pMem, &pos, pArchive))
			goto RECYCLE_AND_FAIL;
	}

	/* Now we look for all of the central files */
	while ((uint32)(pos - Fileimage.Data + 4) < Fileimage.Length &&
#ifdef COMPAQ_FIX /* Intel bug 582 */
		   CENTRAL_FILE_HEADER_SIGNATURE == GETUINT32(pos))
#else  /* COMPAQ_FIX */
		   CENTRAL_FILE_HEADER_SIGNATURE == *(uint32*)pos)
#endif /* COMPAQ_FIX */
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x02);
		if (PK_OK != ParseCentralFile(pMem, &pos, pArchive))
			goto RECYCLE_AND_FAIL;
	}

	/* Finally, parse the end of the Central Directory */
	while ((uint32)(pos - Fileimage.Data + 4) < Fileimage.Length &&
#ifdef COMPAQ_FIX /* Intel bug 582 */
		   END_OF_CENTRAL_DIR_SIGNATURE == GETUINT32(pos))
#else  /* COMPAQ_FIX */
		   END_OF_CENTRAL_DIR_SIGNATURE == *(uint32*)pos)
#endif /* COMPAQ_FIX */
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x03);
		if (PK_OK != ParseCentralDirEnd(pMem, &pos, pArchive))
			goto RECYCLE_AND_FAIL;
	}

	/* Check to see if file was fully parsed */
	if ((uint32)(pos - Fileimage.Data) != Fileimage.Length)
		goto RECYCLE_AND_FAIL;

	return pArchive;

RECYCLE_AND_FAIL:
	(void)pk_RecycleArchive(pArchive);
	return NULL;
}

PK_ARCHIVE_PTR
pk_LoadArchive(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	PK_CONST_DATA			Filename)
{
	PK_CONST_DATA		Fileimage = {0, NULL};

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x00);

	if (Filename.Length > MAX_PATH)
		return NULL;

	if (ISL_OK != isl_LoadFile(pMem, *(ISL_CONST_DATA_PTR)&Filename, 
					(ISL_CONST_DATA_PTR)&Fileimage, 0) || !Fileimage.Length)
		return NULL;

	return pk_InitializeFromMemory (pMem, Fileimage);
}

static void
CRC32_Init(
	uint32					table[256])
{
	uint32	i, j, pos, num;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x04);

	/* First, we need to initialize the CRC table, using the seed specified
	   by the PKWARE specification */
	num = 0xEDB88320L;

    for (i = 0; i < 256; i++)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x05);
		pos = i;
        for (j = 8; j > 0; j--)
        {
           NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x06);
           if (pos & 1)
                pos = (pos >> 1) ^ num;
            else
                pos >>= 1;
        }
        table[i] = pos;
    }
}

static uint32
CRC32(
	uint32					table[256],
	PK_CONST_DATA_PTR		pData)
{
	uint32		newCRC, i;
	const uint8	*data;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x07);
	if (!pData || !pData->Length)
		return 0;

	/* Now, we can get to the business of calculating the CRC */
    newCRC = 0xFFFFFFFF;
	data = pData->Data;
	for (i=0; i<pData->Length; i++)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x08);
		newCRC = ((newCRC>>8) & 0x00FFFFFF) ^ table[ (newCRC^data[i]) & 0xFF ];
	}

    return( newCRC^0xFFFFFFFF );
}


PK_STATUS
pk_RecycleArchive(
	PK_ARCHIVE_PTR	pArchive )
{
	PK_LOCAL_FILE_PTR			pLocal = NULL;
	PK_CENTRAL_FILE_PTR			pCentral = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x08);
	if (!pArchive)
		return PK_OK;

	/* Free local file structures */
	while (pArchive->LocalFiles)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x09);
		pLocal = pArchive->LocalFiles;
		pArchive->LocalFiles = pArchive->LocalFiles->Next;
		isl_FreeMemory(pArchive->pMem, pLocal);
	}

	/* Free central file structures */
	while (pArchive->CentralFiles)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0a);
		pCentral = pArchive->CentralFiles;
		pArchive->CentralFiles = pArchive->CentralFiles->Next;
		isl_FreeMemory(pArchive->pMem, pCentral);
	}

	/* Free central directory end */
	if (pArchive->DirEnd)
	{
		isl_FreeMemory(pArchive->pMem, pArchive->DirEnd);
	}

	/* Free the archive itself */
	isl_FreeMemory(pArchive->pMem, pArchive);

	return PK_OK;
}

PK_STATUS
pk_FindFile(
	PK_ARCHIVE_PTR		pArchive,
	PK_CONST_DATA		Filename,
	PK_CONST_DATA_PTR	Filedata )
{
	PK_LOCAL_FILE_PTR	pLocal = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0b);
	if (!pArchive || !Filename.Length || !Filedata)
		return PK_FAIL;

	pLocal = pArchive->LocalFiles;

	while (pLocal)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0c);
		if (pLocal->Filename.Length == Filename.Length)
		{
			if (!memcmp(pLocal->Filename.Data, Filename.Data, Filename.Length))
			{
				memcpy(Filedata, &pLocal->Filedata, sizeof(PK_CONST_DATA));
				return PK_OK;
			}
		}
		pLocal = pLocal->Next;
	}

	return PK_FAIL;
}

PK_ITERATOR_PTR
pk_CreateFileEnumerator(
	PK_ARCHIVE_PTR		pPKArchive)
{
	PK_ITERATOR_PTR	toReturn;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0d);
	if (!pPKArchive)
		return NULL;

	toReturn = malloc(sizeof(PK_ITERATOR));
	if (!toReturn)
		return NULL;

	(*toReturn) = pPKArchive->LocalFiles;

	return toReturn;
}

PK_STATUS
pk_GetNextFile(
	PK_ITERATOR_PTR		pIterator,
	PK_CONST_DATA_PTR	Filename,
	PK_CONST_DATA_PTR	Filedata )
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0e);
	if (!pIterator || !Filename || !Filedata || !(*pIterator))
		return PK_FAIL;

	memcpy(Filename, &(*pIterator)->Filename, sizeof(PK_CONST_DATA));
	memcpy(Filedata, &(*pIterator)->Filedata, sizeof(PK_CONST_DATA));

	(*pIterator) = (*pIterator)->Next;

	return PK_OK;
}

PK_STATUS
pk_RecycleFileEnumerator(
	PK_ITERATOR_PTR		pIterator )
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x0f);
	if (pIterator)
		free(pIterator);
	return PK_OK;
}


static PK_STATUS
ParseLocalFile(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**			pos,
	PK_ARCHIVE_PTR	pArchive)
{
	PK_LOCAL_FILE_PTR	pLocal = NULL;
	uint32				crc;
	uint32				table[256];

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x11);
	if (!pos || !pArchive)
		return PK_FAIL;

	/* Initialize the CRC table */
	CRC32_Init(table);

	/* Skip over signature */
	(*pos) += 4;

	/* Allocate the data structure and patch the linked list */
	pLocal = isl_AllocateMemory(pMem, sizeof(PK_LOCAL_FILE));
	if (!pLocal) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, sizeof(PK_LOCAL_FILE));
		return PK_FAIL;
	}
	memset(pLocal, 0, sizeof(PK_LOCAL_FILE));
	pLocal->Next = pArchive->LocalFiles;

	pArchive->LocalFiles = pLocal;

	pLocal->Header = (PK_LOCAL_FILE_HEADER_PTR)*pos;
	(*pos) += sizeof(PK_LOCAL_FILE_HEADER);

	/* Set up pointers to filename, extra field, and file data */
	pLocal->Filename.Length = pLocal->Header->FilenameLength;
	pLocal->ExtraField.Length = pLocal->Header->ExtraFieldLength;
	pLocal->Filedata.Length = pLocal->Header->CompressedSize;

	pLocal->Filename.Data = *pos;
	(*pos) += pLocal->Filename.Length;
	pLocal->ExtraField.Data = *pos;
	(*pos) += pLocal->ExtraField.Length;
	pLocal->Filedata.Data = *pos;
	(*pos) += pLocal->Filedata.Length;

	/* Check CRC */
	crc = CRC32(table, &pLocal->Filedata);
	if (crc != pLocal->Header->crc32)
		return PK_FAIL;

	/* If bit 3 of general purpose bit flag is set, then there is
	   another descriptor struct to worry about */
	if (pLocal->Header->GeneralPurposeBitFlag & 0x00001000)
	{
		pLocal->DataDescriptor = (PK_DATA_DESCRIPTOR_PTR)*pos;
		(*pos) += sizeof(PK_DATA_DESCRIPTOR);

		/* Make sure this archive is uncompressed */
		if (pLocal->DataDescriptor->CompressedSize !=
			pLocal->DataDescriptor->UncompressedSize)
			return PK_FAIL;
	} else {
		/* Make sure this archive is uncompressed */
		if (pLocal->Header->CompressedSize !=
			pLocal->Header->UncompressedSize)
			return PK_FAIL;
	}

	return PK_OK;
}

static PK_STATUS
ParseCentralFile(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**			pos,
	PK_ARCHIVE_PTR	pArchive)
{
	PK_CENTRAL_FILE_PTR	pCentral = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x12);
	if (!pos || !pArchive)
		return PK_FAIL;

	/* Skip over the signature */
	(*pos) += 4;

	/* Allocate a new data structure and patch up linked list */
	pCentral = isl_AllocateMemory(pMem, sizeof(PK_CENTRAL_FILE));
	if (!pCentral) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, sizeof(PK_CENTRAL_FILE));
		return PK_FAIL;
	}
	memset(pCentral, 0, sizeof(PK_CENTRAL_FILE));
	pCentral->Next = pArchive->CentralFiles;

	pArchive->CentralFiles = pCentral;

	/* Grab the header */
	pCentral->Header = (PK_CENTRAL_FILE_HEADER_PTR)*pos;
	(*pos) += sizeof(PK_CENTRAL_FILE_HEADER);
	/* Check to make sure the data is not compressed */
	if (pCentral->Header->CompressedSize !=
		pCentral->Header->UncompressedSize)
		return PK_FAIL;

	/* Set pointers to filename, extra field, and file comment */
	pCentral->Filename.Length = pCentral->Header->FilenameLength;
	pCentral->ExtraField.Length = pCentral->Header->ExtraFieldLength;
	pCentral->FileComment.Length = pCentral->Header->FileCommentLength;
	pCentral->Filename.Data = *pos;
	(*pos) += pCentral->Filename.Length;
	pCentral->ExtraField.Data = *pos;
	(*pos) += pCentral->ExtraField.Length;
	pCentral->FileComment.Data = *pos;
	(*pos) += pCentral->FileComment.Length;

	return PK_OK;
}

static PK_STATUS
ParseCentralDirEnd(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	uint8**			pos,
	PK_ARCHIVE_PTR	pArchive)
{
	PK_CENTRAL_DIR_END_PTR pEnd = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x13);
	if (!pos || !pArchive)
		return PK_FAIL;

	/* Skip over signature */
	(*pos) += 4;

	pEnd = isl_AllocateMemory(pMem, sizeof(PK_CENTRAL_DIR_END));
	if (!pEnd) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, sizeof(PK_CENTRAL_DIR_END));
		return PK_FAIL;
	}
	memset(pEnd, 0, sizeof(PK_CENTRAL_DIR_END));
	pArchive->DirEnd = pEnd;

	pEnd->Header = (PK_CENTRAL_DIR_END_HEADER_PTR)*pos;
	(*pos) += sizeof(PK_CENTRAL_DIR_END_HEADER);

	/* Set up the zip file comment field */
	pEnd->ZipFileComment.Length = pEnd->Header->ZipfileCommentLength;
	pEnd->ZipFileComment.Data = *pos;
	(*pos) += pEnd->ZipFileComment.Length;

	return PK_OK;
}

#ifndef EISL

static PK_STATUS
CheckReadFile(
	PK_CONST_DATA	Filename )
{
	char				NTfilename[MAX_PATH+1];
	FILE *fi = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x14);
	if (!Filename.Length)
		return PK_FAIL;

	/* Null terminate the file name and load the archive into memory */
	memcpy(NTfilename, Filename.Data, Filename.Length);
	NTfilename[Filename.Length] = '\0';

	/* open it for read */
	fi = fopen(NTfilename,"rb");
	if (!fi){
		return PK_FAIL;
	}
	fclose(fi);
	return PK_OK;
}

PK_NEW_ARCHIVE_PTR
pk_CreateNewArchive(
	ISL_ARCHIVE_CONTEXT_PTR pArchive )
{
	PK_NEW_ARCHIVE_PTR pNew = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x15);
	/* Create the new archive struct */
	pNew = isl_AllocateMemory(pArchive, sizeof(PK_NEW_ARCHIVE));
	if (!pNew) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, sizeof(PK_NEW_ARCHIVE));
		return NULL;
	}
	memset(pNew, 0, sizeof(PK_NEW_ARCHIVE));

	pNew->pMem = pArchive;

	return pNew;
}

PK_STATUS
pk_AddFileToNewArchive(
	PK_NEW_ARCHIVE_PTR	pNewArchive,
	PK_CONST_DATA		Filename,
	PK_CONST_DATA		FileImage)
{
	PK_NEW_FILE_PTR			pNew = NULL;
	ISL_ARCHIVE_CONTEXT_PTR	pMem = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x16);
	if (!pNewArchive || !Filename.Length)
		return PK_FAIL;
	pMem = pNewArchive->pMem;

	/* Check if readable from disk (if Filedata is null) */
	if (FileImage.Length == 0 &&
		PK_OK != CheckReadFile(Filename))
		return PK_FAIL;

	pNew = isl_AllocateMemory(pMem, sizeof(PK_NEW_FILE));
	if (!pNew) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, sizeof(PK_NEW_FILE));
		return PK_FAIL;
	}
	memset(pNew, 0, sizeof(PK_NEW_FILE));

	pNew->Filename.Length = Filename.Length;
	pNew->Filename.Data = isl_AllocateMemory(pMem, Filename.Length);
	if (!pNew->Filename.Data) {
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, Filename.Length);
		return PK_FAIL;
	}
	memcpy((void*)pNew->Filename.Data, Filename.Data, Filename.Length);

	/* If we're passed a memory image, add it in */
	if (FileImage.Length)
	{
		pNew->FileImage.Length = FileImage.Length;
		pNew->FileImage.Data = (uint8*)FileImage.Data;
	}

	/* Patch up the linked list of files */
	pNew->Next = pNewArchive->Files;
	pNewArchive->Files = pNew;

	return PK_OK;
}

PK_STATUS
pk_WriteNewArchive(
	PK_NEW_ARCHIVE_PTR	pNewArchive,
	PK_CONST_DATA		FilenameToWrite )
{
	FILE						*fi = NULL;
	ISL_ARCHIVE_CONTEXT_PTR		pMem = NULL;
	PK_LOCAL_FILE_HEADER		LocalHeader;
	PK_CENTRAL_FILE_HEADER		CentralHeader;
	PK_CENTRAL_DIR_END_HEADER	DirEnd;
	char						NTFilename[MAX_PATH+1];
	PK_NEW_FILE_PTR				pFile = NULL;
	size_t						size = 0;
	uint32						sigWord = 0, i=0;
	time_t						mTime = 0;
	uint32						table[256];
	struct tm					*tmTime;
	uint16						seconds, minutes, hours, days, months, years;
	BOOL						ILoadedImage = FALSE;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x17);
	if (!pNewArchive || !FilenameToWrite.Length)
		return PK_FAIL;
	pMem = pNewArchive->pMem;
	memset(&LocalHeader, 0, sizeof(PK_LOCAL_FILE_HEADER));
	memset(&CentralHeader, 0, sizeof(PK_CENTRAL_FILE_HEADER));
	memset(&DirEnd, 0, sizeof(PK_CENTRAL_DIR_END_HEADER));

	/* Null terminate the output filename and open it for writing */
	memcpy(NTFilename, FilenameToWrite.Data, FilenameToWrite.Length);
	NTFilename[FilenameToWrite.Length] = '\0';

	fi = fopen(NTFilename,"wb");
	if (!fi){
		return PK_FAIL;
	}

	/* Initialize the CRC table */
	CRC32_Init(table);

	pFile = pNewArchive->Files;
	sigWord = LOCAL_FILE_HEADER_SIGNATURE;

	/* Load images and store local files */
	while (pFile)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x18);
		pFile->RelativeOffsetOfLocalHeader = DirEnd.CentralDirOffsetFromStartingDisk;

		if (!pFile->FileImage.Length)
		{
			if (PK_OK != isl_LoadFile(pMem, *((ISL_CONST_DATA_PTR)&pFile->Filename),
								(ISL_CONST_DATA_PTR)&pFile->FileImage, &mTime))
				goto FAIL;
			ILoadedImage = TRUE;
		} else {
			ILoadedImage = FALSE;
			time(&mTime);
		}

		pFile->crc = CRC32(table, (PK_CONST_DATA_PTR)&pFile->FileImage);

		/* Convert the time structure to DOS format date and time */
		tmTime = localtime(&mTime);
		if (!tmTime)
			goto FAIL;
		seconds = (uint16)(tmTime->tm_sec >> 1);
		minutes = (uint16)(tmTime->tm_min) << 5;
		hours = (uint16)(tmTime->tm_hour) << 11;
		days = (uint16)(tmTime->tm_mday);
		months = (uint16)(tmTime->tm_mon) << 5;
		years = (uint16)(tmTime->tm_year - 80) << 9;

		pFile->Time = seconds | minutes | hours;
		pFile->Date = days | months | years;

		LocalHeader.LastModFileDate = pFile->Date;
		LocalHeader.LastModFileTime = pFile->Time;
		LocalHeader.CompressedSize = pFile->FileImage.Length;
		LocalHeader.UncompressedSize = pFile->FileImage.Length;
		LocalHeader.FilenameLength = (uint16)pFile->Filename.Length;
		LocalHeader.crc32 = pFile->crc;

		/* Change backslashes to forward slashes in filename */
		for (i=0; i<pFile->Filename.Length; i++)
		{
            NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x19);
			if (pFile->Filename.Data[i] == '\\')
				pFile->Filename.Data[i] = '/';
		}

		/* Write local file header signature word */
		size = 4;
		if (size != fwrite(&sigWord, 1, size, fi))
			goto FAIL;

		/* Write local file header */
		size = sizeof(PK_LOCAL_FILE_HEADER);
		if (size != fwrite(&LocalHeader, 1, size, fi))
			goto FAIL;

		/* Write the filename */
		size = pFile->Filename.Length;
		if (size != fwrite(pFile->Filename.Data, 1, size, fi))
			goto FAIL;

		/* Write the file image  */
		size = pFile->FileImage.Length;
		if (size != fwrite(pFile->FileImage.Data, 1, size, fi))
			goto FAIL;

		/* Free the read file image (no longer needed) */
		if (ILoadedImage && pFile->FileImage.Length)
			isl_FreeMemory(pMem, (void*)pFile->FileImage.Data);

		/* Maintain Central Dir End info for writing later */
		DirEnd.CentralDirOffsetFromStartingDisk += 
			4 + sizeof(PK_LOCAL_FILE_HEADER) + pFile->Filename.Length + pFile->FileImage.Length;

		pFile = pFile->Next;
	}

	pFile = pNewArchive->Files;
	sigWord = CENTRAL_FILE_HEADER_SIGNATURE;
	/* Write central files */
	while (pFile)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x1a);
		memset(&CentralHeader, 0, sizeof(PK_CENTRAL_FILE_HEADER));
		CentralHeader.LastModFileDate = pFile->Date;
		CentralHeader.LastModFileTime = pFile->Time;
		CentralHeader.CompressedSize = pFile->FileImage.Length;
		CentralHeader.UncompressedSize = pFile->FileImage.Length;
		CentralHeader.FilenameLength = (uint16)pFile->Filename.Length;
		CentralHeader.crc32 = pFile->crc;
		CentralHeader.RelativeOffsetOfLocalHeader = pFile->RelativeOffsetOfLocalHeader;
		/* Write central file header signature word */
		size = 4;
		if (size != fwrite(&sigWord, 1, size, fi))
			goto FAIL;

		/* Write central file header */
		size = sizeof(PK_CENTRAL_FILE_HEADER);
		if (size != fwrite(&CentralHeader, 1, size, fi))
			goto FAIL;

		/* Write the filename */
		size = pFile->Filename.Length;
		if (size != fwrite(pFile->Filename.Data, 1, size, fi))
			goto FAIL;

		/* Change forward slashes back to backslashes */
		for (i=0; i<pFile->Filename.Length; i++)
		{
			if (pFile->Filename.Data[i] == '/')
#ifdef VMS
		            pFile->Filename.Data[i] = (uint8)'\\';
#else
                            (uint8)pFile->Filename.Data[i] = '\\';
#endif
		}

		/* Maintain Central Dir End Header info for writing later */
		DirEnd.TotalCentralDirEntries++;
		DirEnd.TotalCentralDirEntriesOnDisk++;
		DirEnd.SizeOfCentralDir += 4 + sizeof(PK_CENTRAL_FILE_HEADER) + pFile->Filename.Length;

		pFile = pFile->Next;
	}

	sigWord = END_OF_CENTRAL_DIR_SIGNATURE;
	/* Write end of central directory */
	size = 4;
	if (size != fwrite(&sigWord, 1, size, fi))
		goto FAIL;

	size = sizeof(PK_CENTRAL_DIR_END_HEADER);
	if (size != fwrite(&DirEnd, 1, size, fi))
		goto FAIL;

	fclose(fi);
	
	return PK_OK;

FAIL:
	if (fi)
		fclose(fi);
	return PK_FAIL;
}

PK_STATUS
pk_RecycleNewArchive(
	PK_NEW_ARCHIVE_PTR	pNewArchive )
{
	ISL_ARCHIVE_CONTEXT_PTR		pMem = NULL;
	PK_NEW_FILE_PTR				pNew = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x1b);
	if (pNewArchive)
	{
		pMem = pNewArchive->pMem;
		while (pNewArchive->Files)
		{
            NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_PK_TAG,0x1c);
			if (pNewArchive->Files->Filename.Length)
				isl_FreeMemory(pMem, (void*)pNewArchive->Files->Filename.Data);
			pNew = pNewArchive->Files;
			pNewArchive->Files = pNewArchive->Files->Next;
			isl_FreeMemory(pMem, pNew);
		}
		isl_FreeMemory(pMem, pNewArchive);
	}
	return PK_OK;
}

#endif /* #ifndef EISL */
