/*-----------------------------------------------------------------------
 * File: SELFCHCK.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#include "eislapi.h"
#include "isl_internal.h"
#include "islutil.h"
#include "ber_bn.h"
#include "callouts.h"
#include <isl_memtrack.h>

#define MAX_LENGTH	1024

static ISL_VERIFIED_SIGNATURE_ROOT_PTR
VerifySignatureRoot(
	ISL_SIGNATURE_CONTEXT_PTR pSignatureContext)
{	
	ISL_SIGNER_CONTEXT_PTR	pSignerContext = NULL;
	ISL_CONST_DATA PublicKey;
	uint8 szPublicKey[MAX_LENGTH];


    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SELFCHECK_TAG,0x01);
	/* Fix Me 7/14/1997 */
	/* Need to match Signer with SignerID */
	pSignerContext = pSignatureContext->Signers;

	PublicKey.Length = MAX_LENGTH;
	PublicKey.Data = szPublicKey;

	(void) EISL_RetrieveSelfCheckKey((ISL_CALLOUT_DATA*)&PublicKey);

	if (isl_VerifySignature (pSignerContext, PublicKey) != ISL_OK)
		return NULL;
	else
		return pSignerContext;
}

/*
 * EISL_SelfCheck
 *
 * This function returns a pointer to the verified module object if the module passed 
 * self-check, otherwise NULL.  Implementations must check to see that the return address 
 * and the checking code itself are in the checked module. 
 *
 * NOTE: The public key used to verify the signature is assumed to be buried in the 
 * implementation of the library or referenced by it in an implementation specific manner, 
 * and is not exposed in the API.  Implementers are encouraged to take measures that make 
 * if difficult to modify the public key.  The self-check module also implicitly knows how 
 * to obtain its credentials. 
 *
 * Implementers are encouraged to embed a low-grade secret within the self-check 
 * implementation which makes it more difficult to bypass the self-check. The implementation 
 * should verify that the return address and the address of the function itself are within the 
 * module being verified.
 */

ISL_VERIFIED_MODULE_PTR 
EISL_SelfCheck()
{
	ISL_CONST_DATA					Credentials;
	ISL_CONST_DATA					ModuleSearchPath;
	ISL_CONST_DATA					SectionName;		
	ISL_ARCHIVE_CONTEXT_PTR			pArchive = NULL;
	ISL_VERIFIED_MODULE_PTR			pVModule = NULL;
	ISL_MANIFEST_SECTION_PTR		pManSect = NULL;
	ISL_SIGNATURE_CONTEXT_PTR		pSignatureContext = NULL;
	ISL_VERIFIED_SIGNATURE_ROOT_PTR pVSigRoot = NULL;
	ISL_SIGNATURE_LIST_PTR pSigList = NULL;
	ISL_FUNCTION_PTR				return_address;
	uint8							work[2][MAX_LENGTH];
	uint8							*pManifest = NULL;
	uint32							CredSize = 0;


    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SELFCHECK_TAG,0x00);
	EISL_GetReturnAddress(return_address);

	pArchive = (ISL_ARCHIVE_CONTEXT_PTR) malloc(sizeof(ISL_ARCHIVE_CONTEXT));
	if (NULL == pArchive) 
		return NULL;

	EISL_RetrieveSelfCheckCredentialsSize ((uint32 *)&CredSize); /* ifdef VMS, apply cast */
	if (0 == CredSize)
		return NULL;
	if ((pManifest = malloc (CredSize)) == NULL)
		return NULL;

	Credentials.Data	= (const uint8 *)pManifest;
	Credentials.Length	= CredSize;
	ModuleSearchPath.Data	= (const uint8 *)&work[0];
	ModuleSearchPath.Length	= MAX_LENGTH;
	SectionName.Data	= (const uint8 *)&work[1];
	SectionName.Length	= MAX_LENGTH;

	EISL_RetrieveSelfCheckCredentials((ISL_CALLOUT_DATA *)&Credentials, 
										(ISL_CALLOUT_DATA *)&ModuleSearchPath);
	EISL_RetrieveSelfCheckSectionName((ISL_CALLOUT_DATA *)&SectionName);

	if (ISL_OK != isl_InitArchiveContextFromCredentialsBuffer(pArchive, Credentials, ModuleSearchPath))
		goto CLEANUP_AND_FAIL;

	
	pSigList = pArchive->Signatures;

	while (pSigList) {
		pSignatureContext = pSigList->Signature;
		if (!pSignatureContext)
			goto CLEANUP_AND_FAIL;

		pVSigRoot = VerifySignatureRoot(pSignatureContext);			/* yes, verify signature */
		if (NULL != pVSigRoot)
			break;
		pSigList = pSigList->Next;
	}

	if (NULL == pVSigRoot)
		goto CLEANUP_AND_FAIL;									/* did it verify? */

	/*
	 * Recycle all signers, except for the verified one.
	 */
	if (isl_recycleSignature (pArchive, pSignatureContext) != ISL_OK)
		goto CLEANUP_AND_FAIL;

	pManSect = EISL_FindManifestSection(pVSigRoot, SectionName);	/* yes, let's find our section */
	if (NULL == pManSect) 
		goto CLEANUP_AND_FAIL;									/* did we find it? */

	pVModule = EISL_VerifyLoadedModule(pManSect);				/* yes, let's check our image */
	if (NULL == pVModule) 
		goto CLEANUP_AND_FAIL;									/* are we okay? */
	
	free (pManifest);
	return (pVModule);											/* we feel good! */

CLEANUP_AND_FAIL:
	(void)isl_RecycleArchiveContext(pArchive);
	free(pArchive);
	free (pManifest);
	return NULL;
}
