/*-----------------------------------------------------------------------
 * File: ISL_CSSM.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _ISL_CSSM_H
#define _ISL_CSSM_H

#include "cssm.h"
#include "cssmapi_typecast.h"


/* class context structures */
#define MAX_NUM_CSSM_FUNC 15


typedef struct isl_cssm_2_0_methods {

CSSM_ModuleLoad_ptr CSSM_ModuleLoad;

CSSM_ModuleUnload_ptr CSSM_ModuleUnload;

CSSM_ModuleAttach_ptr CSSM_ModuleAttach;

CSSM_ModuleDetach_ptr CSSM_ModuleDetach;

CSSM_DigestDataInit_ptr CSSM_DigestDataInit;

CSSM_DigestDataUpdate_ptr CSSM_DigestDataUpdate;

CSSM_DigestDataFinal_ptr CSSM_DigestDataFinal;

CSSM_CSP_CreateSignatureContext_ptr CSSM_CSP_CreateSignatureContext;

CSSM_CSP_CreateDigestContext_ptr CSSM_CSP_CreateDigestContext;

CSSM_DeleteContext_ptr CSSM_DeleteContext;

CSSM_VerifyData_ptr CSSM_VerifyData;

CSSM_SignData_ptr CSSM_SignData;

CSSM_CSP_ObtainPrivateKeyFromPublicKey_ptr CSSM_CSP_ObtainPrivateKeyFromPublicKey;

} ISL_CSSM_METHODS, *ISL_CSSM_METHODS_PTR;



/*
 * Structure to encapsulate the information of a CSP.
 */
typedef struct csp_list_item {
	uint32 SubServiceID;
	CSSM_VERSION Version;
	CSSM_GUID Guid;
	struct csp_list_item *Next;
}CSP_LIST_ITEM, *CSP_LIST_ITEM_PTR;


typedef struct csp_list {
	CSP_LIST_ITEM_PTR Items;
	uint32 NumberOfItems;
}CSP_LIST, *CSP_LIST_PTR;

/* class context structures */
typedef struct isl_cssm_module_handle {
	CSSM_MODULE_HANDLE Handle;
	CSSM_GUID Guid;
} ISL_CSSM_MODULE_HANDLE, *ISL_CSSM_MODULE_HANDLE_PTR;

typedef struct isl_csp_handles {
    uint32 NumberOfHandles;						   /* Number of CSP handle in list */
    ISL_CSSM_MODULE_HANDLE *ModuleHandles;		   /* List of CSP handles */
} ISL_CSP_HANDLES, *ISL_CSP_HANDLES_PTR;

typedef struct isl_digest_sign_method {
	ISL_DIGEST_METHODS *DigestMethods;
	ISL_SIGN_VERIFY_METHODS *SignMethods;
}ISL_DIGEST_SIGN_METHOD, *ISL_DIGEST_SIGN_METHOD_PTR;

typedef struct isl_signature_method {
	uint32 NumberOfMethods;
	ISL_DIGEST_SIGN_METHOD *Methods;
}ISL_SIGNATURE_METHOD, *ISL_SIGN_METHOD_PTR; 

typedef struct isl_sign_verify_class_context{
	CSSM_API_MEMORY_FUNCS_PTR MemoryFuncs;		/* memory functions that will be used by CSSM */
	ISL_CSP_HANDLES CSPList;
	ISL_SIGNATURE_METHOD SignatureMethodList;
	ISL_CSSM_METHODS_PTR CssmMethods;
} ISL_SIGN_VERIFY_CLASS_CONTEXT, *ISL_SIGN_VERIFY_CLASS_CONTEXT_PTR;

typedef struct isl_digest_class_context{
	ISL_CSP_HANDLES CSPList;	
	ISL_CSSM_METHODS_PTR CssmMethods;
} ISL_DIGEST_CLASS_CONTEXT, *ISL_DIGEST_CLASS_CONTEXT_PTR;

typedef struct isl_config_class_context {
	ISL_CSSM_METHODS *CssmMethods;
}ISL_CONFIG_CLASS_CONTEXT, *ISL_CONFIG_CLASS_CONTEXT_PTR;

/* Do not edit or remove the following string. 
 * It should be expanded at check-in by version control.
 */
static const char sccs_id_INC_ISL_CSSM_H[] = { "Fix ME" }; 

#endif
