/*-----------------------------------------------------------------------
 * File: JAR_DEFS.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
 
/* Data structures for Javasoft Jar format archive objects
   This header file is not to be used except by implementations of the Jar format methods.
*/
extern IntegrityArchiveConfig JarArchiveConfig;
extern IntegrityArchiveConfigMethods JarArchiveConfigMethods;
extern IntegrityArchiveMethods JarArchiveMethods;
extern IntegritySignableObjectMethods JarSectionMethods;
extern IntegrityDigestValueListMethods JarValueListMethods;
extern IntegrityReferencedObjectListMethods JarSignedListMethods;

extern void *JarCopyMemory(struct jar_archive_struct *archive, 
	void *buffer, 
	IntegritySize size);
extern void *JarAllocateMemory(struct jar_archive_struct *archive, 
	IntegritySize size);

struct algorithm_list_item {
	struct algorithm_list_item *next;
	IntegrityServiceClass ServiceType;
	CSSM_DATA ArchiveName;
	CSSM_DATA HumanName;
	void *Methods;
};

struct memory_list_item {
	struct memory_list_item *next;
	void *buffer;
};

struct section_list_item {
	struct section_list_item *next;
	void *object;
};

struct signer_list_item {
	struct signer_list_item *next;
	void *cert;
	IntegrityCertificateMethods *certMethods;
};

struct attribute_list_item {
	struct attribute_list_item *next;
	CSSM_DATA name;
	CSSM_DATA value;
};

struct signature_list_item {
	struct signature_list_item *next;
	void *signature;
	IntegritySignatureMethods *sigMethods;
};

struct digest_list_item {
	struct digest_list_item *next;
	struct jar_value_list_struct *digest;
};

typedef struct jar_archive_config_struct {
	IntegrityArchiveConfigMethods *methods;
	void *MemoryContext;
	IntegrityMemoryMethods *MemoryMethods;
	struct algorithm_list_item *algorithms;
	void *ArchiveMethods;
	void *SectionMethods;
	void *ListToSignMethods;
	void *DigestValueMethods;
} JarConfig;

typedef struct jar_archive_struct {
	IntegrityArchiveMethods *methods;
	JarConfig *config;
	CSSM_DATA name;
	struct section_list_item *objects;
	struct signer_list_item *certs;
	struct signature_list_item *sigs;
	struct memory_list_item *buffers;
	int ReadOnly;
} JarArchiveContext;

typedef struct jar_object_ref_struct {
	IntegrityReferencedObjectListMethods *methods;
	JarArchiveContext *parent;
	struct section_list_item *objects;
	CSSM_DATA name;
	int ReadOnly;
} JarObjectRefs;

typedef struct jar_object_struct {
	IntegritySignableObjectMethods *methods;
	JarArchiveContext *parent;
	CSSM_DATA name;
	CSSM_DATA protocol;
	CSSM_DATA parameters;
	IntegrityGetDataMethods *getDataMethods;
	void *verifyMethods;
	CSSM_DATA verifyParameters;
	struct attribute_list_item *metadata;
	struct digest_list_item *digests;
	int ReadOnly;
} JarSection;

typedef struct jar_value_list_struct {
	IntegrityDigestValueListMethods *methods;
	JarSection *parent;
	IntegrityDigestMethods *algorithm;
	CSSM_DATA algorithmParameters;
	struct attribute_list_item *values;
	int ReadOnly;
} JarValueList;


