/*-----------------------------------------------------------------------
 * File: PKDEF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _PKDEF_H_
#define _PKDEF_H_

#define LOCAL_FILE_HEADER_SIGNATURE 0x04034b50
#define CENTRAL_FILE_HEADER_SIGNATURE 0x02014b50
#define END_OF_CENTRAL_DIR_SIGNATURE 0x06054b50

#include "isltype.h"
#include "isl_internal.h"

#pragma pack(1)

/* Basic Types */
#ifdef VMS
typedef unsigned char uint8;
typedef short sint16;
typedef int sint32;
#else
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef short sint16;
typedef unsigned int uint32;
typedef int sint32;
#endif

typedef struct pk_data {
	uint32			Length;
	uint8*			Data;
} PK_DATA, *PK_DATA_PTR;

typedef struct pk_const_data {
	uint32			Length;
	const uint8*	Data;
} PK_CONST_DATA, *PK_CONST_DATA_PTR;

typedef enum pk_status {
    PK_OK = 0,
    PK_FAIL = -1
} PK_STATUS;

typedef struct pk_local_file_header {
    uint16	VersionNeededToExtract;
    uint16	GeneralPurposeBitFlag;
    uint16	CompressionMethod;
    uint16	LastModFileTime;
    uint16	LastModFileDate;
    uint32	crc32;
    uint32	CompressedSize;
    uint32	UncompressedSize;
    uint16	FilenameLength;
    uint16	ExtraFieldLength;
} PK_LOCAL_FILE_HEADER, *PK_LOCAL_FILE_HEADER_PTR;

typedef struct pk_data_descriptor {
	uint32	crc32;
	uint32	CompressedSize;
	uint32	UncompressedSize;
} PK_DATA_DESCRIPTOR, *PK_DATA_DESCRIPTOR_PTR;

typedef struct pk_central_file_header {
    uint16	VersionMadeBy;
	uint16	VersionNeededToExtract;
    uint16	GeneralPurposeBitFlag;
    uint16	CompressionMethod;
    uint16	LastModFileTime;
    uint16	LastModFileDate;
    uint32	crc32;
    uint32	CompressedSize;
    uint32	UncompressedSize;
    uint16	FilenameLength;
    uint16	ExtraFieldLength;
    uint16	FileCommentLength;
    uint16	DiskNumberStart;
    uint16	InternalFileAttributes;
    uint32	ExternalFileAttributes;
    uint32	RelativeOffsetOfLocalHeader;
} PK_CENTRAL_FILE_HEADER, *PK_CENTRAL_FILE_HEADER_PTR;

typedef struct pk_central_dir_end_header {
    uint16	ThisDiskNumber;
	uint16	StartDiskNumber;
	uint16	TotalCentralDirEntriesOnDisk;
	uint16	TotalCentralDirEntries;
	uint32	SizeOfCentralDir;
	uint32	CentralDirOffsetFromStartingDisk;
	uint16	ZipfileCommentLength;
} PK_CENTRAL_DIR_END_HEADER, *PK_CENTRAL_DIR_END_HEADER_PTR;

typedef struct pk_central_dir_end {
	PK_CENTRAL_DIR_END_HEADER_PTR	Header;
	PK_CONST_DATA					ZipFileComment;
} PK_CENTRAL_DIR_END, *PK_CENTRAL_DIR_END_PTR;

typedef struct pk_local_file {
	PK_LOCAL_FILE_HEADER_PTR	Header;
	PK_CONST_DATA				Filename;
	PK_CONST_DATA				ExtraField;
	PK_CONST_DATA				Filedata;
	PK_DATA_DESCRIPTOR_PTR		DataDescriptor;
	struct pk_local_file*		Next;
} PK_LOCAL_FILE, *PK_LOCAL_FILE_PTR;

typedef struct pk_central_file {
	PK_CENTRAL_FILE_HEADER_PTR	Header;
	PK_CONST_DATA				Filename;
	PK_CONST_DATA				ExtraField;
	PK_CONST_DATA				FileComment;
	struct pk_central_file*		Next;
} PK_CENTRAL_FILE, *PK_CENTRAL_FILE_PTR;

typedef struct pk_archive {
	ISL_ARCHIVE_CONTEXT_PTR		pMem;		/* For memory context in EISL */
	PK_CONST_DATA				Archive;
	PK_LOCAL_FILE_PTR			LocalFiles;
	PK_CENTRAL_FILE_PTR			CentralFiles;
	PK_CENTRAL_DIR_END_PTR		DirEnd;
} PK_ARCHIVE, *PK_ARCHIVE_PTR;

typedef PK_LOCAL_FILE	*PK_ITERATOR, **PK_ITERATOR_PTR;

#ifndef EISL
typedef struct pk_new_file {
	PK_DATA					Filename;
	PK_DATA					FileImage;
	uint16					Date;
	uint16					Time;
	uint32					crc;
    uint32					RelativeOffsetOfLocalHeader;
	struct pk_new_file*		Next;
} PK_NEW_FILE, *PK_NEW_FILE_PTR;

typedef struct pk_new_archive {
	ISL_ARCHIVE_CONTEXT_PTR		pMem;
	PK_NEW_FILE_PTR				Files;
} PK_NEW_ARCHIVE, *PK_NEW_ARCHIVE_PTR;
#endif /* #ifndef EISL */

#pragma pack()

#endif
