/*-----------------------------------------------------------------------
 * File: MANISEC.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
**	Implementation of ISL_MANIFEST_SECTIONMethods class for Jar archive format
*/
#include <sys/stat.h>
#include <stdio.h>
#include "isl_internal.h"
#include "islutil.h"
#include <isl_memtrack.h>

#ifndef EISL
extern struct isl_manifest_section_methods ManifestSectionMethods;
extern 	ISL_STATUS FindAttributeValue(	
				ISL_MANIFEST_SECTION_PTR Context,	
				ISL_CONST_DATA Attribute,			
				ISL_CONST_DATA_PTR Value); 			
#endif

/*
**	Signable Object (Manifest Section) Class
*/
/* class methods */
static 	ISL_SIZE SizeofObject()			/* returns sizeof object */
{
	return sizeof(ISL_MANIFEST_SECTION);
}

/* object methods */
static	ISL_STATUS InitializeNew(
		ISL_MANIFEST_SECTION_PTR result,		/* memory for new object */
		ISL_ARCHIVE_CONTEXT *Archive,			/* archive which will contain section */
		ISL_CONST_DATA Name,					/* name of object in manifest */
		ISL_CONST_DATA GetServiceName,			/* service to get data (in config) */
		ISL_CONST_DATA GetParameters)			/* get data parameters */
{
	if (!result) return ISL_FAIL;
	if (!Archive) return ISL_FAIL;

	memset(result, 0, sizeof(ISL_MANIFEST_SECTION));
    return ISL_OK;
}
	

static	ISL_STATUS Recycle(						/* destructor */
		ISL_MANIFEST_SECTION_PTR section) 		/* context */
{
	return ISL_OK;
}

static	ISL_ARCHIVE_CONTEXT *GetArchive(		/* get the archive */
		ISL_MANIFEST_SECTION *Context)		/* the object */
{
	if (!Context)
		return NULL;

	return Context->Parent;

}



static	ISL_STATUS AddDigest(			/* add digest algorithm to section */
		ISL_MANIFEST_SECTION_PTR Context,		/* context */
		const ISL_DIGEST_METHODS *DigestMethods) 		/* digest and parameters */
{
	ISL_ALG_INFO_PTR item, temp, prev;
	ISL_SERVICE_CLASS AlgClass;		/* return service type code */
	ISL_CONST_DATA AlgID;			/* return archive-specific encoding */
	ISL_CONST_DATA HumanName;	/* return human-readable description */

	if (!Context || !Context->Parent)
		return ISL_FAIL;

	if (!DigestMethods) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_ADD_DIGEST);
		sprintf(gErrBuf, gErrStr, Context, DigestMethods);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	/* allocate memory for item */
	item = (ISL_ALG_INFO *)isl_AllocateMemory(Context->Parent, sizeof(ISL_ALG_INFO));
	if (!item) {
		REPORT_ALLOCATE_MEMORY_ERROR(Context->Parent, sizeof(ISL_ALG_INFO));
		return ISL_FAIL;
	}

	memset(item, 0, sizeof(ISL_ALG_INFO)); 
	item->Methods = (ISL_DIGEST_METHODS_PTR) DigestMethods;
	item->Parent = Context;
	(item->Methods->ServiceMethods.id)(&AlgClass, &AlgID, &HumanName);
	
	item->AlgName.Length = AlgID.Length;
	item->AlgName.Data = isl_CopyMemory(Context->Parent, 
									(void *)AlgID.Data, AlgID.Length);
	if (!item->AlgName.Data) {
		REPORT_COPY_MEMORY_ERROR(Context->Parent, (void *)AlgID.Data, AlgID.Length);
	}
	/* add the algorithm to the end of algorithm list */
	temp = Context->SectionInfo.Algorithm;
	while (temp!= NULL)	{
		prev = temp;
		temp = temp->Next;
	}
	if (Context->SectionInfo.Algorithm == NULL) /* head of the attribute list */
		Context->SectionInfo.Algorithm = item;
	else
		prev->Next = item;

	Context->SectionInfo.AlgCount++;


	return ISL_OK;
}
	
static	ISL_STATUS RemoveDigest(			/* add digest algorithm to section */
		ISL_MANIFEST_SECTION_PTR Context,	/* archive context */
		const ISL_DIGEST_METHODS *Digest) 	/* digest methods */
{
	ISL_ALG_INFO_PTR	curr, prev;
	ISL_SECTION_PTR pSectionInfo;

	if (!Context || !Context->Parent)
		return ISL_FAIL;

	if (!Digest) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_REMOVE_DIGEST);
		sprintf(gErrBuf, gErrStr, Context, Digest);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	pSectionInfo = &Context->SectionInfo;

	for(curr = pSectionInfo->Algorithm, prev = pSectionInfo->Algorithm;
		curr;
		curr = curr->Next)
	{
		if (curr->Methods == Digest)
		{
			if (prev != curr)
			{
				prev->Next = curr->Next;
			}
			else
			{
				pSectionInfo->Algorithm = curr->Next;
			}

			pSectionInfo->AlgCount--;
			return ISL_OK;
		}
	}

	return ISL_FAIL;
}
static ISL_ITERATOR_PTR CreateDigestEnumerator(
		ISL_MANIFEST_SECTION_PTR Context)			/* pointer to signature context */
{
	ISL_ITERATOR_PTR		pItr = NULL;
	ISL_DATA				DigestMethodPointerArray = {0, NULL};
	ISL_DIGEST_METHODS_PTR	*DigestMethods = NULL;
	ISL_ALG_INFO_PTR pAlgorithms;
	unsigned long i = 0;

	if (!Context || !Context->Parent)
		return NULL;

	DigestMethodPointerArray.Length = sizeof(ISL_DIGEST_METHODS_PTR) *
										 Context->SectionInfo.AlgCount;

	DigestMethods = isl_AllocateMemory(Context->Parent,
										  DigestMethodPointerArray.Length);
	if (!DigestMethods) {
		REPORT_ALLOCATE_MEMORY_ERROR(
			Context->Parent,
			DigestMethodPointerArray.Length);
		return NULL;
	}
	
	for(pAlgorithms = Context->SectionInfo.Algorithm, i = 0; 
		pAlgorithms;
		pAlgorithms = pAlgorithms->Next, i++)
	{
		DigestMethods[i] = pAlgorithms->Methods;
	}

	DigestMethodPointerArray.Data = (void*)DigestMethods;

	pItr = isl_CreateIterator(Context->Parent, &DigestMethodPointerArray);
	isl_FreeMemory(Context->Parent, DigestMethods);
	if (!pItr) {
		REPORT_CREATE_ITERATOR_ERROR(Context->Parent, &DigestMethodPointerArray);
		return NULL;
	}
	return pItr;
}

static const ISL_DIGEST_METHODS *GetNextDigest(
		ISL_ITERATOR_PTR Iterator)
{
	void * node;
	
	if (!Iterator) return NULL;

	node = isl_GetNextNode(Iterator);
	return (ISL_DIGEST_METHODS *) node;
}

static ISL_STATUS RecycleDigestEnumerator(
		ISL_ITERATOR_PTR Iterator)
{
	if (!Iterator) return ISL_FAIL;

	return isl_RecycleIterator(Iterator);

}
	

static
ISL_STATUS
GetDigestValues(
    ISL_MANIFEST_SECTION *Context,
	ISL_DATA_PTR *pDigestValues,
    void *OutputContext,					/* object to update */
	ISL_STATUS (*Update)(					/* object's update method */
		void *UpdateContext,				/* (place OutputContext here) */
		ISL_CONST_DATA Buffer)) 			/* Length and Value of update */  
{
  	ISL_DIGEST_SERVICE_CONTEXT      **pDigestContexts = NULL;
    ISL_ARCHIVE_CONTEXT_PTR         pArchive;
	ISL_CONST_DATA                  algorithmParameter;
	ISL_CONST_DATA                  block;
	ISL_GET_DATA_SERVICE_CONTEXT    *GetDataContext;
	ISL_ALG_INFO_PTR digest;
    ISL_STATUS                      status;
	ISL_ERROR_CODE					error;
#ifdef EISL
	uint8                           name[256];
#endif
	uint32                          NumberOfDigests;
    uint32                          ContextSize;
    uint32                          i;
#ifdef TEST
	FILE							*fp =  NULL;
#endif
	ISL_SECTION_PTR					pSectionInfo;

	if (!Context || !Context->Parent) return ISL_FAIL;
	if (!pDigestValues) 
	{
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_GET_DIGEST_VALUES);
		sprintf(gErrBuf, gErrStr, Context, pDigestValues, OutputContext);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	pSectionInfo = &Context->SectionInfo;
	pArchive = Context->Parent;
    NumberOfDigests = pSectionInfo->AlgCount;
    pDigestContexts = isl_AllocateMemory(pArchive, NumberOfDigests * sizeof(void *));
	if (!pDigestContexts) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, NumberOfDigests * sizeof(void *));
		return ISL_FAIL;
	}

	*pDigestValues = isl_AllocateMemory(pArchive, NumberOfDigests * sizeof(ISL_DATA));
	if (!*pDigestValues) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, NumberOfDigests * sizeof(ISL_DATA));
		return ISL_FAIL;
	}
    /* setup all the digest algorithms */
	algorithmParameter.Data = NULL;
	algorithmParameter.Length = 0;
	for (i = 0, digest = pSectionInfo->Algorithm; 
        digest; 
        i++, digest = digest->Next) 
    {
        ISL_DIGEST_SERVICE_CONTEXT *pDigestContext = NULL;
        ISL_DATA                   DigestValue;

        ContextSize = (digest->Methods->SizeofObject)();
        pDigestContext = isl_AllocateMemory(pArchive,ContextSize);
		if (!pDigestContext) {
            REPORT_ALLOCATE_MEMORY_ERROR(pArchive,ContextSize);
		}
		status =  (digest->Methods->InitializeWithClass)(
				    pDigestContext, 
				    algorithmParameter, /* algorithm parameter */
				    (unsigned int *)&DigestValue.Length,            /* ifdef VMS apply cast */
					digest->Methods->ServiceMethods.Class);
		if (status != ISL_OK) {
			REPORT_DIGEST_INITIALIZE_ERROR(
					pArchive,
				    pDigestContext, 
				    algorithmParameter, /* algorithm parameter */
				    &DigestValue.Length);
			return ISL_FAIL;
		}

		DigestValue.Data = isl_AllocateMemory(pArchive, DigestValue.Length);
		if (!DigestValue.Data) {
			REPORT_ALLOCATE_MEMORY_ERROR(pArchive, DigestValue.Length);
			return ISL_FAIL;
		}
        pDigestContexts[i] = pDigestContext;
        (*pDigestValues)[i]   = DigestValue;
    }

	/* setup the get data routine */
    ContextSize = (*Context->GetDataMethods->SizeofObject)();
	GetDataContext = isl_AllocateMemory(pArchive,ContextSize);
	if (!GetDataContext) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive,ContextSize);
		return ISL_FAIL;
	}
#ifdef TEST
	fp = fopen("dump.bin","wb");
	if (!fp)
		return ISL_FAIL;
#endif
	status = (*Context->GetDataMethods->InitializeWithClass)(GetDataContext,
		Context->Parameters, Context->GetDataMethods->ServiceMethods.Class, Context); 
	if (status != ISL_OK) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_GET_DATA_INITIALIZE);
		sprintf(gErrBuf, gErrStr, GetDataContext, Context->Parameters.Data);
		isl_ReportError(pArchive, ISL_NO_MEMORY, gErrBuf);
#endif
		return ISL_FAIL;
	}
	/* run all the data through all the digests */
	for (;;) {
		error = (*Context->GetDataMethods->Update)(GetDataContext, &block);
		if (error != ISL_NO_ERROR) {
#if (!defined(EISL) || defined(DEBUG))
			gErrStr = isl_GetErrorStr(ISL_GET_DATA_UPDATE);
			sprintf(gErrBuf, gErrStr, GetDataContext, &block);
			isl_ReportError(pArchive, ISL_NO_MEMORY, gErrBuf);
#endif
			return ISL_FAIL;
		}
		if (block.Length == 0)
			break;
#ifdef TEST
		if (fwrite(block.Data, block.Length, 1, fp) != 1)
			return ISL_FAIL;
#endif
        if (Update)
        {
            status = (*Update)(OutputContext, block);
            if (status != ISL_OK)
                return status;
        }

		for (i = 0, digest = pSectionInfo->Algorithm; digest; i++, digest = digest->Next) {
			error = (digest->Methods->Update)(pDigestContexts[i], block);
			if (error != ISL_NO_ERROR) {
				REPORT_DIGEST_UPDATE_ERROR(pArchive, pDigestContexts[i], block);
				return ISL_FAIL;
			}
		}
	}
#ifdef TEST
	fclose(fp);
#endif
	error = (*Context->GetDataMethods->Recycle)(GetDataContext);
	if (error != ISL_NO_ERROR) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_GET_DATA_RECYCLE);
		sprintf(gErrBuf, gErrStr, GetDataContext);
		isl_ReportError(pArchive, ISL_NO_MEMORY, gErrBuf);
#endif
		return ISL_FAIL;
	}
	for (i = 0, digest = pSectionInfo->Algorithm; digest; i++, digest = digest->Next) {
		error = (digest->Methods->Recycle)(pDigestContexts[i], *pDigestValues + i);
		if (error != ISL_NO_ERROR) {
			REPORT_DIGEST_RECYCLE_ERROR(pArchive, pDigestContexts[i], *pDigestValues + i);
			return ISL_FAIL;
		}
    }
	return ISL_OK;
}

static BOOL HashReferenceOnly(
		ISL_MANIFEST_SECTION_PTR Context)
{
		ISL_STATUS status;
		ISL_CONST_DATA attrName;
		ISL_CONST_DATA attrValue;
		ISL_CONST_DATA value;

		/*check if the manifest section containing the 
		 "Integrity-VerifyData:reference" attribute */
		attrName.Data = (const unsigned char *)"Integrity-VerifyData";  /* ifdef VMS apply cast */
		attrName.Length = strlen((char*)attrName.Data);
		value.Data = (const unsigned char *)"reference";                /* ifdef VMS apply cast */
		value.Length = strlen((char*)value.Data);

		status = FindAttributeValue(Context, attrName, &attrValue);			
		if (status != ISL_OK)
			return CSSM_FALSE;

		if (IS_EQUAL(attrValue, value))
			return CSSM_TRUE;
		else
			return CSSM_FALSE;
}

	
static  ISL_STATUS VerifyCommon(
		ISL_MANIFEST_SECTION_PTR Context,		/* context */
		void *OutputContext,					/* object to update */
		ISL_STATUS (*Update)(					/* object's update method */
			void *UpdateContext,				/* (place OutputContext here) */
			ISL_CONST_DATA Buffer)) 			/* Length and Value of update */
{
	uint32 i;
	ISL_CONST_DATA *DigestValues;
	ISL_STATUS status;
	ISL_ALG_INFO_PTR digest;
	ISL_HASH_INFO_PTR hash;
	ISL_SECTION_PTR pSectionInfo;

	/*if this manifest section was hash by reference, then we don't
	  need to compare the hash value since there is no hash values for
	  the refernece object */
	if (HashReferenceOnly(Context)) 
		return ISL_OK;

    /* count the number of digest algorithms */
    status = GetDigestValues(Context, (ISL_DATA **)&DigestValues, OutputContext, Update);
    if (status != ISL_OK) {
		REPORT_SIGNABLE_OBJ_GET_DIGEST_VALUES(Context, &DigestValues, OutputContext, Update);
		return ISL_FAIL;
    }
	
	pSectionInfo = &Context->SectionInfo;
	for (i = 0, digest = pSectionInfo->Algorithm; digest; i++, digest = digest->Next) 
    {
        for (hash = digest->Hash; hash; hash = hash->Next) 
        {
            if (IS_EQUAL(hash->Value, DigestValues[i]))
                break;
        }

        if (!hash)
            return ISL_FAIL;
	}
	return ISL_OK;
}

static	ISL_STATUS Verify(
		ISL_MANIFEST_SECTION_PTR Context) 		/* context */
{
    return VerifyCommon(Context, NULL, NULL);
}

static	ISL_STATUS ExtractAndVerify(
		ISL_MANIFEST_SECTION_PTR Context,		/* context */
		void *OutputContext,					/* object to update */
		ISL_STATUS (*Update)(					/* object's update method */
			void *UpdateContext,				/* (place OutputContext here) */
			ISL_CONST_DATA Buffer)) 			/* Length and Value of update */
{
    return VerifyCommon(Context, OutputContext, Update);
}

static	ISL_STATUS AddDigestValue(			/* add new digest value for all digests */
		ISL_MANIFEST_SECTION *Context,		/* context */
		ISL_CONST_DATA VersionParameters) 	/* extra parameters for get method */
{
	int i;
	ISL_DATA *DigestValues;
	ISL_STATUS status;
	ISL_DATA DigestName;
	ISL_ALG_INFO_PTR digest;
	ISL_HASH_INFO_PTR hash, temp, prev;
	ISL_SECTION_PTR pSectionInfo;

	if (!Context || !Context->Parent) return ISL_FAIL;
	/* count the number of digest algorithms */
    status = GetDigestValues(Context, &DigestValues, NULL, NULL);
    if (status != ISL_OK) {
        REPORT_SIGNABLE_OBJ_GET_DIGEST_VALUES(Context, &DigestValues, NULL, NULL);
		return ISL_FAIL;
	}

	pSectionInfo = &Context->SectionInfo;
	for (i = 0, digest = pSectionInfo->Algorithm; digest; i++, digest = digest->Next) {
		/* the format of DigestName is :
		(algorithm)-Digest-(VersionParameters) */
		DigestName.Length =  digest->AlgName.Length + 7; /* 7 bytes for "Digest-" */
		if (VersionParameters.Length > 0)
			DigestName.Length += VersionParameters.Length + 1; /* 1 byte for "-" before
																 VersionParameters */
		DigestName.Data = isl_AllocateMemory(Context->Parent, DigestName.Length);
		if (!DigestName.Data) {
			REPORT_ALLOCATE_MEMORY_ERROR(Context->Parent, DigestName.Length);
		}
		memcpy(DigestName.Data, digest->AlgName.Data, digest->AlgName.Length);
		memcpy(DigestName.Data+digest->AlgName.Length, "-Digest", 7);
		if (VersionParameters.Length > 0){
			memcpy(DigestName.Data+digest->AlgName.Length+7, "-", 1);
			memcpy(DigestName.Data+digest->AlgName.Length+7+1, VersionParameters.Data, VersionParameters.Length);
		}
		digest->HashCount++;
		hash = isl_AllocateMemory(Context->Parent, sizeof(ISL_HASH_INFO));
		if (!hash) {
			REPORT_ALLOCATE_MEMORY_ERROR(Context->Parent, sizeof(ISL_HASH_INFO));
			return ISL_FAIL;
		}
		hash->Name.Length = DigestName.Length;
		hash->Name.Data = DigestName.Data;
		hash->Value.Length = DigestValues[i].Length;
		hash->Value.Data = DigestValues[i].Data;
		hash->Next = NULL;

		temp = digest->Hash;
		while (temp!= NULL)	{
			prev = temp;
			temp = temp->Next;
		}
		if (digest->Hash == NULL) /* head of the attribute list */
			digest->Hash = hash;
		else
			prev->Next = hash;
	}
	return ISL_OK;


}
	
static	ISL_STATUS AddAttribute(			/* add metadata to section */
		ISL_MANIFEST_SECTION_PTR Context,	/* archive context */
		ISL_CONST_DATA Attribute,			/* attribute "name" */
		ISL_CONST_DATA Value) 				/* attribute value */
{

	ISL_NAME_VALUE *item;
	ISL_NAME_VALUE *temp, *prev;
	ISL_SECTION_PTR pSectionInfo;

	if (!Context || !Context->Parent)
		return ISL_FAIL;

	if ( (!Attribute.Data) ||
		 (!Value.Data) )
	{
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_ADD_ATTRIBUTE);
		sprintf(gErrBuf, gErrStr, Context, Attribute.Data, Value.Data);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	if ((item = isl_CreateAttribute(Context->Parent, Attribute, Value)) == NULL)
		return ISL_FAIL;

	pSectionInfo = &Context->SectionInfo;

	/* add the attribute to the end of attribute list */
	temp = pSectionInfo->Attribute;
	while (temp!= NULL)	{
		prev = temp;
		temp = temp->Next;
	}
	if (pSectionInfo->Attribute == NULL) /* head of the attribute list */
		pSectionInfo->Attribute = item;
	else
		prev->Next = item;

	pSectionInfo->AttributeCount++;
	return ISL_OK;

}

static	ISL_STATUS RemoveAttribute(			/* add metadata to section */
		ISL_MANIFEST_SECTION_PTR Context,		/* archive context */
		ISL_CONST_DATA Attribute) 				/* attribute "name" */
{
	ISL_NAME_VALUE *curr, *prev;
	ISL_SECTION_PTR pSectionInfo;
	
	if ( !Attribute.Data) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_REMOVE_ATTRIBUTE);
		sprintf(gErrBuf, gErrStr, Context, Attribute.Data);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	
	pSectionInfo = &Context->SectionInfo;
	prev = pSectionInfo->Attribute;
	for (curr = prev; curr; curr = curr->Next)
	{
		if (IS_EQUAL(curr->Name, Attribute))
		{
			if (prev != curr) {
				prev->Next = curr->Next;
			} else {
				pSectionInfo->Attribute = curr->Next;
			}

			pSectionInfo->AttributeCount--;
			return ISL_OK;
		}
		prev = curr;
	}

#if (!defined(EISL) || defined(DEBUG))
	gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_REMOVE_ATTRIBUTE);
	sprintf(gErrBuf, gErrStr, Context, Attribute.Data);
	isl_ReportError(Context->Parent, ISL_OBJECT_NOT_FOUND, gErrBuf);
#endif
	return ISL_FAIL;
}

ISL_STATUS FindAttributeValue(			/* find metadata in section */
	ISL_MANIFEST_SECTION_PTR Context,	/* manifest section */
	ISL_CONST_DATA Attribute,			/* attribute "name" */
	ISL_CONST_DATA_PTR Value) 			/* fill in Value and Length if found */
{
	ISL_NAME_VALUE *curr;
	ISL_SECTION_PTR pSectionInfo;
	
	if ((!Attribute.Data) || 
		(!Value)) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_FIND_ATTRIBUTE_VALUE);
		sprintf(gErrBuf, gErrStr, Context, Attribute.Data, Value);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	pSectionInfo = &Context->SectionInfo;
	for (curr = pSectionInfo->Attribute; curr; curr = curr->Next)
	{
		if (IS_EQUAL(curr->Name, Attribute))
		{
			Value->Length  = curr->Value.Length;	/* copy the unique section name */
			Value->Data =  isl_CopyMemory(Context->Parent, curr->Value.Data, Value->Length);
			if (!Value->Data) {
				REPORT_COPY_MEMORY_ERROR(Context->Parent, curr->Value.Data, Value->Length);
			}
			return ISL_OK;
		}
	}

#if (!defined(EISL) || defined(DEBUG))
	gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_FIND_ATTRIBUTE_VALUE);
	sprintf(gErrBuf, gErrStr, Context, Attribute.Data, Value);
	isl_ReportError(Context->Parent, ISL_OBJECT_NOT_FOUND, gErrBuf);
#endif
	return ISL_FAIL;
}

static ISL_ITERATOR_PTR CreateAttributeEnumerator(
	ISL_MANIFEST_SECTION_PTR Context)		/* pointer to section */
{
	ISL_ITERATOR_PTR pItr = NULL;
	ISL_ARCHIVE_CONTEXT_PTR pArchive;
	ISL_DATA				AttributePointerArray = {0, NULL};
	ISL_NAME_VALUE_PTR		*Attributes = NULL;
	ISL_NAME_VALUE_PTR		pAttribute = NULL;
	uint32					i = 0;
	ISL_SECTION_PTR pSectionInfo;

	if (!Context || !Context->Parent)
		return NULL;

	pArchive = Context->Parent;
	pSectionInfo = &Context->SectionInfo;

	AttributePointerArray.Length = sizeof(ISL_NAME_VALUE_PTR) * pSectionInfo->AttributeCount;
	Attributes = isl_AllocateMemory(pArchive, AttributePointerArray.Length);
	if (!Attributes) {
		REPORT_ALLOCATE_MEMORY_ERROR(pArchive, AttributePointerArray.Length);
		return NULL;
	}

	AttributePointerArray.Data = (void*)Attributes;

	for(pAttribute = pSectionInfo->Attribute, i = 0;
		pAttribute;
		pAttribute = pAttribute->Next, i++)
	{
		Attributes[i] = pAttribute;
	}

	pItr = isl_CreateIterator(pArchive, &AttributePointerArray);
	isl_FreeMemory(pArchive, Attributes);
	if (!pItr) {
		REPORT_CREATE_ITERATOR_ERROR(pArchive, &AttributePointerArray);
		return NULL;
	}
	return pItr;
}

static ISL_STATUS GetNextAttribute(
	ISL_ITERATOR_PTR Context,
	ISL_CONST_DATA_PTR Name,
	ISL_CONST_DATA_PTR Value)
{
	ISL_NAME_VALUE_PTR node = NULL;

	if (!Context) return ISL_FAIL;

	if ( (!Name) || (!Value)) {
	#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_GET_NEXT_ATTRIBUTE);
		sprintf(gErrBuf, gErrStr, Context, Name, Value);
		isl_ReportError(Context->pArchive, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	node = (ISL_NAME_VALUE_PTR) isl_GetNextNode(Context);
	if (!node)
		return ISL_FAIL;

	Name->Length  = node->Name.Length;
	Name->Data	  = node->Name.Data;
	Value->Length = node->Value.Length;
	Value->Data   = node->Value.Data;
	return ISL_OK;
}

static ISL_STATUS RecycleAttributeEnumerator(
		ISL_ITERATOR_PTR Iterator)
{
	return isl_RecycleIterator(Iterator);
}

static ISL_STATUS GetServiceName(
		ISL_MANIFEST_SECTION_PTR Context,
		ISL_CONST_DATA_PTR Value) 
{
	if (!Context || (!Context->SectionInfo.Name.Data) || (Context->SectionInfo.Name.Length == 0))
		return ISL_FAIL;

	if (!Value) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_GET_SERVICE_NAME);
		sprintf(gErrBuf, gErrStr, Context, Value);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	/* get service name */

	Value->Length  = Context->Protocol.Length;	/* copy the unique section name */
	Value->Data =  isl_CopyMemory(Context->Parent, (void *)Context->Protocol.Data, Context->Protocol.Length);
	if (!Value->Data) {
		REPORT_COPY_MEMORY_ERROR(Context->Parent, Context->Protocol.Data, Context->Protocol.Length);
	}
	return ISL_OK;
}

static ISL_STATUS GetParameters(
		ISL_MANIFEST_SECTION_PTR Context,
		ISL_CONST_DATA_PTR Value)
{
	if ((!Context) || (!Context->Parameters.Data) || (Context->Parameters.Length == 0))
		return ISL_FAIL;

	if (!Value) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNABLE_OBJ_GET_PARAMETERS);
		sprintf(gErrBuf, gErrStr, Context, Value);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	/* get service name */
	Value->Length  = Context->Parameters.Length;	/* copy the unique section name */
	Value->Data =  isl_CopyMemory(Context->Parent, (void*)Context->Parameters.Data, Context->Parameters.Length);
	if (!Value->Data) {
		REPORT_COPY_MEMORY_ERROR(Context->Parent, (void*)Context->Parameters.Data, Context->Parameters.Length);
	}
	return ISL_OK;
}

static ISL_STATUS GetSectionName(
		ISL_MANIFEST_SECTION_PTR Context,
		ISL_CONST_DATA_PTR Value) 
{
	ISL_SECTION_PTR pSectionInfo;

	if ((!Context) || (!Context->SectionInfo.SectionName.Data) || (Context->SectionInfo.SectionName.Length == 0))
		return ISL_FAIL;

	if (!Value) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_ARCHIVE_FIND_SIGNATURE);
		sprintf(gErrBuf, gErrStr, Context, Value);
		isl_ReportError(Context->Parent, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	pSectionInfo = &Context->SectionInfo;
	/* get service name */
	Value->Length  = pSectionInfo->SectionName.Length;	/* copy the unique section name */
	Value->Data =  isl_CopyMemory(Context->Parent, 
		pSectionInfo->SectionName.Data, pSectionInfo->SectionName.Length);
	if (!Value->Data) {
		REPORT_COPY_MEMORY_ERROR(Context->Parent, pSectionInfo->SectionName.Data,
			pSectionInfo->SectionName.Length);
	}
	return ISL_OK;
}



struct isl_manifest_section_methods ManifestSectionMethods = {
	SizeofObject,
	Recycle,
	GetArchive,
	AddDigest,
	RemoveDigest,
	CreateDigestEnumerator,
	GetNextDigest,
	RecycleDigestEnumerator,
	Verify,
	ExtractAndVerify,
	AddDigestValue,
	AddAttribute,
	RemoveAttribute,
	FindAttributeValue,
	CreateAttributeEnumerator,
	GetNextAttribute,
	RecycleAttributeEnumerator,
	GetServiceName,
	GetParameters,
	GetSectionName,
	0	
};
