/*-----------------------------------------------------------------------
* File: CRTCREAT.C
*
* Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
*-----------------------------------------------------------------------
*/
/*
* INTEL CONFIDENTIAL 
* This file, software, or program is supplied under the terms of a 
* license agreement or nondisclosure agreement with Intel Corporation 
* and may not be copied or disclosed except in accordance with the 
* terms of that agreement. This file, software, or program contains 
* copyrighted material and/or trade secret information of Intel 
* Corporation, and must be treated as such. Intel reserves all rights 
* in this material, except as the license agreement or nondisclosure 
* agreement specifically indicate. 
*/
/* 
* WARNING: EXPORT RESTRICTED. 
* This software listing contains cryptographic methods and technology. 
* It is export restricted by the Office of Defense Trade Controls, United 
* States Department of State and cannot be downloaded or otherwise 
* exported or re-exported (i) into (or to a national or resident of) Cuba, 
* Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
* to which the US has embargoed goods; or (ii) to anyone on the US 
* Treasury Department's list of Specially Designated Nationals or the US 
* Commerce Department's Table of Denial Orders. By downloading or using 
* this product, you are agreeing to the foregoing and you are representing 
* and warranting that you are not located in, under the control of, or a 
* national or resident of any such country or on any such list. 
*/


#include <stdio.h>
#include <string.h>

#include "cssm.h"
#include "cssmMemTrack.h"
#include "algoid.h"
#include "x509defs.h"

#include "oidsalg.h"
#include "credutil.h"
#include "oidsattr.h"
#include "oidscert.h"
#include "cl_public.h"
#include "memfuncs.h"
#include "certgen.h"

extern CSSM_UPCALLS MemoryFunctions;
extern CSSM_CSP_HANDLE CSPHandle;
extern CSSM_CL_HANDLE CLHandle;

static CSSM_BOOL x509_FreeName(CSSM_X509_NAME_PTR ptr)
{
    uint32 i=0, j=0;

    /***** Remove this line after finding the freeing issue */
    return CSSM_TRUE;


    if (!ptr)
    {
        return CSSM_TRUE;
    }
    if (!ptr->RelativeDistinguishedName)
    {
        return CSSM_TRUE;
    }
    /* For each RDN */
    for (i=0; i < ptr->numberOfRDNs; i++)
    {
        /* Check that the AV pair can be freed */
        /* If not, continue */
        if (!ptr->RelativeDistinguishedName[i].numberOfPairs || !ptr->RelativeDistinguishedName[i].AttributeTypeAndValue )
            continue;
        
        /* For each attribute/value pair */
        for (j=0; j < ptr->RelativeDistinguishedName[i].numberOfPairs; j++)
        {
            CSSM_FREE_WRAPPER(_app_free, ptr->RelativeDistinguishedName[i].AttributeTypeAndValue[j].type.Data, NULL);
									
            CSSM_FREE_WRAPPER(_app_free, ptr->RelativeDistinguishedName[i].AttributeTypeAndValue[j].value.Data, NULL);
									
        }
        CSSM_FREE_WRAPPER(_app_free, ptr->RelativeDistinguishedName[i].AttributeTypeAndValue, NULL);
								
    }
    CSSM_FREE_WRAPPER(_app_free, ptr->RelativeDistinguishedName, NULL);

    return CSSM_TRUE;
}

static void x509_freesignedCert(CSSM_X509_SIGNED_CERTIFICATE_PTR CertPtr,
                                CSSM_BOOL SelfSigned)
{
    /* Frees SPKI, Issuer Name, Subject Name */
    /* If Self Signed, only needs to free one of Issuer and Subject */

    if (CertPtr->certificate.subjectPublicKeyInfo.algorithm.algorithm.Data)
        CSSM_FREE_WRAPPER(_app_free, CertPtr->certificate.subjectPublicKeyInfo.algorithm.algorithm.Data, NULL);
    if (CertPtr->certificate.subjectPublicKeyInfo.algorithm.parameters.Data)
        CSSM_FREE_WRAPPER(_app_free, CertPtr->certificate.subjectPublicKeyInfo.algorithm.parameters.Data, NULL);
    if (CertPtr->certificate.subjectPublicKeyInfo.subjectPublicKey.Data)
        CSSM_FREE_WRAPPER(_app_free, CertPtr->certificate.subjectPublicKeyInfo.subjectPublicKey.Data, NULL);

    x509_FreeName(&CertPtr->certificate.issuer);
    if (!SelfSigned)
    {
        x509_FreeName(&CertPtr->certificate.subject);
    }
    memset(CertPtr, 0, sizeof(CSSM_X509_SIGNED_CERTIFICATE));
    return;
}

CSSM_RETURN CreateCert(const CSSM_DATA * CertTemplateOrVendorCertDataPtr,
                       CSSM_BOOL VendorCert, 
                       CSSM_BOOL SelfSigned,
                       CSSM_BOOL UseOldDSAOID,
                       uint32 Serial,
                       const CSSM_DATA * IssuerCertPtr,
                       uint32 KeySize, 
                       CSSM_ALGORITHMS KeyAlg,
                       uint8 *szNotBefore,
                       uint8 *szNotAfter,
                       char *subject_pass,
                       char *issuer_pass,
                       CSSM_DATA_PTR CertPtr)
{
    CSSM_X509_NAME_PTR x509_name_ptr = NULL;
    CSSM_RETURN ret = CSSM_OK;
    CSSM_X509_SIGNED_CERTIFICATE SignedCert;
    CSSM_DATA_PTR tmpCertPtr = NULL;

    CREDUTIL_PASSWORD_ACCESS_CREDENTIAL Sub_AccessCred, Iss_AccessCred;
    CREDUTIL_PASSWORD_RESOURCE_CONTROL_CONTEXT Sub_InitialAcl, Iss_InitialAcl;
    CSSM_RESOURCE_CONTROL_CONTEXT *Sub_CredAndAclEntry, *Iss_CredAndAclEntry;

    CSSM_FIELD field;
    uint32 num_fields = 0;

    CSSM_HANDLE ResultsHandle = 0;
    uint32 NumMatchedFields = 0;
    CSSM_DATA_PTR Value = NULL;
    CSSM_OID *tmpalgoid;


    
    CSSM_CC_HANDLE   KeyCCHandle = 0;

    uint8			 SeedBuf[] = "This is a seed.";
    CSSM_CRYPTO_DATA Seed;

    
    CSSM_KEY         sub_pub_key;
    CSSM_KEY         sub_priv_key;

    CSSM_ALGORITHMS tmpalgid ;
    CSSM_KEY_PTR     iss_pub_key_ptr = NULL;
    CSSM_KEY         iss_priv_key;
    CSSM_KEY_PTR     vendor_pub_key_ptr = NULL;
    

    
    CSSM_FIELD_PTR ParsedFieldPtr = NULL;
    uint32 NumParsedFields = 0;


    memset(&Seed, 0 , sizeof(CSSM_CRYPTO_DATA));
    Seed.Param.Length = sizeof(SeedBuf);
    Seed.Param.Data = SeedBuf;
    
    /* Sanity Check */
    if (SelfSigned && VendorCert )
    {
        /* Nice try. */
        fprintf(stdout, "Not possible to generate a SelfSigned Vendor Cert!\n");
        return CSSMERR_CL_FUNCTION_FAILED;
    }
    /* Initializations */
    CertPtr->Data = NULL;
    CertPtr->Length = 0;
    memset(&SignedCert, 0, sizeof(CSSM_X509_SIGNED_CERTIFICATE));
    iss_priv_key.KeyData.Data = NULL;

    if (!VendorCert)
    {
        /* Get the X509 Name parsed from XML file */
        ret = ParseTemplate(CertTemplateOrVendorCertDataPtr, &ParsedFieldPtr, &NumParsedFields);
                


        if (CSSM_OK != ret)
        {
            return ret;
        }

        x509_name_ptr = (CSSM_X509_NAME_PTR)ParsedFieldPtr->FieldValue.Data;
        CSSM_FREE_WRAPPER(_app_free, ParsedFieldPtr, NULL);
    
        Sub_CredAndAclEntry = CREDUTIL_CreatePasswordRCContext( &Sub_InitialAcl,
                                            subject_pass,
                                            NULL, /* Prompt */
                                            NULL, NULL, /* Subject callback */
                                            NULL, 0, /* Auth tags */
                                            NULL ); /* Access creds */


    

    
        /* Generate a key */
    
        ret = CSSM_CSP_CreateKeyGenContext (
                CSPHandle, 
                KeyAlg, 
                KeySize, 
                &Seed, 
                NULL, /* Salt */
                NULL, /* StartDate */
                NULL, /* EndDate */
                NULL, /*Params */
                &KeyCCHandle);

        if (CSSM_OK != ret)
        {
            x509_FreeName(x509_name_ptr);
            return ret;
        }
    
        memset(&sub_pub_key, 0, sizeof(CSSM_KEY));
        memset(&sub_priv_key, 0, sizeof(CSSM_KEY));
    

        ret = CSSM_GenerateKeyPair(KeyCCHandle, 
                CSSM_KEYUSE_ANY,
                CSSM_KEYATTR_RETURN_DEFAULT|CSSM_KEYATTR_EXTRACTABLE, 
                NULL, /* PublicKeyLabel */
                &sub_pub_key, 
                CSSM_KEYUSE_ANY,
                CSSM_KEYATTR_RETURN_DEFAULT|CSSM_KEYATTR_PERMANENT, 
                NULL, /* PrivateKeyLabel */
                Sub_CredAndAclEntry, /* CredAndAclEntry */
                &sub_priv_key);

    

        CSSM_DeleteContext(KeyCCHandle);

        if (CSSM_OK != ret)
        {
            x509_FreeName(x509_name_ptr);
            return ret;
        }

        
    }
    else
    {
    
        /* Verify Vendor cert */
        ret = CSSM_CL_CertVerify(CLHandle, 
                                 0,
                                 CertTemplateOrVendorCertDataPtr,
                                 CertTemplateOrVendorCertDataPtr,
                                 NULL,
                                 0);
        if (CSSM_OK != ret)
        {
            fprintf(stdout, "Vendor Certificate failed to verify\n");
            return ret;
        }



        /* Get public key from Vendor cert */
        ret = CSSM_CL_CertGetKeyInfo(CLHandle,
                                     CertTemplateOrVendorCertDataPtr,
                                     &vendor_pub_key_ptr);
        
        if (CSSM_OK != ret || !vendor_pub_key_ptr)
        {
            fprintf(stdout, "Unable to obtain public key from Vendor Cert\n");
            if (vendor_pub_key_ptr)
            {
                CSSM_FREE_WRAPPER(_app_free, vendor_pub_key_ptr, NULL);
            }
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }    
        }
        sub_pub_key = *vendor_pub_key_ptr;
        CSSM_FREE_WRAPPER(_app_free, vendor_pub_key_ptr, NULL);
        memset(&sub_priv_key, 0, sizeof(CSSM_KEY));
        
        /* Get subject name from vendor cert */
        Value = NULL;
        NumMatchedFields = 0;

        ret = CSSM_CL_CertGetFirstFieldValue(CLHandle,
                                             CertTemplateOrVendorCertDataPtr,
                                             &CSSMOID_X509V1SubjectNameCStruct,
                                             &ResultsHandle,
                                             &NumMatchedFields,
                                             &Value);
        
        if (!Value || !Value->Data || CSSM_OK != ret 
            || NumMatchedFields != 1 || ResultsHandle != 0)
        {
            /* Error control */
            if (Value && Value->Data)
            {
                CSSM_FREE_WRAPPER(_app_free, Value->Data, NULL);
            }
            if (Value)
            {
                CSSM_FREE_WRAPPER(_app_free, Value, NULL);
            }
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }

        }
        
        x509_name_ptr = (CSSM_X509_NAME_PTR)Value->Data;
        CSSM_FREE_WRAPPER(_app_free, Value, NULL);
        Value = NULL;
        NumMatchedFields = 0;

    }

    
    
    /* Set up Cert fields */
    /* Lets set up the SPKI first since we might need the ALGOID/Params */
    ret = CSSMKeyToSpki(&sub_pub_key,
                  &SignedCert.certificate.subjectPublicKeyInfo);

    if (CSSM_OK != ret)
    {
        if (!VendorCert) 
            CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
        CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
        x509_FreeName(x509_name_ptr);
        return ret;
    }

    /* Version is default and already set to 0 */

    /* Serial number */
    SignedCert.certificate.serialNumber.Length = sizeof(Serial);
    SignedCert.certificate.serialNumber.Data = (uint8*)&Serial;
    
    /* Set up validity fields */
    SignedCert.certificate.validity.notBefore.timeType = BER_TAG_UTC_TIME;
    SignedCert.certificate.validity.notBefore.time.Length = strlen((char *)szNotBefore);
    SignedCert.certificate.validity.notBefore.time.Data = szNotBefore;
    
    SignedCert.certificate.validity.notAfter.timeType = BER_TAG_UTC_TIME;
    SignedCert.certificate.validity.notAfter.time.Length = strlen((char *)szNotAfter);
    SignedCert.certificate.validity.notAfter.time.Data = szNotAfter;
        

    if (SelfSigned)
    {
        /* Signature OID/params */
        if (CSSM_ALGID_DSA == KeyAlg)
        {
            SignedCert.certificate.signature.algorithm = CSSMOID_X9CM_DSAWithSHA1;
        }
        else 
        {
            SignedCert.certificate.signature.algorithm = CSSMOID_MD5WithRSA;
        }
        SignedCert.certificate.signature.parameters  = 
            SignedCert.certificate.subjectPublicKeyInfo.algorithm.parameters;
            
        
        /* Issuer */
        SignedCert.certificate.issuer = *x509_name_ptr;
        
        /* Subject */
        SignedCert.certificate.subject = *x509_name_ptr;

    }
    else
    {

        /* Obtain signature ALGOID/Params from issuercert */
        ret = CSSM_CL_CertGetFirstFieldValue(CLHandle,
                                             IssuerCertPtr,
                                             &CSSMOID_X509V1SubjectPublicKeyAlgorithm,
                                             &ResultsHandle,
                                             &NumMatchedFields,
                                             &Value);
        if (!Value || !Value->Data || CSSM_OK != ret 
            || NumMatchedFields != 1 || ResultsHandle != 0)
        {
            /* Error control */

            if (Value && Value->Data)
            {
                CSSM_FREE_WRAPPER(_app_free, Value->Data, NULL);
            }
            if (Value)
            {
                CSSM_FREE_WRAPPER(_app_free, Value, NULL);
            }
            if (!VendorCert) 
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            x509_FreeName(x509_name_ptr);
            x509_freesignedCert(&SignedCert, SelfSigned);
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }

        }
        
        tmpalgoid = (CSSM_OID *)Value;
        tmpalgid = X509_AlgorithmOidToAlgId(tmpalgoid);
        if (CSSM_ALGID_DSA == tmpalgid)
        {
            SignedCert.certificate.signature.algorithm = CSSMOID_X9CM_DSAWithSHA1;
        }
        else if (CSSM_ALGID_RSA == tmpalgid)
        {
            SignedCert.certificate.signature.algorithm = CSSMOID_MD5WithRSA;
        }

        CSSM_FREE_WRAPPER(_app_free, Value->Data, NULL);
        CSSM_FREE_WRAPPER(_app_free, Value, NULL);

        Value = NULL;
        NumMatchedFields = 0;

        ret = CSSM_CL_CertGetFirstFieldValue(CLHandle,
                                             IssuerCertPtr,
                                             &CSSMOID_X509V1SubjectPublicKeyAlgorithmParameters,
                                             &ResultsHandle,
                                             &NumMatchedFields,
                                             &Value);
        
        if (!Value || !Value->Data || CSSM_OK != ret 
            || NumMatchedFields != 1 || ResultsHandle != 0)
        {
            /* Error control */
            if (Value && Value->Data)
            {
                CSSM_FREE_WRAPPER(_app_free, Value->Data, NULL);
            }
            if (Value)
            {
                CSSM_FREE_WRAPPER(_app_free, Value, NULL);
            }
            if (!VendorCert) 
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            x509_FreeName(x509_name_ptr);
            x509_freesignedCert(&SignedCert, SelfSigned);
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }
        }
        
        
        SignedCert.certificate.signature.parameters = *Value;
        CSSM_FREE_WRAPPER(_app_free, Value, NULL);

        /* Subject */
        SignedCert.certificate.subject = *x509_name_ptr;

        /* Obtain Issuer name from IssuerCert */
        Value = NULL;
        NumMatchedFields = 0;

        ret = CSSM_CL_CertGetFirstFieldValue(CLHandle,
                                             IssuerCertPtr,
                                             &CSSMOID_X509V1SubjectNameCStruct,
                                             &ResultsHandle,
                                             &NumMatchedFields,
                                             &Value);
        
        if (!Value || !Value->Data || CSSM_OK != ret 
            || NumMatchedFields != 1 || ResultsHandle != 0)
        {
            /* Error control */
            if (Value && Value->Data)
            {
                CSSM_FREE_WRAPPER(_app_free, Value->Data, NULL);
            }
            if (Value)
            {
                CSSM_FREE_WRAPPER(_app_free, Value, NULL);
            }
            if (!VendorCert) 
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            x509_FreeName(x509_name_ptr);
            x509_freesignedCert(&SignedCert, SelfSigned);
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }

        }
        
        CSSM_FREE_WRAPPER(_app_free, x509_name_ptr, NULL);
        x509_name_ptr = (CSSM_X509_NAME_PTR)Value->Data;
        SignedCert.certificate.issuer = *x509_name_ptr;
        CSSM_FREE_WRAPPER(_app_free, Value, NULL);
        CSSM_FREE_WRAPPER(_app_free, x509_name_ptr, NULL);

        /* Obtain Public Key from IssuerCert */
        
        ret = CSSM_CL_CertGetKeyInfo(CLHandle,
                                     IssuerCertPtr,
                                     &iss_pub_key_ptr);
        
        if (CSSM_OK != ret || !iss_pub_key_ptr)
        {
            /* Error control */
            if (!VendorCert) 
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            if (iss_pub_key_ptr && iss_pub_key_ptr->KeyData.Data)
            {
                CSSM_FREE_WRAPPER(_app_free, iss_pub_key_ptr->KeyData.Data, NULL);
            }
            if (iss_pub_key_ptr)
            {
                CSSM_FREE_WRAPPER(_app_free, iss_pub_key_ptr, NULL);
            }
            x509_freesignedCert(&SignedCert, SelfSigned);
            if (CSSM_OK != ret)
            {
                return ret;
            }
            else
            {
                return CSSMERR_CL_FUNCTION_FAILED;
            }
        }
        
        
        
        memset(&iss_priv_key, 0, sizeof(CSSM_KEY));
        ret = CSSM_CSP_ObtainPrivateKeyFromPublicKey(CSPHandle,
                                                     iss_pub_key_ptr,
                                                     &iss_priv_key);
        CSSM_FREE_WRAPPER(_app_free, iss_pub_key_ptr->KeyData.Data, NULL);
        CSSM_FREE_WRAPPER(_app_free, iss_pub_key_ptr, NULL);
        
        if (CSSM_OK != ret)
        {
            /* Error Control */
            if (!VendorCert) 
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
            CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
            if (iss_priv_key.KeyData.Data)
            {
                CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Iss_AccessCred, issuer_pass, NULL, NULL), &iss_priv_key, CSSM_FALSE);
            }
            x509_freesignedCert(&SignedCert, SelfSigned);
            return ret;       
        }
    }
    
    SignedCert.signature.algorithmIdentifier = SignedCert.certificate.signature;

    /* Create template */
    field.FieldOid = CSSMOID_X509V3SignedCertificateCStruct;
    field.FieldValue.Length = sizeof(CSSM_X509_SIGNED_CERTIFICATE);
    field.FieldValue.Data = (uint8 *)&SignedCert;
    num_fields = 1;


    

    if (SelfSigned)
    {
        ret = CL_TEST_PUBLIC_CreateNewCert(CLHandle, 
                CSPHandle, 
                &field, 
                num_fields, 
                X509_AlgorithmOidToAlgId(&SignedCert.signature.algorithmIdentifier.algorithm), 
                &sub_priv_key, 
                CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), 
                MemoryFunctions, 
                &tmpCertPtr);
    }
    else
    {
        
        Iss_CredAndAclEntry = CREDUTIL_CreatePasswordRCContext( &Iss_InitialAcl,
                                        issuer_pass,
                                        NULL, /* Prompt */
                                        NULL, NULL, /* Subject callback */
                                        NULL, 0, /* Auth tags */
                                        NULL ); /* Access creds */

        ret = CL_TEST_PUBLIC_CreateNewCert(CLHandle, 
                CSPHandle, 
                &field, 
                num_fields, 
                X509_AlgorithmOidToAlgId(&SignedCert.signature.algorithmIdentifier.algorithm), 
                &iss_priv_key, 
                CREDUTIL_CreatePasswordCredentials(&Iss_AccessCred, issuer_pass, NULL, NULL), 
                MemoryFunctions, 
                &tmpCertPtr);
    }


    if (!VendorCert)
        CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Sub_AccessCred, subject_pass, NULL, NULL), &sub_priv_key, CSSM_FALSE);
    CSSM_FREE_WRAPPER(_app_free, sub_pub_key.KeyData.Data, NULL);
    if (!SelfSigned)
    {
        CSSM_FreeKey(CSPHandle, CREDUTIL_CreatePasswordCredentials(&Iss_AccessCred, issuer_pass, NULL, NULL), &iss_priv_key, CSSM_FALSE);
    }
    
    x509_freesignedCert(&SignedCert, SelfSigned);
    if (CSSM_OK != ret)
    {
        /* Error */

        if (tmpCertPtr && tmpCertPtr->Data)
        {
            CSSM_FREE_WRAPPER(_app_free, tmpCertPtr->Data, NULL);
        }
        if (tmpCertPtr)
        {
            CSSM_FREE_WRAPPER(_app_free, tmpCertPtr, NULL);
        }
        return ret;

    }
    *CertPtr = *tmpCertPtr;
    return CSSM_OK;
}
