
#include "cssm.h"
#include <stdio.h>
#ifdef VMS
#include <stdlib.h>
#include <string.h>
#endif

int CSSMDataToOIDDotNotation(CSSM_DATA_PTR pData,char* buffer,int maxLength)
{
	unsigned char* Octets ;
	uint32 dataLength;
	uint32 arrayLength;
    uint32 *Numbers;
	uint32 count = 0;
	char temp[32];
	uint32 i;

	if((pData == NULL) || (pData->Data == NULL)|| (pData->Length == 0))
	{
		return 0;
	}

	// allocate enough space to unpack the OID into an array of ints
	// it won't be longer than byte array of the oid +1, but it might be
	// shorter.
	Octets = pData->Data;
	dataLength = pData->Length;
	arrayLength = dataLength+1;
    Numbers = (uint32*)malloc(sizeof(uint32)*arrayLength);

   
	// decode BER number into an array of integers
    /* first byte contains two multiplicatively packed numbers */
    dataLength--;
    Numbers[count++] = *Octets / 40;
    Numbers[count++] = *Octets++ % 40;
    while (dataLength > 0) 
	{
        if (count >= arrayLength) 
        {
			free(Numbers);
			return 0;
		}
        Numbers[count] = 0;
        do 
		{
            if (dataLength-- <= 0)
            {
				free(Numbers);
				return 0;
			}
			Numbers[count] = Numbers[count] * 128 + (*Octets & 127);
        } 
		while ((*Octets++ & 0x80) != 0);
        count++;
    }


	// now convert each integer in array into a string
	// and separate by dots
	buffer[0] = 0;
	
	for (  i = 0; i < count; i++ )
	{	
		sprintf(temp,"%d",Numbers[i]);
		strcat(buffer,temp);
		
		if ( i+1 < count )
		{
			strcat(buffer,".");
		}
	}

	free(Numbers);
	return strlen(buffer);
}

int DotOidToCSSMData(char* pOidStr,CSSM_DATA_PTR pOID)
{
	int strLength;
	int numInts = 1;
	uint32 * Numbers;
	char* pCurr;
	char* pNext;
	sint16 size, i, j;
    uint32 value;
	//CSSM_DATA_PTR pOID;
	unsigned char* Octets;
	int cntr;

	if(pOidStr == NULL)
	{
		return 0;
	}

	// how many integers?
	strLength = strlen(pOidStr);
	
	for (cntr = 0; cntr < strLength; cntr++)
	{
		if(pOidStr[cntr] == '.')
		{
			numInts ++;
		}
	}

	if(numInts < 2)
	{
		return 0;
	}

	// allocate an array of ints
	Numbers = (uint32*)malloc( sizeof(uint32) * numInts);

	// convert the string to an array of ints
	pCurr = pOidStr;
	pNext = pOidStr;

	for( cntr = 0; cntr < numInts; cntr++)
	{
		while((*pNext != '\0') && (*pNext != '.'))
		{
			pNext++;
		}

		*pNext = '\0';
		Numbers[cntr] = atoi(pCurr);
		
		pNext++;
		pCurr = pNext;
	}

	// now we have an array of integers, 
	// package them as a BER num
	
	/* precompute the sizes */
    size = 1;   /* first two ints turns into one BER byte*/
    for (i = 2; i < numInts; i++) {
        for (j = 1; j < 5; j++)
            if ((1UL << (j * 7)) > Numbers[i]) break;
        size += j;
    }
    
	// package the whole thing as a CSSM data
	//pOID = (CSSM_DATA_PTR)malloc( sizeof(CSSM_DATA));
	pOID->Length = size;
	pOID->Data = (unsigned char *)malloc( size);
	Octets = pOID->Data;

	 *Octets++ = (uint8) (Numbers[0] * 40 + Numbers[1]);
    for (i = 2; i < numInts; i++) 
	{
        value = Numbers[i];
        for (j = 4; j > 0; j--) 
            if ((1UL << (j * 7)) <= value) break;
        if (j == 4) 
		{   /* special for MSFT C++ bug */
            *Octets++ = (uint8) (((value >> 28) & 15) | 128);
            j--;
        }
        while (j > 0) 
		{
            *Octets++ = (uint8) (((value >> (j * 7)) & 127) | 128);
            j--;
        }
        *Octets++ = (uint8) (value & 127);
    }
	
	free(Numbers);
	return 1;
}

