/*-----------------------------------------------------------------------
 * File: OBJECTREF.H
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// ObjectRef.h
//
// This file contains the class definition for the CObjectRef
//  class, a class which implements an object reference based on 
//  the CDSA 2.0 Signed Manifest Specification (Open Group standard)
//  and its definition of a "manifest section".
//
// Copyright(c) 1998, Intel Corporation.  All rights reserved.
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Class description:
//  Members:
//      mType(public)
//          type associated with the reference.  Mainly this is used 
//          to tell how the object is to be retrieved.
//      mName(public)
//          name to associate with the given object
//      mReferencePath(public)
//          path to be referenced by
//      mAttributes(public)
//          pointer to the lis of attributes associated with the object
//          reference
//  Methods:
//      none
/////////////////////////////////////////////////////////////////////

#ifndef OBJECTREF_H
#define OBJECTREF_H

#include "AttributeList.h"

enum REF_TYPE {RefNone=0, RefFile, RefURL, RefExecutable, RefSignature};

class DllExport CObjectRef  
{
public:
	// member vars
	int mType;
    char* mName;
	char* mReferencePath;
    CAttributeList *mAttributes;
	
	// constructors
    CObjectRef( char* Name, 
                REF_TYPE Type, 
                char* SourcePath,
                char* ReferencePath,
                CAttributeList *Attributes );
    CObjectRef( char* Name, 
                REF_TYPE Type, 
                char* ManifestPath);
	CObjectRef( const CObjectRef *object );
	CObjectRef();
	~CObjectRef();

	// member funcs
};

#endif // !def OBJECTREF_H
