/*-----------------------------------------------------------------------
 * File: SIGNATUREGRAPH.H
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// SignatureGraph.h
//
// This file contains the class definition for the CSignatureGraph
//  and CSignatureGraphNode classes. These classes implement a graph
//  of signers indexed by objects signed.
//
//////////////////////////////////////////////////////////////////////
#ifndef SIGNATUREGRAPH_H
#define SIGNATUREGRAPH_H

#include "ObjectRef.h"
#include "SignerList.h"

//////////////////////////////////////////////////////////////////////
// Class description: CSignatureGraphNode  
//  Members:
//      mObjectReference(public)      
//          decribes the object which this node refers to
//      mSigners(public)
//          list of signers for this node
//      mNext(public)
//          pointer to the next node in the graph
//  Methods:
//      none
/////////////////////////////////////////////////////////////////////
class DllExport CSignatureGraphNode  
{
public:
	// member vars
	CSignatureGraphNode *mNext;
	CSignerList *mSigners;
	CObjectRef *mObjectReference;
	
	// constructors
	CSignatureGraphNode(const CSignatureGraphNode *node);
	CSignatureGraphNode(CObjectRef* ObjectRef,
                        CSignerList* Signers);
	~CSignatureGraphNode();
    
    // public methods
    WF_RETURN AddSigner(CSignerListNode* SignerToAdd);
};


//////////////////////////////////////////////////////////////////////
// Class description: CSignatureGraph  
//  Members:
//      mHead(public)      
//          pointer to the head node at the top of the graph
//      mTail(public)      
//          pointer to the tail node at the top of the graph
//      mNumberOfNodes(public)
//          count of the number of nodes at the top of the graph
//  Methods:
//      AddNode(public)
//          appends a node to the end of the top level of the graph
/////////////////////////////////////////////////////////////////////
class DllExport CSignatureGraph  
{
public:
	// member vars
	CSignatureGraphNode *mHead;
	CSignatureGraphNode *mTail;
	int mNumberOfNodes;
	
	// constructors
	CSignatureGraph(const CSignatureGraph *graph);
	CSignatureGraph();
	~CSignatureGraph();

	// public methods
	WF_RETURN AddNode( CSignatureGraphNode *NodeToAdd );
};

#endif // !def SIGNATUREGRAPH_H
