/*-----------------------------------------------------------------------
 * File: MEMFUNCS.CPP
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// MemFuncs.cpp
//  memory functions for use with ISL 
///////////////////////////////////////////////////////////////////////////

#ifdef CSSM_MEMTRACK
#undef new
#include <crtdbg.h>
static char SZ_FILE_NAME[] = __FILE__;
#define new new(_NORMAL_BLOCK, SZ_FILE_NAME, __LINE__)  /*need space*/
#endif /* ifdef VMS comment this out - CSSM_MEMTRACK */

#include "cssmport.h"
#include "isltype.h"
#include <scMemTrack.h>

/*
**	a diagnostic memory allocator that checks for memory leaks or bad frees
*/
struct memlist {
	struct memlist *next;
	void *buffer;
};

/* methods */
ISL_SIZE myMemorySizeof()
{
	return sizeof(struct memlist *);
}

/* construct integrity memory */
ISL_STATUS myMemorySetup(void *context)
{
	struct memlist **list = (struct memlist **) context;
	*list = 0;
	return (ISL_STATUS)ISL_OK;
}
#ifdef CSSM_MEMTRACK
/* allocate and add a buffer to our list of allocated buffers */
void *myMalloc(ISL_MEMORY_CONTEXT *context, size_t size, const char * szFilename,
	uint32 u32LineNumber)
{
	struct memlist **list = (struct memlist**) context;
	struct memlist *newItem;
	newItem = (struct memlist *)_malloc_dbg(sizeof(struct memlist), 
						_NORMAL_BLOCK,szFilename,u32LineNumber);
	if (!newItem) return 0;
	newItem->buffer = _malloc_dbg(size, _NORMAL_BLOCK,szFilename,u32LineNumber);
	if (!newItem->buffer) return 0;
	newItem->next = *list;
	*list = newItem;
	return newItem->buffer;
}

/* make sure the buffer is on our list */
void myFree(ISL_MEMORY_CONTEXT *context, void *buffer)
{
	struct memlist **list = (struct memlist**) context;
	struct memlist *item, *last;
	for (last = 0, item = *list; ((item != NULL) && (!IsBadWritePtr( item, sizeof( memlist)) ) ); last = item, item = item->next) {
		if (item->buffer == buffer)	{
			if (last) {
				last->next = item->next;
			} else {
				*list = item->next;
			}
			_free_dbg(item, _NORMAL_BLOCK);
			_free_dbg(buffer,_NORMAL_BLOCK);
			return;
		}
	}
}
#else
/* allocate and add a buffer to our list of allocated buffers */
void *myMalloc(ISL_MEMORY_CONTEXT *context, size_t size)
{
	struct memlist **list = (struct memlist**) context;
	struct memlist *newItem;
	newItem = (struct memlist *)malloc(sizeof(struct memlist));
	if (!newItem) return 0;
	newItem->buffer = malloc(size);
	if (!newItem->buffer) return 0;
	newItem->next = *list;
	*list = newItem;
	return newItem->buffer;
}

/* make sure the buffer is on our list */
void myFree(ISL_MEMORY_CONTEXT *context, void *buffer)
{
	struct memlist **list = (struct memlist**) context;
	struct memlist *item, *last;
	for (last = 0, item = *list; ((item != NULL) && (!IsBadWritePtr( item, sizeof( memlist)) ) ); last = item, item = item->next) {
		if (item->buffer == buffer)	{
			if (last) {
				last->next = item->next;
			} else {
				*list = item->next;
			}
			free(item);
			free(buffer);
			return;
		}
	}
}

#endif
/*	Destroy memory allocation context and check for unreturned memory. Complain as
**	we recycle it.
*/
extern void myCheckForLeaks(ISL_MEMORY_CONTEXT *context)
{
	struct memlist **list = (struct memlist**) context;
	struct memlist *item, *last;
	for (item = *list; item; ) {
		last = item;
		item = item->next;
		free(last->buffer);
		free(last);
	}
	*list = 0;	/* just for cleanliness (or in case we get destroyed twice?) */
}

//memory functions used by ISL
ISL_MEMORY_METHODS myMemoryMethods = {myMemorySizeof, 
		                                     myMemorySetup,
                                             myMalloc,
                                             myFree,
                                             myCheckForLeaks };
